@@ -1167,7 +1167,7 @@ styles standards de TeXmacs")
 ("reset" "remettre")
 ("resize object" "redimensionner objet")
 ("resized" "redimensionn")
-("results of the search for" "resultats de la recherche pour")
+("search results for" "resultats de la recherche pour")
 ("return" "entre")
 ("revert" "ractualiser")
 ("revert buffer" "actualiser tampon")
