@@ -622,7 +622,7 @@
 ("keyboard" "teclado")
 ("keyboard equivalent" "equivalencia de teclado")
 ("keyboard modifiers" "modificadores de teclado")
-("keyboard shorthand" "atajo de teclado")
+("keyboard shortcut" "atajo de teclado")
 ("keymaps" "disposicin del teclado")
 ("keyword" "palabra clave")
 ("keywords" "palabras clave")
@@ -1381,7 +1381,7 @@
 ("remove fields" "quitar campos")
 ("remove input field" "quitar campo de entrada")
 ("remove input field above" "quitar campo de entrada arriba")
-("results of the search for" "resultados de la bsqueda de")
+("search results for" "resultados de la bsqueda de")
 ("right arrow" "flecha derecha")
 ("search words in my documents" "buscar palabras en mis documentos")
 ("search words in the documentation" "buscar palabras en la documentacin")
