@@ -35,7 +35,7 @@
   (:mode bib-plain?)
   (let* ((ff (if (bib-null? (list-ref x 1)) "" `(concat ,(list-ref x 1) (nbsp))))
 	 (vv (if (bib-null? (list-ref x 2)) "" `(concat ,(list-ref x 2) (nbsp))))
-	 (ll (if (bib-null? (list-ref x 3)) "" (list-ref x 3)))
+	 (ll (if (bib-null? (list-ref x 3)) "" (bib-purify (list-ref x 3))))
 	 (jj (if (bib-null? (list-ref x 4)) "" `(concat ", " ,(list-ref x 4)))))
     `(concat ,ff ,vv ,ll ,jj)))
 
@@ -55,9 +55,10 @@
 	    (bib-format-name (list-ref a 1))
 	    (let* ((b (bib-format-name (list-ref a 1)))
 		   (m (bib-format-names-rec 2 (- n 1) a))
-		   (e (if (equal? (list-ref (list-ref a (- n 1)) 4) "others")
+		   (e (if (or (== (list-ref (list-ref a (- n 1)) 3) "others")
+                              (== (list-ref (list-ref a (- n 1)) 4) "others"))
 			  `(concat " et" (nbsp) "al")
-			  `(concat ,(bib-translate " et ")
+			  `(concat ,(bib-translate " and ")
 				   ,(bib-format-name (list-ref a (- n 1)))))))
 	      `(concat ,b ,m ,e))))))
 
@@ -118,8 +119,9 @@
   (:mode bib-plain?)
   (with p (bib-field x "pages")
     (cond
-      ((equal? 1 (length p)) "")
-      ((equal? 2 (length p))
+      ((bib-null? p) "")
+      ((== (length p) 1) "")
+      ((== (length p) 2)
        `(concat ,(bib-translate "page ") ,(list-ref p 1)))
       (else
 	`(concat ,(bib-translate "pages ")
@@ -163,10 +165,11 @@
   (:mode bib-plain?)
   (let* ((t (bib-field x "type"))
 	 (n (bib-field x "number"))
-	 (type (if (bib-null? t) ,(bib-translate "Technical Report") t))
+	 (type (if (bib-null? t) (bib-translate "Technical Report") t))
 	 (number (if (bib-null? n) "" n))
 	 (sep (if (< (bib-text-length n) 3) `(nbsp) " ")))
-    `(concat ,type ,sep ,number)))
+    (if (bib-null? n) type
+        `(concat ,type ,sep ,number))))
 
 (tm-define (bib-format-bibitem n x)
   (:mode bib-plain?)
@@ -365,7 +368,7 @@
 	 ,(bib-new-block
 	   (bib-new-sentence
 	    `(,(if (bib-empty? x "type")
-		   ,(bib-translate "Master's thesis")
+		   (bib-translate "Master's thesis")
 		   (bib-format-field-Locase x "type"))
 	      ,(bib-format-field x "school")
 	      ,(bib-format-field x "address")
