@@ -167,8 +167,9 @@
 		       `(concat ,(tmhtml-force-string title)
 				" (FSF GNU project)"))
 		      (else (tmhtml-force-string title))))
-    (if (or (in? "tmdoc" styles) (in? "tmweb" styles) (in? "mmxdoc" styles))
-	(with ss (if (in? "mmxdoc" styles)
+    (if (or (in? "tmdoc" styles) (in? "tmweb" styles)
+            (in? "mmxdoc" styles) (in? "magix-web" styles))
+	(with ss (if (or (in? "mmxdoc" styles) (in? "magix-web" styles))
 		     "http://www.texmacs.org/css/mmxdoc.css"
 		     "http://www.texmacs.org/css/tmdoc.css")
 	  (set! css `(h:link (@ (rel "stylesheet")
@@ -683,7 +684,7 @@
 	   (string-append (number->htmlstring val) "px"))
 	  (css? len)
 	  (else (number->htmlstring (* cmpx val incm))))))
-
+ 
 (define (tmlength->px len)
   (and-let* ((tmlen (string->tmlength len))
 	     (dummy? (not (tmlength-null? tmlen)))
@@ -1037,6 +1038,7 @@
   (tmhtml-png (cons 'graphics l)))
 
 (define (tmhtml-image-name name)
+  ;; FIXME: we should replace ~, environment variables, etc.
   (with u (url-relative current-save-target (string->url name))
     (if (and (or (string-ends? name ".ps") (string-ends? name ".eps"))
 	     (url-exists? u))
