@@ -70,6 +70,7 @@
   (mathd "\\mathrm{d}")
   (mathe "\\mathrm{e}")
   (matheuler "\\gamma")
+  (mathlambda "\\lambda")
   (mathi "\\mathrm{i}")
   (mathpi "\\pi")
   (Alpha "\\mathrm{A}")
@@ -87,6 +88,7 @@
   (Zeta "\\mathrm{Z}")
 
   ;; other extra symbols
+  (exterior "\\wedge")
   (Exists "\\exists")
   (bigintwl "\\int")
   (bigointwl "\\oint")
@@ -104,6 +106,7 @@
   (nni "\\not\\ni")
   (notni "\\not\\ni")
   (nequiv "\\not\\equiv")
+  (nleadsto "\\not\\leadsto")
   (dotamalg "\\mathaccent95{\\amalg}")
   (dottimes "\\mathaccent95{\\times}")
   (dotoplus "\\mathaccent95{\\oplus}")
@@ -164,6 +167,8 @@
   (pari "{\\sc Pari}"))
 
 (drd-table latex-texmacs-1%
+  (tmrsub (ensuremath (!append "_{" (textrm 1) "}")))
+  (tmrsup (ensuremath (!append "^{" (textrm 1) "}")))
   (tmtextrm (!group (rmfamily) (!group 1)))
   (tmtextsf (!group (sffamily) (!group 1)))
   (tmtexttt (!group (ttfamily) (!group 1)))
