@@ -26,8 +26,8 @@
 ;; Global variables
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(define-public tmtex-style "generic")
-(define-public tmtex-packages '())
+(tm-define tmtex-style "generic")
+(tm-define tmtex-packages '())
 (define tmtex-env (make-ahash-table))
 (define tmtex-serial 0)
 (define tmtex-auto-produce 0)
@@ -436,7 +436,7 @@
   (let* ((doc (car l))
 	 (styles (cadr l))
 	 (lang (caddr l))
-	 (init (cadddr l))
+	 (init (or (cadddr l) '(collection)))
 	 (init-bis (map (lambda (x) (cons (cadr x) (caddr x))) (cdr init)))
 	 (doc-preamble (tmtex-filter-preamble doc))
 	 (doc-body-pre (tmtex-filter-body doc))
@@ -524,12 +524,13 @@
 (define (tmtex-next-line l) (list '!nextline))
 (define (tmtex-no-break l) '(!group (nobreak)))
 
-(define (tmtex-decode-length s)
+(define (tmtex-decode-length len)
   ;; FIXME: should be completed
-  (cond ((string-ends? s "fn") (string-replace s "fn" "em"))
-	((string-ends? s "spc") (string-replace s "spc" "em"))
-	((string-ends? s "par") (string-replace s "par" "\\columnwidth"))
-	(else s)))
+  (with s (force-string len)
+    (cond ((string-ends? s "fn") (string-replace s "fn" "em"))
+	  ((string-ends? s "spc") (string-replace s "spc" "em"))
+	  ((string-ends? s "par") (string-replace s "par" "\\columnwidth"))
+	  (else s))))
 
 (define (tmtex-hspace l)
   (let ((s (if (= (length l) 1) (car l) (cadr l))))
@@ -690,10 +691,14 @@
 	(list which r))))
 
 (define (tmtex-rsub l)
-  (tmtex-script '!sub (car l)))
+  (if (tmtex-math-mode?)
+      (tmtex-script '!sub (car l))
+      (list 'tmrsub (tmtex (car l)))))
 
 (define (tmtex-rsup l)
-  (tmtex-script '!sup (car l)))
+  (if (tmtex-math-mode?)
+      (tmtex-script '!sup (car l))
+      (list 'tmrsup (tmtex (car l)))))
 
 (define (tmtex-frac l)
   (tmtex-function 'frac l))
@@ -1042,14 +1047,36 @@
 	    (convert-to-file u fm "postscript-file" name-url)
 	    (list 'includegraphics name-string))))))
 
+(define (tmtex-image-length len)
+  (let* ((s (force-string len))
+	 (unit (and (tm-length? s) (tm-length-unit len))))
+    (cond ((== s "") "!")
+	  ((string-ends? s "%") "!")
+	  ((in? unit '("w" "h")) "!")
+	  (else (tmtex-decode-length len)))))
+
+(define (tmtex-image-mag len)
+  (let* ((s (force-string len))
+	 (val (and (tm-length? s) (tm-length-value len)))
+	 (unit (and (tm-length? s) (tm-length-unit len))))
+    (cond ((== s "") 0.0)
+	  ((string-ends? s "%")
+	   (with x (string->number (string-drop-right s 1))
+	     (if x (/ x 100.0) 0)))
+	  ((in? unit '("w" "h")) (or val 0))
+	  (else #f))))
+
 (define (tmtex-image l)
   (let* ((fig (tmtex-as-eps (force-string (car l))))
-	 (hor (if (== (cadr l) "") "!" (tmtex-decode-length (cadr l))))
-	 (ver (if (== (caddr l) "") "!" (tmtex-decode-length (caddr l)))))
-    (if (or (string-starts? hor "*") (string-starts? hor "/")) (set! hor "!"))
-    (if (or (string-starts? ver "*") (string-starts? ver "/")) (set! ver "!"))
-    (if (and (== hor "!") (== ver "!")) fig
-	(list 'resizebox hor ver fig))))
+	 (hor (tmtex-image-length (cadr l)))
+	 (ver (tmtex-image-length (caddr l)))
+	 (mhor (tmtex-image-mag (cadr l)))
+	 (mver (tmtex-image-mag (caddr l))))
+    (cond ((or (not mhor) (not mver)) (list 'resizebox hor ver fig))
+	  ((and (== mhor 0.0) (== mver 0.0)) fig)
+	  ((or (== mhor 1.0) (== mver 1.0)) fig)
+	  ((== mhor 0.0) (list 'scalebox (number->string mver) fig))
+	  (else (list 'scalebox (number->string mhor) fig)))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Titles of documents
@@ -1215,6 +1242,54 @@
 (define (tmtex-math s l)
   (tmtex `(with "mode" "math" ,(car l))))
 
+(define (tmtex-textual x)
+  (tmtex-env-set "mode" "text")
+  (with r (tmtex x)
+    (tmtex-env-reset "mode")
+    r))
+
+(define (tmtex-math-up s l)
+  (list 'mathup (tmtex-textual (car l))))
+
+(define (tmtex-math-ss s l)
+  (list 'mathsf (tmtex-textual (car l))))
+
+(define (tmtex-math-tt s l)
+  (list 'mathtt (tmtex-textual (car l))))
+
+(define (tmtex-math-bf s l)
+  (list 'mathbf (tmtex-textual (car l))))
+
+(define (tmtex-math-sl s l)
+  (list 'mathsl (tmtex-textual (car l))))
+
+(define (tmtex-math-it s l)
+  (list 'mathit (tmtex-textual (car l))))
+
+(define (tmtex-mathord s l)
+  (list 'mathord (tmtex (car l))))
+
+(define (tmtex-mathbin s l)
+  (list 'mathbin (tmtex (car l))))
+
+(define (tmtex-mathrel s l)
+  (list 'mathrel (tmtex (car l))))
+
+(define (tmtex-mathopen s l)
+  (list 'mathopen (tmtex (car l))))
+
+(define (tmtex-mathclose s l)
+  (list 'mathclose (tmtex (car l))))
+
+(define (tmtex-mathpunct s l)
+  (list 'mathpunct (tmtex (car l))))
+
+(define (tmtex-mathop s l)
+  (list 'mathop (tmtex (car l))))
+
+(define (tmtex-syntax s l)
+  (tmtex (car l)))
+
 (define (tmtex-dummy s l)
   "")
 
@@ -1527,6 +1602,7 @@
   ((:or unquote* copy
 	if if* case while for-each
 	extern include use-package) tmtex-noop)
+  (syntax tmtex-syntax)
 
   ((:or or xor and not plus minus times over div mod
 	merge length range number date translate change-case find-file
@@ -1611,9 +1687,37 @@
   (really-huge (,tmtex-Huge 1))
 
   (math (,tmtex-math 1))
+  (math-up (,tmtex-math-up 1))
+  (math-ss (,tmtex-math-ss 1))
+  (math-tt (,tmtex-math-tt 1))
+  (math-bf (,tmtex-math-bf 1))
+  (math-sl (,tmtex-math-sl 1))
+  (math-it (,tmtex-math-it 1))
+  (math-separator (,tmtex-mathpunct 1))
+  (math-quantifier (,tmtex-mathord 1))
+  (math-imply (,tmtex-mathbin 1))
+  (math-or (,tmtex-mathbin 1))
+  (math-and (,tmtex-mathbin 1))
+  (math-not (,tmtex-mathord 1))
+  (math-relation (,tmtex-mathrel 1))
+  (math-union (,tmtex-mathbin 1))
+  (math-intersection (,tmtex-mathbin 1))
+  (math-exclude (,tmtex-mathbin 1))
+  (math-plus (,tmtex-mathbin 1))
+  (math-minus (,tmtex-mathbin 1))
+  (math-times (,tmtex-mathbin 1))
+  (math-over (,tmtex-mathbin 1))
+  (math-big (,tmtex-mathop 1))
+  (math-prefix (,tmtex-mathord 1))
+  (math-postfix (,tmtex-mathord 1))
+  (math-open (,tmtex-mathopen 1))
+  (math-close (,tmtex-mathclose 1))
+  (math-ordinary (,tmtex-mathord 1))
+  (math-ignore (,tmtex-mathord 1))
   ((:or equation equation*) (,tmtex-equation-wrapper 1))
   ((:or eqnarray eqnarray* leqnarray*) (,tmtex-eqnarray 1))
   (eq-number (,tmtex-default -1))
+
   (the-index (,tmtex-dummy -1))
   (glossary (,tmtex-glossary 1))
   (glossary-explain (,tmtex-glossary 2))
