@@ -5,10 +5,9 @@
 ;; DESCRIPTION : setup texmacs converters
 ;; COPYRIGHT   : (C) 2003  Joris van der Hoeven
 ;;
-;; This software falls under the GNU general public license and comes WITHOUT
-;; ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for details.
-;; If you don't have this file, write to the Free Software Foundation, Inc.,
-;; 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+;; This software falls under the GNU general public license version 3 or later.
+;; It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+;; in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ;;
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
@@ -52,25 +51,25 @@
 ;; Scheme format for TeXmacs (no information loss)
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(define (scheme-recognizes? s)
+(define (stm-recognizes? s)
   (and (string? s) (string-starts? s "(document (TeXmacs")))
 
-(define-format scheme
+(define-format stm
   (:name "Scheme")
-  (:suffix "scm")
-  (:must-recognize scheme-recognizes?))
+  (:suffix "stm" "scm")
+  (:must-recognize stm-recognizes?))
 
-(converter texmacs-tree scheme-document
-  (:function texmacs->scheme))
+(converter texmacs-tree stm-document
+  (:function texmacs->stm))
 
-(converter scheme-document texmacs-tree
-  (:function scheme->texmacs))
+(converter stm-document texmacs-tree
+  (:function stm->texmacs))
 
-(converter texmacs-tree scheme-snippet
-  (:function texmacs->scheme))
+(converter texmacs-tree stm-snippet
+  (:function texmacs->stm))
 
-(converter scheme-snippet texmacs-tree
-  (:function scheme-snippet->texmacs))
+(converter stm-snippet texmacs-tree
+  (:function stm-snippet->texmacs))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Verbatim
@@ -79,19 +78,28 @@
 (tm-define (texmacs->verbatim x . opts)
   (if (list-1? opts) (set! opts (car opts)))
   (let* ((wrap? (== (assoc-ref opts "texmacs->verbatim:wrap") "on"))
-	 (enc (or (assoc-ref opts "texmacs->verbatim:encoding") "iso-8859-1")))
+	 (enc (or (assoc-ref opts "texmacs->verbatim:encoding") "utf-8")))
+    (cpp-texmacs->verbatim x wrap? enc)))
+
+(tm-define (texmacs->verbatim-snippet x . opts)
+  (if (list-1? opts) (set! opts (car opts)))
+  (let* ((wrap? (== (assoc-ref opts "texmacs->verbatim:wrap") "on"))
+	 (enc (or (assoc-ref opts "texmacs->verbatim:encoding") "utf-8")))
+    ;; FIXME: dirty hack for "copy to verbatim" of code snippets
+    (if (or (== (get-env "mode") "prog") (== (get-env "font-family") "tt"))
+        (set! wrap? #f))
     (cpp-texmacs->verbatim x wrap? enc)))
 
 (tm-define (verbatim->texmacs x . opts)
   (if (list-1? opts) (set! opts (car opts)))
   (let* ((wrap? (== (assoc-ref opts "verbatim->texmacs:wrap") "on"))
-	 (enc (or (assoc-ref opts "verbatim->texmacs:encoding") "iso-8859-1")))
+	 (enc (or (assoc-ref opts "verbatim->texmacs:encoding") "utf-8")))
     (cpp-verbatim->texmacs x wrap? enc)))
 
 (tm-define (verbatim-snippet->texmacs x . opts)
   (if (list-1? opts) (set! opts (car opts)))
   (let* ((wrap? (== (assoc-ref opts "verbatim->texmacs:wrap") "on"))
-	 (enc (or (assoc-ref opts "verbatim->texmacs:encoding") "iso-8859-1")))
+	 (enc (or (assoc-ref opts "verbatim->texmacs:encoding") "utf-8")))
     (cpp-verbatim-snippet->texmacs x wrap? enc)))
 
 (define-format verbatim
@@ -100,12 +108,12 @@
 
 (converter verbatim-document texmacs-tree
   (:function-with-options verbatim->texmacs)
-  (:option "verbatim->texmacs:wrap" "off")
+  (:option "verbatim->texmacs:wrap" "on")
   (:option "verbatim->texmacs:encoding" "utf-8"))
 
 (converter verbatim-snippet texmacs-tree
   (:function-with-options verbatim-snippet->texmacs)
-  (:option "verbatim->texmacs:wrap" "off")
+  (:option "verbatim->texmacs:wrap" "on")
   (:option "verbatim->texmacs:encoding" "utf-8"))
 
 (converter texmacs-tree verbatim-document
@@ -114,6 +122,6 @@
   (:option "texmacs->verbatim:encoding" "utf-8"))
 
 (converter texmacs-tree verbatim-snippet
-  (:function-with-options texmacs->verbatim)
+  (:function-with-options texmacs->verbatim-snippet)
   (:option "texmacs->verbatim:wrap" "off")
   (:option "texmacs->verbatim:encoding" "utf-8"))
