@@ -137,7 +137,7 @@
   ;; (cork-grave->backquote (utf8->cork (url-decode s)))
   ;; NOTE: don't decode URL names, or also implement a corresponding
   ;; routine for the encoding, when we click on a hyperlink
-  (string-encode s))
+  (string->tmstring s))
 
 ;; Conversion of Cork GRAVE ACCENT to LEFT SINGLE QUOTATION MARK
 
@@ -541,6 +541,5 @@
   ;; Collapse whitespaces.
   ;; TODO: consolidate with htmltm-serial
   (with c (apply tmconcat l)
-    (if (func? c 'concat)
-	(stm-concat (cdr c) htmltm-make-concat)
-	c)))
+    (if (not (func? c 'concat)) c
+	(stm-concat (cdr c) htmltm-make-concat))))
