@@ -6,10 +6,9 @@
 ;; COPYRIGHT   : (C) 2001  Joris van der Hoeven
 ;;               (C) 2004, 2005, 2006  Joris van der Hoeven and Henri Lesourd
 ;;
-;; This software falls under the GNU general public license and comes WITHOUT
-;; ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for details.
-;; If you don't have this file, write to the Free Software Foundation, Inc.,
-;; 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+;; This software falls under the GNU general public license version 3 or later.
+;; It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+;; in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ;;
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
@@ -279,7 +278,7 @@
                  `(point ,current-x ,current-y)
 		  moveclick-tolerance))
 	    (begin
-	       (undo)
+	       (undo 0)
 	       (set! choosing #f)
 	       (set! leftclick-waiting #f)
 	       (set! just-started-dragging #f))
@@ -388,16 +387,17 @@
       (if current-obj
 	  (begin
 	    (if (not just-started-dragging)
-	    (begin
-	       (edit_tab-key 'edit
-			     (if (graphics-states-void?)
-				 #f
-				 choosing))))
+                (begin
+                  (edit_tab-key 'edit
+                                (if (graphics-states-void?)
+                                    #f
+                                    choosing)
+                                1)))
 	    (if (or (not just-started-dragging) (not choosing))
-	    (begin
-	       (if (not (graphics-states-void?))
-		   (graphics-pop-state))
-	       (graphics-store-state #f)))
+                (begin
+                  (if (not (graphics-states-void?))
+                      (graphics-pop-state))
+                  (graphics-store-state #f)))
 	    (set! choosing #t))
 	  (edit-insert x y))))
 
@@ -440,7 +440,7 @@
   (:require (eq? mode 'edit))
   (display* "Right button(edit) currently unused\n"))
 
-(tm-define (edit_tab-key mode next)
+(tm-define (edit_tab-key mode next inc)
   (:require (eq? mode 'edit))
   (:state graphics-state)
  ;(display* "\nGraphics] Edit(Tab)[" next "," current-path "]=" current-obj "\n")
@@ -449,7 +449,7 @@
 	(if current-obj
 	    (graphics-decorations-update))
 	(if next
-	    (select-next))
+	    (select-next inc))
       )
       (invalidate-graphical-object)))
 
@@ -852,14 +852,14 @@
 ; FIXME : test due to timing problems in detecting the drag
       (graphics-last-point x y)))
 
-(tm-define (graphics-choose-point)
+(tm-define (graphics-choose-point inc)
   (:state graphics-state)
   ;(display* "Graphics] Choose\n")
   (if (not (graphics-states-void?))
       (graphics-pop-state))
   (set! choosing #t)
   (graphics-store-state #f)
-  (edit_tab-key (car (graphics-mode)) #t))
+  (edit_tab-key (car (graphics-mode)) #t inc))
 
 (tm-define (graphics-enter-mode old-mode new-mode)
   (:state graphics-state)
@@ -872,7 +872,7 @@
 	   (graphics-group-mode? new-mode)
       )
       (begin
-	 (if sticky-point (undo))
+	 (if sticky-point (undo 0))
 	 (sketch-reset)
 	 (graphics-decorations-reset))))
 
