@@ -6,10 +6,9 @@
 ;; COPYRIGHT   : (C) 2001  Joris van der Hoeven
 ;;               (C) 2004-2007  Joris van der Hoeven and Henri Lesourd
 ;;
-;; This software falls under the GNU general public license and comes WITHOUT
-;; ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for details.
-;; If you don't have this file, write to the Free Software Foundation, Inc.,
-;; 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+;; This software falls under the GNU general public license version 3 or later.
+;; It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+;; in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ;;
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
@@ -173,8 +172,8 @@
 	     graphical-textat-valign)))
 	((list? mode)
 	 (if (== prop "magnification")
-		 (graphics-eval-magnification-at (rcons mode 0))
-		 (graphics-path-property mode prop)))
+	     (graphics-eval-magnification-at (rcons mode 0))
+	     (graphics-path-property mode prop)))
 	((== mode 'new)
 	 (graphics-get-property (string-append "gr-" prop)))
 	(else ;(== mode 'default)
@@ -398,7 +397,7 @@
 				     (if edge
 				        (with l2 (list-head l 2)
 				        (if sticky-point
-					   `(,(list* 'with
+					   `(,(cons* 'with
 					         "point-style"
 					         "square"
 					        `((concat .
@@ -525,7 +524,7 @@
 	)
 	(set! res (append res
 			  (if props
-			     `(,(append props `(,(list* 'concat t))))
+			     `(,(append props `(,(cons* 'concat t))))
 			      t)))))
   )
   res)
