@@ -43,7 +43,11 @@
 	  (list 'begin defn arch1 arch2 drd-cmd)))))
 
 (define-public-macro (texmacs-modes . l)
-  `(begin ,@(map texmacs-mode l)))
+  `(begin
+     (set! temp-module ,(current-module))
+     (set-current-module texmacs-user)
+     ,@(map texmacs-mode l)
+     (set-current-module temp-module)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Checking modes
@@ -116,6 +120,7 @@
   (in-mmxdoc% (style-has? "mmxdoc-style") in-tmdoc%)
   (in-plugin-with-converters%
    (plugin-supports-math-input-ref (get-env "prog-language")))
+  (in-screens% (inside? 'screens))
   (with-any-selection% (selection-active-any?))
   (with-active-selection% (selection-active-normal?))
   (in-scheme% (== (get-env "prog-language") "scheme"))
@@ -157,6 +162,9 @@
 ;; Keyboard related
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
+(define-public remote-control-flag? #f)
+(define-public remote-control-remap (make-ahash-table))
+
 (define (cyrillic-input-method? what)
   (== (get-preference "cyrillic input method") what))
 
@@ -171,14 +179,52 @@
   (detailed-menus% (== (get-preference "detailed menus") "detailed"))
   (with-debugging-tool% (== (get-preference "debugging tool") "on"))
   (with-linking-tool% (== (get-preference "linking tool") "on"))
+  (with-source-tool% (== (get-preference "source tool") "on"))
   (with-versioning-tool% (== (get-preference "versioning tool") "on"))
   (with-remote-connections% (== (get-preference "remote connections") "on"))
   (search-mode% (== (get-input-mode) 1))
   (replace-mode% (== (get-input-mode) 2))
   (spell-mode% (== (get-input-mode) 3))
   (complete-mode% (== (get-input-mode) 4))
+  (remote-control-mode% (== remote-control-flag? #t))
   (in-cyrillic-cp1251% (cyrillic-input-method? "cp1251") in-cyrillic%)
   (in-cyrillic-jcuken% (cyrillic-input-method? "jcuken") in-cyrillic%)
   (in-cyrillic-koi8% (cyrillic-input-method? "koi8") in-cyrillic%)
   (in-cyrillic-translit% (cyrillic-input-method? "translit") in-cyrillic%)
   (in-cyrillic-yawerty% (cyrillic-input-method? "yawerty") in-cyrillic%))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; Lazy initializations
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(define-public lazy-initialize-id 0)
+(define-public lazy-initialize-pending '())
+
+(define-public (lazy-initialize-impl id pred? module)
+  (set! lazy-initialize-pending
+	(cons (list id pred? module) lazy-initialize-pending)))
+
+(define-public (lazy-initialize-do l id)
+  (cond ((null? l) l)
+	((or (== (caar l) id) (and (== id #t) ((cadar l))))
+	 ((caddar l))
+	 (lazy-initialize-do (cdr l) id))
+	(else (cons (car l) (lazy-initialize-do (cdr l) id)))))
+
+(define-public-macro (lazy-initialize module pred?)
+  `(with id lazy-initialize-id
+     (set! lazy-initialize-id (+ id 1))
+     (lazy-initialize-impl id
+       (lambda ()
+	 ,pred?)
+       (lambda ()
+	 (import-from ,module)))
+     (delayed
+       (:idle 5000)
+       (set! lazy-initialize-pending
+	     (lazy-initialize-do lazy-initialize-pending id))
+       (import-from ,module))))
+
+(define-public (lazy-initialize-force)
+  (set! lazy-initialize-pending
+	(lazy-initialize-do lazy-initialize-pending #t)))
