@@ -37,7 +37,7 @@
 
   (define Quantifier-symbol
     (:type prefix)
-    "<forall>" "<exists>" "<nexists>")
+    "<forall>" "<exists>" "<nexists>" "<Exists>" "<mathlambda>")
 
   (define Imply-nolim-symbol
     (:type infix)
@@ -83,13 +83,14 @@
 
     "=" "<ne>" "<neq>" "<longequal>" "<less>" "<gtr>" "<le>" "<leq>"
     "<prec>" "<preceq>" "<ll>" "<lleq>" "<subset>" "<subseteq>"
-    "<sqsubset>" "<sqsubseteq>" "<in>" "<ge>" "<geq>" "<succ>" "<succeq>"
-    "<gg>" "<ggeq>" "<supset>" "<supseteq>" "<sqsupset>" "<sqsupseteq>" "<ni>"
+    "<sqsubset>" "<sqsubseteq>" "<in>" "<ni>" "<of>"
+    "<ge>" "<geq>" "<succ>" "<succeq>"
+    "<gg>" "<ggeq>" "<supset>" "<supseteq>" "<sqsupset>" "<sqsupseteq>"
     "<equiv>" "<nequiv>" "<sim>" "<simeq>" "<asymp>" "<approx>" "<cong>"
     "<subsetsim>" "<supsetsim>" "<doteq>" "<propto>" "<varpropto>"
     "<perp>" "<bowtie>" "<Join>" "<smile>" "<frown>" "<signchange>"
-    "<mid>" "<parallel>" "<shortmid>" "<shortparallel>" "<nmid>"
-    "<nparallel>" "<nshortmid>" "<nshortparallel>" "<divides>" "<ndivides>"
+    "<parallel>" "<shortparallel>" "<nparallel>" "<nshortparallel>"
+    "<shortmid>" "<nshortmid>" "<nmid>" "<divides>" "<ndivides>"
 
     "<approxeq>" "<backsim>" "<backsimeq>" "<Bumpeq>" "<bumpeq>" "<circeq>"
     "<curlyeqprec>" "<curlyeqsucc>" "<Doteq>" "<doteqdot>" "<eqcirc>"
@@ -138,7 +139,7 @@
     "<equallim>" "<longequallim>")
 
   (define Relation-symbol
-    Relation-nolim-symbol Relaton-symbol)
+    Relation-nolim-symbol Relation-lim-symbol)
 
   (define Arrow-nolim-symbol
     (:type infix)
@@ -152,7 +153,7 @@
     "<longmapsto>" "<longmapsfrom>"
     "<longhookleftarrow>" "<longhookrightarrow>" "<leftharpoonup>"
     "<leftharpoondown>" "<rightleftharpoons>" "<rightharpoonup>"
-    "<rightharpoondown>" "<leadsto>"
+    "<rightharpoondown>" "<leadsto>" "<nleadsto>"
     "<nearrow>" "<searrow>" "<swarrow>" "<nwarrow>"
     "<longtwoheadleftarrow>" "<longtwoheadrightarrow>"
     "<leftprec>" "<leftpreceq>" "<succright>" "<succeqright>"
@@ -227,7 +228,7 @@
     (:type infix)
     (:penalty 30)
     (:spacing default default)
-    "-" "<pm>" "<mp>" "<ominus>" "<boxminus>")
+    "-" "<pm>" "<mp>" "<minus>" "<ominus>" "<boxminus>" "<longminus>")
 
   (define Minus-prefix-symbol
     (:type prefix)
@@ -241,7 +242,7 @@
     (:spacing default default)
     "<cdot>" "<times>" "<otimes>" "<circ>" "<boxdot>" "<boxtimes>"
     "<dottimes>" "<dototimes>" "<ltimes>" "<rtimes>" "<atimes>" "<btimes>"
-    "<join>" "<ast>" "<star>" "<oast>")
+    "<exterior>" "<join>" "<ast>" "<star>" "<oast>" "<asterisk>")
 
   (define Times-invisible-symbol
     (:type infix)
@@ -301,23 +302,23 @@
     (:type prefix)
     (:penalty invalid)
     (:spacing none none)
-    "<card>")
+    "#" "<card>")
 
   (define Other-postfix-symbol
     (:type postfix)
     (:penalty panic)
-    "!")
+    "!" "%" "<permil>" "<postup>" "<postdown>")
 
   (define Prime-symbol
     (:type symbol)
     (:penalty panic)
-    "'" "`" "<dag>" "<ddag>")
+    "'" "`" "<dag>" "<ddag>" "<ast>")
 
   (define Ponctuation-visible-symbol
     (:type separator)
     (:penalty 0)
     (:spacing none default)
-    "," ";" ":" "<point>")
+    "," ";" ":" "<point>" "<mid>")
 
   (define Ponctuation-invisible-symbol
     (:type separator)
@@ -369,37 +370,37 @@
   (define Letter-symbol
     (:type symbol)
 
-    "<mathd>" "<mathe>" "<mathi>" "<mathpi>"
-    "<mathD>" "<matheuler>" "<mathcatalan>"
+    "<mathe>" "<mathi>" "<mathpi>"
+    "<matheuler>" "<mathcatalan>" "<mathGamma>"
     "<aleph>" "<beth>" "<gimel>" "<daleth>"
-    "<hbar>" "<hslash>" "<imath>" "<jmath>" "<ell>"
-    "<wp>" "<digamma>"
+    "<hbar>" "<hslash>" "<imath>" "<jmath>"
+    "<ell>" "<b-ell>" "<wp>" "<digamma>"
 
     "<alpha>" "<beta>" "<gamma>" "<delta>" "<epsilon>"
     "<varepsilon>" "<zeta>" "<eta>" "<theta>" "<vartheta>"
     "<iota>" "<kappa>" "<varkappa>" "<lambda>" "<mu>" "<nu>"
-    "<xi>" "<omikron>" "<pi>" "<varpi>" "<rho>" "<varrho>"
+    "<xi>" "<omicron>" "<pi>" "<varpi>" "<rho>" "<varrho>"
     "<sigma>" "<varsigma>" "<tau>" "<upsilon>"
     "<phi>" "<varphi>" "<psi>" "<chi>" "<omega>"
     "<backepsilon>" "<mho>"
 
     "<Alpha>" "<Beta>" "<Gamma>" "<Delta>" "<Epsilon>" "<Zeta>"
     "<Eta>" "<Theta>" "<Iota>" "<Kappa>" "<Lambda>" "<Mu>" "<Nu>"
-    "<Xi>" "<Omikron>" "<Pi>" "<Rho>" "<Sigma>" "<Tau>"
+    "<Xi>" "<Omicron>" "<Pi>" "<Rho>" "<Sigma>" "<Tau>"
     "<Upsilon>" "<Phi>" "<Psi>" "<Chi>" "<Omega>"
     "<Backepsilon>" "<Mho>"
 
     "<b-alpha>" "<b-beta>" "<b-gamma>" "<b-delta>" "<b-epsilon>"
     "<b-varepsilon>" "<b-zeta>" "<b-eta>" "<b-theta>" "<b-vartheta>"
     "<b-iota>" "<b-kappa>" "<b-varkappa>" "<b-lambda>" "<b-mu>" "<b-nu>"
-    "<b-xi>" "<b-omikron>" "<b-pi>" "<b-varpi>" "<b-rho>" "<b-varrho>"
+    "<b-xi>" "<b-omicron>" "<b-pi>" "<b-varpi>" "<b-rho>" "<b-varrho>"
     "<b-sigma>" "<b-varsigma>" "<b-tau>" "<b-upsilon>"
     "<b-phi>" "<b-varphi>" "<b-psi>" "<b-chi>" "<b-omega>"
     "<b-backepsilon>" "<b-mho>"
 
     "<b-Alpha>" "<b-Beta>" "<b-Gamma>" "<b-Delta>" "<b-Epsilon>"
     "<b-Zeta>" "<b-Eta>" "<b-Theta>" "<b-Iota>" "<b-Kappa>"
-    "<b-Lambda>" "<b-Mu>" "<b-Nu>" "<b-Xi>" "<b-Omikron>"
+    "<b-Lambda>" "<b-Mu>" "<b-Nu>" "<b-Xi>" "<b-Omicron>"
     "<b-Pi>" "<b-Rho>" "<b-Sigma>" "<b-Tau>" "<b-Upsilon>"
     "<b-Phi>" "<b-Psi>" "<b-Chi>" "<b-Omega>"
     "<b-Backepsilon>" "<b-Mho>"
@@ -413,11 +414,25 @@
     "<b-H>" "<b-I>" "<b-J>" "<b-K>" "<b-L>" "<b-M>" "<b-N>"
     "<b-O>" "<b-P>" "<b-Q>" "<b-R>" "<b-S>" "<b-T>" "<b-U>"
     "<b-V>" "<b-W>" "<b-X>" "<b-Y>" "<b-Z>"
+
+    "<b-0>" "<b-1>" "<b-2>" "<b-3>" "<b-4>"
+    "<b-5>" "<b-6>" "<b-7>" "<b-8>" "<b-9>"
+    
+    "<cal-a>" "<cal-b>" "<cal-c>" "<cal-d>" "<cal-e>" "<cal-f>" "<cal-g>"
+    "<cal-h>" "<cal-i>" "<cal-j>" "<cal-k>" "<cal-l>" "<cal-m>" "<cal-n>"
+    "<cal-o>" "<cal-p>" "<cal-q>" "<cal-r>" "<cal-s>" "<cal-t>" "<cal-u>"
+    "<cal-v>" "<cal-w>" "<cal-x>" "<cal-y>" "<cal-z>"
     
     "<cal-A>" "<cal-B>" "<cal-C>" "<cal-D>" "<cal-E>" "<cal-F>" "<cal-G>"
     "<cal-H>" "<cal-I>" "<cal-J>" "<cal-K>" "<cal-L>" "<cal-M>" "<cal-N>"
     "<cal-O>" "<cal-P>" "<cal-Q>" "<cal-R>" "<cal-S>" "<cal-T>" "<cal-U>"
     "<cal-V>" "<cal-W>" "<cal-X>" "<cal-Y>" "<cal-Z>"
+    
+    "<b-cal-a>" "<b-cal-b>" "<b-cal-c>" "<b-cal-d>" "<b-cal-e>" "<b-cal-f>"
+    "<b-cal-g>" "<b-cal-h>" "<b-cal-i>" "<b-cal-j>" "<b-cal-k>" "<b-cal-l>"
+    "<b-cal-m>" "<b-cal-n>" "<b-cal-o>" "<b-cal-p>" "<b-cal-q>" "<b-cal-r>"
+    "<b-cal-s>" "<b-cal-t>" "<b-cal-u>" "<b-cal-v>" "<b-cal-w>" "<b-cal-x>"
+    "<b-cal-y>" "<b-cal-z>"
 
     "<b-cal-A>" "<b-cal-B>" "<b-cal-C>" "<b-cal-D>" "<b-cal-E>" "<b-cal-F>"
     "<b-cal-G>" "<b-cal-H>" "<b-cal-I>" "<b-cal-J>" "<b-cal-K>" "<b-cal-L>"
@@ -437,10 +452,18 @@
     "<frak-S>" "<frak-T>" "<frak-U>" "<frak-V>" "<frak-W>" "<frak-X>"
     "<frak-Y>" "<frak-Z>"
 
+    "<bbb-a>" "<bbb-b>" "<bbb-c>" "<bbb-d>" "<bbb-e>" "<bbb-f>" "<bbb-g>"
+    "<bbb-h>" "<bbb-i>" "<bbb-j>" "<bbb-k>" "<bbb-l>" "<bbb-m>" "<bbb-n>"
+    "<bbb-o>" "<bbb-p>" "<bbb-q>" "<bbb-r>" "<bbb-s>" "<bbb-t>" "<bbb-u>"
+    "<bbb-v>" "<bbb-w>" "<bbb-x>" "<bbb-y>" "<bbb-z>"
+
     "<bbb-A>" "<bbb-B>" "<bbb-C>" "<bbb-D>" "<bbb-E>" "<bbb-F>" "<bbb-G>"
     "<bbb-H>" "<bbb-I>" "<bbb-J>" "<bbb-K>" "<bbb-L>" "<bbb-M>" "<bbb-N>"
     "<bbb-O>" "<bbb-P>" "<bbb-Q>" "<bbb-R>" "<bbb-S>" "<bbb-T>" "<bbb-U>"
-    "<bbb-V>" "<bbb-W>" "<bbb-X>" "<bbb-Y>" "<bbb-Z>")
+    "<bbb-V>" "<bbb-W>" "<bbb-X>" "<bbb-Y>" "<bbb-Z>"
+
+    "<bbb-0>" "<bbb-1>" "<bbb-2>" "<bbb-3>" "<bbb-4>"
+    "<bbb-5>" "<bbb-6>" "<bbb-7>" "<bbb-8>" "<bbb-9>")
   
   (define Miscellaneous-symbol
     (:type symbol)
@@ -499,7 +522,9 @@
     (:type unary)
     (:penalty panic)
     (:spacing none none)
-    "<partial>" "<nabla>" "<Re>" "<Im>" "<complement>")
+    "<mathd>" "<mathD>" "<mathLaplace>" "<partial>" "<nabla>"
+    "<Re>" "<Im>" "<complement>"
+    "<sum>" "<prod>" "<int>" "<oint>")
 
   (define Unary-operator-textual-symbol
     (:type unary)
@@ -516,7 +541,7 @@
   (define Binary-operator-symbol
     (:type binary)
     (:penalty panic)
-    (:spacing default default)
+    (:spacing none default)
     "div" "mod")
 
   (define N-ary-operator-symbol
@@ -527,6 +552,7 @@
     "inf" "lim" "liminf" "limsup" "max" "min" "sup")
 
   (define Prefix-symbol
+    Not-symbol
     Minus-prefix-symbol
     Plus-prefix-symbol
     Other-prefix-symbol)
@@ -552,13 +578,7 @@
     Power-symbol)
 
   (define Reserved-symbol
-    :<frac :<sqrt :<wide
+    :<frac :<sqrt :<wide :<table :<row
     :<around :<around* :<big-around :<left :<mid :<right :<big
     :<lsub :<lsup :<rsub :<rsup :<lprime :<rprime
-    :<Prefix :<Postfix)
-
-  (define Skip-symbol
-    :<space :<hspace :<vspace :<vspace* :<htab
-    :<with-limits :<line-break :<new-line :<next-line :<no-break
-    :<no-indent :<yes-indent :<no-indent* :<yes-indent*
-    :<label :<eq-number :<text :<assign :<hidden))
+    :<Prefix :<Postfix))
