@@ -22,6 +22,7 @@
   (:mode in-math?)
   ;; must come first in order not to screw the menus up
   ("accent:deadhat" (make-script #t #t))
+  ("accent:deadhat var" "^")
   ("math:left accent:deadhat" (make-script #t #f))
   ("math accent:tilde" (make-wide "~"))          ; yields M-M-~
   ("math accent:deadhat" (make-wide "^"))        ; yields M-M-^
@@ -41,6 +42,11 @@
   ("math:under accent:check" (make-wide-under "<check>"))
   ;; end
 
+  ("altcmd F5" (make 'math-it))
+  ("altcmd F6" (make 'math-bf))
+  ("altcmd F7" (make 'math-tt))
+  ("altcmd F8" (make 'math-ss))
+
   ("$" (math-make-math))
   ("math $" (make-with "mode" "text"))
   ("`" (make-lprime "`"))
@@ -50,19 +56,45 @@
   ("math:greek +" (make-rprime "<kreuz>"))
   ("\"" (make-rprime "'") (make-rprime "'"))
   ("_" (make-script #f #t))
+  ("_ var" "_")
   ("^" (make-script #t #t))
+  ("^ var" "^")
   ("math:left _" (make-script #f #f))
   ("math:left ^" (make-script #t #f))
   ("math f" (make-fraction))
-  ("math f var" (make 'frac*))
+  ("math f var" (make 'tfrac))
+  ("math f var var" (make 'dfrac))
+  ("math f var var var" (make 'frac*))
   ("math s" (make-sqrt))
-  ("math S" (make-var-sqrt))
+  ("math s var" (make-var-sqrt))
   ("math n" (make-neg))
   ("math O" (make 'op))
   ("math +" (make-rprime "<dag>"))
   ("math a" (make-above))
   ("math b" (make-below))
 
+  ("math:syntax o" (make 'math-ordinary))
+  ("math:syntax space" (make 'math-ignore))
+  ("math:syntax ," (make 'math-separator))
+  ("math:syntax q" (make 'math-quantifier))
+  ("math:syntax = >" (make 'math-imply))
+  ("math:syntax %" (make 'math-or))
+  ("math:syntax &" (make 'math-and))
+  ("math:syntax / var" (make 'math-not))
+  ("math:syntax =" (make 'math-relation))
+  ("math:syntax % var" (make 'math-union))
+  ("math:syntax & var" (make 'math-intersection))
+  ("math:syntax - var" (make 'math-setminus))
+  ("math:syntax +" (make 'math-plus))
+  ("math:syntax -" (make 'math-minus))
+  ("math:syntax *" (make 'math-times))
+  ("math:syntax /" (make 'math-over))
+  ("math:syntax #" (make 'math-prefix))
+  ("math:syntax !" (make 'math-postfix))
+  ("math:syntax (" (make 'math-open))
+  ("math:syntax )" (make 'math-close))
+  ("math:syntax x" (make 'syntax))
+
   ("math B" (make-wide "<bar>"))
   ("math C" (make-wide "<check>"))
   ("math U" (make-wide "<breve>"))
@@ -136,16 +168,22 @@
   ("onehalf" (insert '(frac "1" "2")))
   ("threequarters" (insert '(frac "3" "4")))
 
-  ("space" " ")
+  ("space" (insert-invisible " "))
   ("space var" "<space>")
   ("space var var" "<nospace>")
   ("space var var var" (noop) (make-space "1em"))
   (", var" "<comma>")
   (", var var" "<nocomma>")
+  (", space" "<nocomma>")
+  (", space var" (insert '(separating-space "0.2em")))
+  (", space var var" (insert '(separating-space "1em")))
+  (", ," ",<ldots>,")
+  ("; ;" ";<ldots>;")
   ("." ".")
   (". var" "<point>")
   (". var var" "<cdummy>")
   (". var var var" "<nosymbol>")
+  (". space" "<nosymbol>")
 
   ("math:symbol a" "<amalg>")
   ("math:symbol d" "<partial>")
@@ -187,50 +225,58 @@
   ("math:symbol:circled x" (math-big-operator "otimes"))
   ("math:symbol ." (math-big-operator "."))
 
+  ("symbol &" "&")
+  ("symbol %" "%")
+  ("symbol % var" "<permil>")
   ("symbol \\" "\\")
   ("symbol \\ var" "<setminus>")
   ("symbol \\ var var" "<smallsetminus>")
   ("symbol |" "|")
   ("symbol | /" "<nmid>")
-  ("symbol | var" "<divides>")
-  ("symbol | var /" "<ndivides>")
-  ("symbol | var var" "<shortmid>")
-  ("symbol | var var /" "<nshortmid>")
+  ("symbol | var" "<shortmid>")
+  ("symbol | var /" "<nshortmid>")
+  ("symbol | var var" "<divides>")
+  ("symbol | var var /" "<ndivides>")
   ("symbol | |" "<||>")
   ("symbol | | /" "<nparallel>")
   ("symbol | | var" "<shortparallel>")
   ("symbol | | var /" "<nshortparallel>")
 
-  ("(" (math-bracket-open "(" ")" #f))
-  ("( var" (math-bracket-open "<nobracket>" "<nobracket>" #f))
-  (")" (math-bracket-close ")" "(" #f))
-  ("[" (math-bracket-open "[" "]" #f))
-  ("[ var" (math-bracket-open "[" "[" #f))
-  ("[ var var" (math-bracket-open "]" "]" #f))
-  ("[ var var var" (math-bracket-open "]" "[" #f))
-  ("]" (math-bracket-close "]" "[" #f))
-  ("[ [" (twice (math-bracket-open "[" "]" #f)))
-  ("] ]" (twice (math-bracket-close "]" "[" #f)))
-  ("[ [ var" (math-bracket-open "<llbracket>" "<rrbracket>" #f))
-  ("] ] var" (math-bracket-close "<rrbracket>" "<llbracket>" #f))
-  ("{" (math-bracket-open "{" "}" #f))
-  ("{ var" (math-bracket-open "<langle>" "<rangle>" #f))
-  ("{ var var" (math-bracket-open "<llangle>" "<rrangle>" #f))
-  ("{ {" (twice (math-bracket-open "{" "}" #f)))
-  ("{ { var" (math-bracket-open "<llangle>" "<rrangle>" #f))
-  ("}" (math-bracket-close "}" "{" #f))
-  ("|" (math-bracket-open "|" "|" #f))
-  ("| |" (math-bracket-open "<||>" "<||>" #f))
-  ("| | |" (math-bracket-open "<interleave>" "<interleave>" #f))
-  ("| ," (math-bracket-open "<lfloor>" "<rfloor>" #f))
-  ("| , var" (math-bracket-open "<lfloor>" "<rceil>" #f))
-  (", |" (math-bracket-close "<rfloor>" "<lfloor>" #f))
-  ("| '" (math-bracket-open "<lceil>" "<rceil>" #f))
-  ("| ' var" (math-bracket-open "<lceil>" "<rfloor>" #f))
-  ("' |" (math-bracket-close "<rceil>" "<lceil>" #f))
-  ("| accent:acute" (math-bracket-open "<lceil>" "<rceil>" #f))
-  ("| accent:acute var" (math-bracket-open "<lceil>" "<rfloor>" #f))
-  ("accent:acute |" (math-bracket-close "<rceil>" "<lceil>" #f))
+  ("math:small (" (math-bracket-open "(" ")" #f))
+  ("math:small ( var" (math-bracket-open "<nobracket>" "<nobracket>" #f))
+  ("math:small )" (math-bracket-close ")" "(" #f))
+  ("math:small [" (math-bracket-open "[" "]" #f))
+  ("math:small [ var" (math-bracket-open "[" "[" #f))
+  ("math:small [ var var" (math-bracket-open "]" "]" #f))
+  ("math:small [ var var var" (math-bracket-open "]" "[" #f))
+  ("math:small ]" (math-bracket-close "]" "[" #f))
+  ("math:small [ [" (twice (math-bracket-open "[" "]" #f)))
+  ("math:small ] ]" (twice (math-bracket-close "]" "[" #f)))
+  ("math:small [ [ var" (math-bracket-open "<llbracket>" "<rrbracket>" #f))
+  ("math:small ] ] var" (math-bracket-close "<llbracket>" "<rrbracket>" #f))
+  ("math:small {" (math-bracket-open "{" "}" #f))
+  ("math:small }" (math-bracket-close "}" "{" #f))
+  ("math:small <" (math-bracket-open "<langle>" "<rangle>" #f))
+  ("math:small < var" (math-bracket-open "<llangle>" "<rrangle>" #f))
+  ("math:small < <" (math-bracket-open "<llangle>" "<rrangle>" #f))
+  ("math:small >" (math-bracket-close "<rangle>" "<langle>" #f))
+  ("math:small > var" (math-bracket-close "<rrangle>" "<llangle>" #f))
+  ("math:small > >" (math-bracket-close "<rrangle>" "<llangle>" #f))
+  ("math:small |" (math-bracket-open "|" "|" #f))
+  ("math:small | var" (math-separator "|" #f))
+  ("math:small | |" (math-bracket-open "<||>" "<||>" #f))
+  ("math:small | | var" (math-separator "<||>" #f))
+  ("math:small | | |" (math-bracket-open "<interleave>" "<interleave>" #f))
+  ("math:small /" (math-bracket-open "/" "\\" #f))
+  ("math:small / var" (math-separator "/" #f))
+  ("math:small \\" (math-bracket-open "\\" "/" #f))
+  ("math:small \\ var" (math-separator "\\" #f))
+  ("math:small | ." (math-bracket-open "<lfloor>" "<rfloor>" #f))
+  ("math:small . |" (math-bracket-close "<rfloor>" "<lfloor>" #f))
+  ("math:small | `" (math-bracket-open "<lceil>" "<rceil>" #f))
+  ("math:small ` |" (math-bracket-close "<rceil>" "<lceil>" #f))
+  ("math:small | accent:grave" (math-bracket-open "<lceil>" "<rceil>" #f))
+  ("math:small accent:grave |" (math-bracket-close "<rceil>" "<lceil>" #f))
 
   ("math:left (" (math-bracket-open "(" ")" #t))
   ("math:left )" (math-bracket-open ")" "(" #t))
@@ -238,21 +284,21 @@
   ("math:left ]" (math-bracket-open "]" "[" #t))
   ("math:left {" (math-bracket-open "{" "}" #t))
   ("math:left }" (math-bracket-open "}" "{" #t))
-  ("math:left <" (math-bracket-open "langle" "rangle" #t))
-  ("math:left >" (math-bracket-open "rangle" "langle" #t))
+  ("math:left <" (math-bracket-open "<langle>" "<rangle>" #t))
+  ("math:left >" (math-bracket-open "<rangle>" "<langle>" #t))
   ("math:left |" (math-bracket-open "|" "|" #t))
-  ("math:left | |" (math-bracket-open "||" "||" #t))
+  ("math:left | |" (math-bracket-open "<||>" "<||>" #t))
   ("math:left /" (math-bracket-open "/" "\\" #t))
   ("math:left \\" (math-bracket-open "\\" "/" #t))
-  ("math:left | ," (math-bracket-open "lfloor" "rfloor" #t))
-  ("math:left | , var" (math-bracket-open "lfloor" "rceil" #t))
-  ("math:left , |" (math-bracket-open "rfloor" "lfloor" #t))
-  ("math:left | '" (math-bracket-open "lceil" "rceil" #t))
-  ("math:left | '" (math-bracket-open "lceil" "rfloor" #t))
-  ("math:left ' | var" (math-bracket-open "rceil" "lceil" #t))
-  ("math:left | accent:acute" (math-bracket-open "lceil" "rceil" #t))
-  ("math:left | accent:acute var" (math-bracket-open "lceil" "rfloor" #t))
-  ("math:left accent:acute |" (math-bracket-open "rceil" "lceil" #t))
+  ("math:left | ." (math-bracket-open "<lfloor>" "<rfloor>" #t))
+  ("math:left | . var" (math-bracket-open "<lfloor>" "<rceil>" #t))
+  ("math:left . |" (math-bracket-open "<rfloor>" "<lfloor>" #t))
+  ("math:left | `" (math-bracket-open "<lceil>" "<rceil>" #t))
+  ("math:left | `" (math-bracket-open "<lceil>" "<rfloor>" #t))
+  ("math:left ` | var" (math-bracket-open "<rceil>" "<lceil>" #t))
+  ("math:left | accent:grave" (math-bracket-open "<lceil>" "<rceil>" #t))
+  ("math:left | accent:grave var" (math-bracket-open "<lceil>" "<rfloor>" #t))
+  ("math:left accent:grave |" (math-bracket-open "<rceil>" "<lceil>" #t))
   ("math:left ." (math-bracket-open "." "." #t))
 
   ("math:middle (" (math-separator "(" #t))
@@ -261,18 +307,18 @@
   ("math:middle ]" (math-separator "]" #t))
   ("math:middle {" (math-separator "{" #t))
   ("math:middle }" (math-separator "}" #t))
-  ("math:middle <" (math-separator "langle" #t))
-  ("math:middle >" (math-separator "rangle" #t))
+  ("math:middle <" (math-separator "<langle>" #t))
+  ("math:middle >" (math-separator "<rangle>" #t))
   ("math:middle |" (math-separator "|" #t))
-  ("math:middle | |" (math-separator "||" #t))
+  ("math:middle | |" (math-separator "<||>" #t))
   ("math:middle /" (math-separator "/" #t))
   ("math:middle \\" (math-separator "\\" #t))
-  ("math:left | ," (math-separator "lfloor" #t))
-  ("math:left , |" (math-separator "rfloor" #t))
-  ("math:left | '" (math-separator "lceil" #t))
-  ("math:left ' |" (math-separator "rceil" #t))
-  ("math:left | accent:acute" (math-separator "lceil" #t))
-  ("math:left accent:acute |" (math-separator "rceil" #t))
+  ("math:left | ." (math-separator "<lfloor>" #t))
+  ("math:left . |" (math-separator "<rfloor>" #t))
+  ("math:left | `" (math-separator "<lceil>" #t))
+  ("math:left ` |" (math-separator "<rceil>" #t))
+  ("math:left | accent:grave" (math-separator "<lceil>" #t))
+  ("math:left accent:grave |" (math-separator "<rceil>" #t))
   ("math:middle ." (math-separator "." #t))
 
   ("math:right (" (math-bracket-close "(" ")" #t))
@@ -281,50 +327,63 @@
   ("math:right ]" (math-bracket-close "]" "[" #t))
   ("math:right {" (math-bracket-close "{" "}" #t))
   ("math:right }" (math-bracket-close "}" "{" #t))
-  ("math:right <" (math-bracket-close "langle" "rangle" #t))
-  ("math:right >" (math-bracket-close "rangle" "langle" #t))
+  ("math:right <" (math-bracket-close "<langle>" "<rangle>" #t))
+  ("math:right >" (math-bracket-close "<rangle>" "<langle>" #t))
   ("math:right |" (math-bracket-close "|" "|" #t))
-  ("math:right | |" (math-bracket-close "||" "||" #t))
+  ("math:right | |" (math-bracket-close "<||>" "<||>" #t))
   ("math:right /" (math-bracket-close "/" "\\" #t))
   ("math:right \\" (math-bracket-close "\\" "/" #t))
-  ("math:right | ," (math-bracket-close "lfloor" "rfloor" #t))
-  ("math:right , |" (math-bracket-close "rfloor" "lfloor" #t))
-  ("math:right | '" (math-bracket-close "lceil" "rceil" #t))
-  ("math:right ' |" (math-bracket-close "rceil" "lceil" #t))
-  ("math:right | accent:acute" (math-bracket-close "lceil" "rceil" #t))
-  ("math:right accent:acute |" (math-bracket-close "rceil" "lceil" #t))
+  ("math:right | ." (math-bracket-close "<lfloor>" "<rfloor>" #t))
+  ("math:right . |" (math-bracket-close "<rfloor>" "<lfloor>" #t))
+  ("math:right | `" (math-bracket-close "<lceil>" "<rceil>" #t))
+  ("math:right ` |" (math-bracket-close "<rceil>" "<lceil>" #t))
+  ("math:right | accent:grave" (math-bracket-close "<lceil>" "<rceil>" #t))
+  ("math:right accent:grave |" (math-bracket-close "<rceil>" "<lceil>" #t))
   ("math:right ." (math-bracket-close "." "." #t))
 
-  ("math:large (" (math-bracket-open "(" ")" #t))
-  ("math:large ( var" (math-bracket-open "." "." #t))
-  ("math:large )" (math-bracket-close ")" "(" #t))
-  ("math:large [" (math-bracket-open "[" "]" #t))
-  ("math:large [ var" (math-bracket-open "[" "[" #t))
-  ("math:large [ var var" (math-bracket-open "]" "]" #t))
-  ("math:large [ var var var" (math-bracket-open "]" "[" #t))
-  ("math:large ]" (math-bracket-close "]" "[" #t))
-  ("math:large [ [" (twice (math-bracket-open "[" "]" #t)))
-  ("math:large ] ]" (twice (math-bracket-close "]" "[" #t)))
-  ("math:large [ [ var" (math-bracket-open "llbracket" "rrbracket" #t))
-  ("math:large ] ] var" (math-bracket-close "llbracket" "rrbracket" #t))
-  ("math:large {" (math-bracket-open "{" "}" #t))
-  ("math:large }" (math-bracket-close "}" "{" #t))
-  ("math:large <" (math-bracket-open "langle" "rangle" #t))
-  ("math:large >" (math-bracket-close "rangle" "langle" #t))
-  ("math:large |" (math-bracket-open "|" "|" #t))
-  ("math:large | var" (math-separator "|" #t))
-  ("math:large | |" (math-bracket-open "||" "||" #t))
-  ("math:large | | var" (math-separator "||" #t))
-  ("math:large /" (math-bracket-open "/" "\\"#t))
-  ("math:large / var" (math-separator "/" #t))
-  ("math:large \\" (math-bracket-open "\\" "/" #t))
-  ("math:large \\ var" (math-separator "\\" #t))
-  ("math:large | ," (math-bracket-open "lfloor" "rfloor" #t))
-  ("math:large , |" (math-bracket-close "rfloor" "lfloor" #t))
-  ("math:large | '" (math-bracket-open "lceil" "rceil" #t))
-  ("math:large ' |" (math-bracket-close "rceil" "lceil" #t))
-  ("math:large | accent:acute" (math-bracket-open "lceil" "rceil" #t))
-  ("math:large accent:acute |" (math-bracket-close "rceil" "lceil" #t))
+  ("(" (math-bracket-open "(" ")" 'default))
+  ("( var" (math-bracket-open "." "." 'default))
+  ("( space" (math-bracket-open "." "." 'default))
+  (")" (math-bracket-close ")" "(" 'default))
+  ("[" (math-bracket-open "[" "]" 'default))
+  ("[ var" (math-bracket-open "[" "[" 'default))
+  ("[ var var" (math-bracket-open "]" "]" 'default))
+  ("[ var var var" (math-bracket-open "]" "[" 'default))
+  ("]" (math-bracket-close "]" "[" 'default))
+  ("[ [" (twice (math-bracket-open "[" "]" 'default)))
+  ("] ]" (twice (math-bracket-close "]" "[" 'default)))
+  ("[ [ var" (math-bracket-open "<llbracket>" "<rrbracket>" 'default))
+  ("] ] var" (math-bracket-close "<rrbracket>" "<llbracket>" 'default))
+  ("{" (math-bracket-open "{" "}" 'default))
+  ("{ var" (math-bracket-open "<langle>" "<rangle>" 'default))
+  ("{ var var" (math-bracket-open "<llangle>" "<rrangle>" #f))
+  ("{ {" (twice (math-bracket-open "{" "}" 'default)))
+  ("{ { var" (math-bracket-open "<llangle>" "<rrangle>" #f))
+  ("}" (math-bracket-close "}" "{" 'default))
+  ("} var" (math-bracket-close "<rangle>" "<langle>" 'default))
+  ("} var var" (math-bracket-close "<rrangle>" "<llangle>" #f))
+  ("} }" (twice (math-bracket-open "}" "{" 'default)))
+  ("} } var" (math-bracket-open "<rrangle>" "<llangle>" #f))
+  ("|" (math-bracket-open "|" "|" 'default))
+  ("| |" (math-bracket-open "<||>" "<||>" 'default))
+  ("| | |" (math-bracket-open "<interleave>" "<interleave>" #f))
+  ("| ." (math-bracket-open "<lfloor>" "<rfloor>" 'default))
+  ("| . var" (math-bracket-open "<lfloor>" "<rceil>" 'default))
+  (". |" (math-bracket-close "<rfloor>" "<lfloor>" 'default))
+  ("| `" (math-bracket-open "<lceil>" "<rceil>" 'default))
+  ("| ` var" (math-bracket-open "<lceil>" "<rfloor>" 'default))
+  ("` |" (math-bracket-close "<rceil>" "<lceil>" 'default))
+  ("| accent:grave" (math-bracket-open "<lceil>" "<rceil>" 'default))
+  ("| accent:grave var" (math-bracket-open "<lceil>" "<rfloor>" 'default))
+  ("accent:grave |" (math-bracket-close "<rceil>" "<lceil>" 'default))
+  ("| -" (math-bracket-open "|" "|" 'default) (insert "-"))
+  ("| =" (math-bracket-open "|" "|" 'default) (insert "="))
+  ("| | -" (math-bracket-open "<||>" "<||>" 'default) (insert "-"))
+  ("| | =" (math-bracket-open "<||>" "<||>" 'default) (insert "="))
+  ("| | | -" (math-bracket-open "<interleave>" "<interleave>" 'default)
+             (insert "-"))
+  ("| | | =" (math-bracket-open "<interleave>" "<interleave>" 'default)
+             (insert "="))
 
   ("<" "<less>")
   (">" "<gtr>")
@@ -359,48 +418,11 @@
   ("< = ." "<lesseqdot>")
   (". > =" "<gtreqdot>")
 
-  ("< var" "<prec>")
-  ("> var" "<succ>")
-  ("< var /" "<nprec>")
-  ("> var /" "<nsucc>")
-  ("< var =" "<preccurlyeq>")
-  ("> var =" "<succcurlyeq>")
-  ("< var = /" "<npreccurlyeq>")
-  ("> var = /" "<nsucccurlyeq>")
-  ("< var = var" "<preceq>")
-  ("> var = var" "<succeq>")
-  ("< var = var /" "<npreceq>")
-  ("> var = var /" "<nsucceq>")
-  ("< var = var / var" "<precneqq>")
-  ("> var = var / var" "<succneqq>")
-  ("< < var" "<precprec>")
-  ("< < < var" "<precprecprec>")
-  ("> > var" "<succsucc>")
-  ("> > > var" "<succsuccsucc>")
-  ("< < var =" "<precpreceq>")
-  ("< < < var =" "<precprecpreceq>")
-  ("> > var =" "<succsucceq>")
-  ("> > > var =" "<succsuccsucceq>")
-  ("< < var /" "<nprecprec>")
-  ("< < < var /" "<nprecprecprec>")
-  ("> > var /" "<nsuccsucc>")
-  ("> > > var /" "<nsuccsuccsucc>")
-  ("< < var = /" "<nprecpreceq>")
-  ("< < < var = /" "<nprecprecpreceq>")
-  ("> > var = /" "<nsuccsucceq>")
-  ("> > > var = /" "<nsuccsuccsucceq>")
-  ("< var ." "<precdot>")
-  (". > var" "<dotsucc>")
-  ("< var = ." "<preceqdot>")
-  (". > var =" "<dotsucceq>")
-
   ("< >" "<lessgtr>")
   ("> <" "<gtrless>")
   ("> < var" "<join>")
   ("< ~" "<lesssim>")
   ("< ~ ~" "<lessapprox>")
-  ("< var ~" "<precsim>")
-  ("< var ~ ~" "<precapprox>")
   ("> ~" "<gtrsim>")
   ("> ~ ~" "<gtrapprox>")
   ("> var ~" "<gtrsim>")
@@ -417,18 +439,10 @@
   ("< ~ / var" "<lnsim>")
   ("< ~ ~ /" "<nlessapprox>")
   ("< ~ ~ / var" "<lnapprox>")
-  ("< var ~ /" "<nprecsim>")
-  ("< var ~ / var" "<precnsim>")
-  ("< var ~ ~ /" "<nprecapprox>")
-  ("< var ~ ~ / var" "<precnapprox>")
   ("> ~ /" "<ngtrsim>")
   ("> ~ / var" "<gnsim>")
   ("> ~ ~ /" "<ngtrapprox>")
   ("> ~ ~ / var" "<gnapprox>")
-  ("> var ~ /" "<nsuccsim>")
-  ("> var ~ / var" "<succnsim>")
-  ("> var ~ ~ /" "<nsuccapprox>")
-  ("> var ~ ~ / var" "<succnapprox>")
   ("< = var var /" "<nleqq>")
   ("> = var var /" "<ngeqq>")
   ("< = var > /" "<nlesseqgtr>")
@@ -442,6 +456,106 @@
   ("> = var var / var" "<gneqq>")
   ("> = var var / var var" "<gvertneqq>")
 
+  ("< var" "<in>")
+  ("> var" "<ni>")
+  ("< var /" "<nin>")
+  ("> var /" "<nni>")
+
+  ("< var var" "<subset>")
+  ("> var var" "<supset>")
+  ("< var var /" "<nsubset>")
+  ("> var var /" "<nsupset>")
+  ("< var var =" "<subseteq>")
+  ("> var var =" "<supseteq>")
+  ("< var var = /" "<nsubseteq>")
+  ("> var var = /" "<nsupseteq>")
+  ("< var var = var" "<subseteqq>")
+  ("> var var = var" "<supseteqq>")
+  ("< var var = /" "<nsubseteq>")
+  ("> var var = /" "<nsupseteq>")
+  ("< var var = var /" "<nsubseteqq>")
+  ("> var var = var /" "<nsupseteqq>")
+  ("< var var = / var" "<subsetneq>")
+  ("> var var = / var" "<supsetneq>")
+  ("< var var = / var var" "<varsubsetneq>")
+  ("> var var = / var var" "<varsupsetneq>")
+  ("< var var = var / var" "<subsetneqq>")
+  ("> var var = var / var" "<supsetneqq>")
+  ("< var var = var / var var" "<varsubsetneqq>")
+  ("> var var = var / var var" "<varsupsetneqq>")
+  ("< var var +" "<subsetplus>")
+  ("> var var +" "<supsetplus>")
+  ("< var var + =" "<subsetpluseq>")
+  ("> var var + =" "<supsetpluseq>")
+  ("< var var = +" "<subsetpluseq>")
+  ("> var var = +" "<supsetpluseq>")
+  ("< var var ~" "<subsetsim>")
+  ("> var var ~" "<supsetsim>")
+
+  ("< var var var" "<prec>")
+  ("> var var var" "<succ>")
+  ("< var var var /" "<nprec>")
+  ("> var var var /" "<nsucc>")
+  ("< var var var =" "<preccurlyeq>")
+  ("> var var var =" "<succcurlyeq>")
+  ("< var var var = /" "<npreccurlyeq>")
+  ("> var var var = /" "<nsucccurlyeq>")
+  ("< var var var = var" "<preceq>")
+  ("> var var var = var" "<succeq>")
+  ("< var var var = var /" "<npreceq>")
+  ("> var var var = var /" "<nsucceq>")
+  ("< var var var = var / var" "<precneqq>")
+  ("> var var var = var / var" "<succneqq>")
+  ("< var var var ~" "<precsim>")
+  ("> var var var ~" "<succsim>")
+  ("< var var var ~ ~" "<precapprox>")
+  ("> var var var ~ ~" "<succapprox>")
+  ("< var var var ~ /" "<nprecsim>")
+  ("> var var var ~ /" "<nsuccsim>")
+  ("< var var var ~ / var" "<precnsim>")
+  ("> var var var ~ / var" "<succnsim>")
+  ("< var var var ~ ~ /" "<nprecapprox>")
+  ("> var var var ~ ~ /" "<nsuccapprox>")
+  ("< var var var ~ ~ / var" "<precnapprox>")
+  ("> var var var ~ ~ / var" "<succnapprox>")
+  ("< var var var ." "<precdot>")
+  ("< var var var = ." "<preceqdot>")
+  ("< . var" "<precdot>")
+  (". > var" "<dotsucc>")
+  ("< . var =" "<preceqdot>")
+  (". > var =" "<dotsucceq>")
+
+  ("< < var" "<precprec>")
+  ("< < < var" "<precprecprec>")
+  ("> > var" "<succsucc>")
+  ("> > > var" "<succsuccsucc>")
+  ("< < var =" "<precpreceq>")
+  ("< < < var =" "<precprecpreceq>")
+  ("> > var =" "<succsucceq>")
+  ("> > > var =" "<succsuccsucceq>")
+  ("< < var /" "<nprecprec>")
+  ("< < < var /" "<nprecprecprec>")
+  ("> > var /" "<nsuccsucc>")
+  ("> > > var /" "<nsuccsuccsucc>")
+  ("< < var = /" "<nprecpreceq>")
+  ("< < < var = /" "<nprecprecpreceq>")
+  ("> > var = /" "<nsuccsucceq>")
+  ("> > > var = /" "<nsuccsuccsucceq>")
+
+  ("< var var var var" "<sqsubset>")
+  ("> var var var var" "<sqsupset>")
+  ("< var var var var /" "<nsqsubset>")
+  ("> var var var var /" "<nsqsupset>")
+  ("< var var var var =" "<sqsubseteq>")
+  ("> var var var var =" "<sqsupseteq>")
+  ("< var var var var = /" "<nsqsubseteq>")
+  ("> var var var var = /" "<nsqsupseteq>")
+
+  ("< var var var var var" (math-bracket-open "<langle>" "<rangle>" 'default))
+  ("> var var var var var" (math-bracket-close "<rangle>" "<langle>" 'default))
+  ("< < var var" (math-bracket-open "<llangle>" "<rrangle>" 'default))
+  ("> > var var" (math-bracket-close "<rrangle>" "<llangle>" 'default))
+
   ("- >" "<rightarrow>")
   ("- > var" "<downarrow>")
   ("- > var var" "<uparrow>")
@@ -494,6 +608,18 @@
   ("| - - > var var var" "<longhookdownarrow>")
   ("| - - > var var var var" "<longmapsup>")
   ("| - - > var var var var var" "<longhookuparrow>")
+  ("| var - >" "<mapsto>")
+  ("| var - > var" "<hookrightarrow>")
+  ("| var - > var var" "<mapsdown>")
+  ("| var - > var var var" "<hookdownarrow>")
+  ("| var - > var var var var" "<mapsup>")
+  ("| var - > var var var var var" "<hookuparrow>")
+  ("| var - - >" "<longmapsto>")
+  ("| var - - > var" "<longhookrightarrow>")
+  ("| var - - > var var" "<longmapsdown>")
+  ("| var - - > var var var" "<longhookdownarrow>")
+  ("| var - - > var var var var" "<longmapsup>")
+  ("| var - - > var var var var var" "<longhookuparrow>")
   ("< - |" "<mapsfrom>")
   ("< - | var" "<hookleftarrow>")
   ("< - | var var" "<mapsup>")
@@ -536,10 +662,12 @@
   ("< | -" "<leftarrowtriangle>")
   ("< | - | >" "<leftrightarrowtriangle>")
   ("| - | >" "<longmapstotriangle>")
+  ("| var - | >" "<longmapstotriangle>")
   ("- - | >" "<longrightarrowtriangle>")
   ("< | - -" "<longleftarrowtriangle>")
   ("< | - - | >" "<longleftrightarrowtriangle>")
   ("| - - | >" "<longmapstotriangle>")
+  ("| var - - | >" "<longmapstotriangle>")
   ("< - < var" "<leftprec>")
   ("> var - >" "<succright>")
   ("< - < var =" "<leftpreceq>")
@@ -549,10 +677,12 @@
   ("< - !" "<leftarrowlim>")
   ("< - > !" "<leftrightarrowlim>")
   ("| - > !" "<mapstolim>")
+  ("| var - > !" "<mapstolim>")
   ("- - > !" "<longrightarrowlim>")
   ("< - - !" "<longleftarrowlim>")
   ("< - - > !" "<longleftrightarrowlim>")
   ("| - - > !" "<longmapstolim>")
+  ("| var - - > !" "<longmapstolim>")
   ("~ > !" "<rightsquigarrowlim>")
   ("< ~ !" "<leftsquigarrowlim>")
   ("< ~ > !" "<leftrightsquigarrowlim>")
@@ -618,28 +748,72 @@
   ("~ ~ /" "<napprox>")
   ("~ - /" "<nsimeq>")
   ("~ = /" "<ncong>")
-    
+
+  ("# var" "<sharp>")
+  ("# var var" "<natural>")
+  ("# var var var" "<flat>")
+  ("&" "<wedge>")
+  ("& var" "<cap>")
+  ("& var var" "<sqcap>")
+  ("& var var var" "&")
+  ("%" "<vee>")
+  ("% var" "<cup>")
+  ("% var +" "<uplus>")
+  ("% var var" "<sqcup>")
+  ("% var var var" "<amalg>")
+  ("% var var var var" "%")
+  ("% var var var var var" "<permil>")
+
+  ("| /" "<nmid>")
+  ("| var" "|")
+  ("| var /" "<nmid>")
+  ("| var var" (math-separator "|" 'default))
+  ("| var var/" "<nmid>")
+  ("| var var var" "<shortmid>")
+  ("| var var var /" "<nshortmid>")
+  ("| var var var var" "<divides>")
+  ("| var var var var var" "<divides>")
+  ("| | var" "<||>")
+  ("| | var /" "<nparallel>")
+  ("| | var var" (math-separator "<||>" 'default))
+  ("| | var var /" "<nparallel>")
+  ("| | var var var" "<shortparallel>")
+  ("| | var var var /" "<nshortparallel>")
+  ("| | | var" "<interleave>")
+
   ("| - var" "<vdash>")
   ("| - var -" "<vdash>")
   ("| - -" "<longvdash>")
+  ("| var -" "<vdash>")
+  ("| var - -" "<longvdash>")
   ("| | - var" "<Vdash>")
   ("| | - var -" "<longVdash>")
   ("| | - -" "<longVdash>")
+  ("| | var -" "<Vdash>")
+  ("| | var - -" "<longVdash>")
   ("| | | - var" "<Vvdash>")
   ("| | | - var -" "<longVvdash>")
   ("| | | - -" "<longVvdash>")
+  ("| | | var -" "<Vvdash>")
+  ("| | | var - -" "<longVvdash>")
   ("- | var" "<dashv>")
   ("- - |" "<longdashv>")
   ("| = var" "<vDash>")
   ("| = var =" "<longvDash>")
   ("| = =" "<longvDash>")
+  ("| var =" "<vDash>")
+  ("| var = =" "<longvDash>")
   ;; ("| | = var" "<VDash>")
   ;; ("| | = var =" "<longVDash>")
   ;; ("| | = =" "<longVDash>")
+  ;; ("| | var =" "<VDash>")
+  ;; ("| | var = =" "<longVDash>")
   ("| - var /" "<nvdash>")
   ("| - /" "<nvdash>")
+  ("| var - /" "<nvdash>")
   ("| | - var /" "<nVdash>")
   ("| | - /" "<nVdash>")
+  ("| | var - /" "<nVdash>")
   ;; ("- | var /" "<ndashv>")
   ;; ("- | /" "<ndashv>")
   ;; ("- | var | /" "<ndashV>")
@@ -647,8 +821,10 @@
   ;; ("- | | /" "<ndashV>")
   ("| = var /" "<nvDash>")
   ("| = /" "<nvDash>")
+  ("| var = /" "<nvDash>")
   ;; ("| | = var /" "<nVDash>")
   ;; ("| | = /" "<nVDash>")
+  ;; ("| | var = /" "<nVDash>")
   ;; ("= | var /" "<nDashv>")
   ;; ("= | /" "<nDashv>")
   ;; ("= | var | /" "<nDashV>")
@@ -669,12 +845,21 @@
   ("| > var = /" "<ntrianglerighteqslant>")
   ("| > var = var" "<trianglerighteq>")
   ("| > var = var /" "<ntrianglerighteq>")
-    
+  ("| var >" "<vartriangleright>")
+  ("| var > var" "<blacktriangleright>")
+  ("| var > /" "<ntriangleright>")
+  ("| var > =" "<trianglerighteqslant>")
+  ("| var > = /" "<ntrianglerighteqslant>")
+  ("| var > = var" "<trianglerighteq>")
+  ("| var > = var /" "<ntrianglerighteq>")
+
   ("+ var" "<upl>")
   ("+ var var" "<noplus>")
   ("+ var var var" "<kreuz>")
   ("+ var var var var" "<dag>")
   ("+ var var var var var" "<ddag>")
+  ("+ space" "<noplus>")
+  ("+ +" "+<cdots>+")
   ("- var" "<um>")
   ("+ -" "<pm>")
   ("+ - var" "<upm>")
@@ -691,91 +876,27 @@
   (". . var var var" "<vdots>")
   (". . var var var var" "<ddots>")
   (". . var var var var var" "<udots>")
+  (":" ":")
+  (": var" "<of>")
   (": :" "<colons>")
   (": >" "<transtype>")
   (": =" "<assign>")
   ("+ =" "<plusassign>")
   ("- =" "<minusassign>")
   ("/ var" "<div>")
+  ("*" (insert-invisible "*"))
+  ("* *" (insert-invisible "*") (insert "<cdots>") (insert-invisible "*"))
   ("* var" "<times>")
   ("* var var" "<ast>")
   ("* var var var" "<cdot>")
+  ("* &" "<exterior>")
   ("| * var" "<ltimes>")
   ("| * var |" "<join>")
+  ("| var *" "<ltimes>")
+  ("| var * |" "<join>")
   ("* | var" "<rtimes>")
   ("* var var |" "<rtimes>")
 
-  ("< var var" "<subset>")
-  ("> var var" "<supset>")
-  ("< var var /" "<nsubset>")
-  ("> var var /" "<nsupset>")
-  ("< var var =" "<subseteq>")
-  ("> var var =" "<supseteq>")
-  ("< var var = /" "<nsubseteq>")
-  ("> var var = /" "<nsupseteq>")
-  ("< var var = var" "<subseteqq>")
-  ("> var var = var" "<supseteqq>")
-  ("< var var = /" "<nsubseteq>")
-  ("> var var = /" "<nsupseteq>")
-  ("< var var = var /" "<nsubseteqq>")
-  ("> var var = var /" "<nsupseteqq>")
-  ("< var var = / var" "<subsetneq>")
-  ("> var var = / var" "<supsetneq>")
-  ("< var var = / var var" "<varsubsetneq>")
-  ("> var var = / var var" "<varsupsetneq>")
-  ("< var var = var / var" "<subsetneqq>")
-  ("> var var = var / var" "<supsetneqq>")
-  ("< var var = var / var var" "<varsubsetneqq>")
-  ("> var var = var / var var" "<varsupsetneqq>")
-  ("< var var +" "<subsetplus>")
-  ("> var var +" "<supsetplus>")
-  ("< var var + =" "<subsetpluseq>")
-  ("> var var + =" "<supsetpluseq>")
-  ("< var var = +" "<subsetpluseq>")
-  ("> var var = +" "<supsetpluseq>")
-  ("< var var ~" "<subsetsim>")
-  ("> var var ~" "<supsetsim>")
-  ("< var var var" "<in>")
-  ("> var var var" "<ni>")
-  ("< var var var /" "<nin>")
-  ("> var var var /" "<nni>")
-
-  ("< var var var var" "<sqsubset>")
-  ("> var var var var" "<sqsupset>")
-  ("< var var var var /" "<nsqsubset>")
-  ("> var var var var /" "<nsqsupset>")
-  ("< var var var var =" "<sqsubseteq>")
-  ("> var var var var =" "<sqsupseteq>")
-  ("< var var var var = /" "<nsqsubseteq>")
-  ("> var var var var = /" "<nsqsupseteq>")
-
-  ("< var var var var var" (math-bracket-open "<langle>" "<rangle>" #f))
-  ("> var var var var var" (math-bracket-close "<rangle>" "<langle>" #f))
-  ("< < var var" (math-bracket-open "<llangle>" "<rrangle>" #f))
-  ("> > var var" (math-bracket-close "<rrangle>" "<llangle>" #f))
-
-  ("# var" "<sharp>")
-  ("# var var" "<natural>")
-  ("# var var var" "<flat>")
-  ("& var" "<wedge>")
-  ("& var var" "<cap>")
-  ("& var var var" "<sqcap>")
-  ("| var " "<vee>")
-  ("| var var" "<cup>")
-  ("| var var +" "<uplus>")
-  ("| var var var" "<sqcup>")
-  ("| var var var var" "|")
-  ("| var var var var /" "<nmid>")
-  ("| var var var var var" "<divides>")
-  ("| var var var var var /" "<ndivides>")
-  ("| var var var var var var" "<shortmid>")
-  ("| var var var var var var /" "<nshortmid>")
-  ("| | var" "<||>")
-  ("| | var /" "<nparallel>")
-  ("| | var var" "<shortparallel>")
-  ("| | var var /" "<nshortparallel>")
-  ("| | | var" "<interleave>")
-
   ("math:greek a" "<alpha>")
   ("math:greek b" "<beta>")
   ("math:greek g" "<gamma>")
@@ -792,6 +913,7 @@
   ("math:greek k" "<kappa>")
   ("math:greek k var" "<varkappa>")
   ("math:greek l" "<lambda>")
+  ("math:greek l var" "<mathlambda>")
   ("math:greek m" "<mu>")
   ("math:greek n" "<nu>")
   ("math:greek x" "<xi>")
@@ -818,7 +940,9 @@
   ("math:greek A" "<Alpha>")
   ("math:greek B" "<Beta>")
   ("math:greek G" "<Gamma>")
+  ("math:greek G var" "<mathGamma>")
   ("math:greek D" "<Delta>")
+  ("math:greek D var" "<mathLaplace>")
   ("math:greek E" "<Epsilon>")
   ("math:greek E var" "<Backepsilon>")
   ("math:greek Z" "<Zeta>")
@@ -844,15 +968,18 @@
   ("math:greek W var" "<Mho>")
   ("math:greek V" "<Phi>")
 
+  ("math:greek &" "&")
+  ("math:greek %" "%")
+  ("math:greek % var" "<permil>")
   ("math:greek \\" "\\")
   ("math:greek \\ var" "<setminus>")
   ("math:greek \\ var var" "<smallsetminus>")
   ("math:greek |" "|")
   ("math:greek | /" "<nmid>")
-  ("math:greek | var" "<divides>")
-  ("math:greek | var /" "<ndivides>")
-  ("math:greek | var var" "<shortmid>")
-  ("math:greek | var var /" "<nshortmid>")
+  ("math:greek | var" "<shortmid>")
+  ("math:greek | var /" "<nshortmid>")
+  ("math:greek | var var" "<divides>")
+  ("math:greek | var var /" "<ndivides>")
   ("math:greek | |" "<||>")
   ("math:greek | | /" "<nparallel>")
   ("math:greek | | var" "<shortparallel>")
@@ -990,6 +1117,32 @@
   ("math:bold:greek W var" "<b-Mho>")
   ("math:bold:greek V" "<b-Phi>")
 
+  ("math:cal a" "<cal-a>")
+  ("math:cal b" "<cal-b>")
+  ("math:cal c" "<cal-c>")
+  ("math:cal d" "<cal-d>")
+  ("math:cal e" "<cal-e>")
+  ("math:cal f" "<cal-f>")
+  ("math:cal g" "<cal-g>")
+  ("math:cal h" "<cal-h>")
+  ("math:cal i" "<cal-i>")
+  ("math:cal j" "<cal-j>")
+  ("math:cal k" "<cal-k>")
+  ("math:cal l" "<cal-l>")
+  ("math:cal m" "<cal-m>")
+  ("math:cal n" "<cal-n>")
+  ("math:cal o" "<cal-o>")
+  ("math:cal p" "<cal-p>")
+  ("math:cal q" "<cal-q>")
+  ("math:cal r" "<cal-r>")
+  ("math:cal s" "<cal-s>")
+  ("math:cal t" "<cal-t>")
+  ("math:cal u" "<cal-u>")
+  ("math:cal v" "<cal-v>")
+  ("math:cal w" "<cal-w>")
+  ("math:cal x" "<cal-x>")
+  ("math:cal y" "<cal-y>")
+  ("math:cal z" "<cal-z>")
   ("math:cal A" "<cal-A>")
   ("math:cal B" "<cal-B>")
   ("math:cal C" "<cal-C>")
@@ -1017,6 +1170,32 @@
   ("math:cal Y" "<cal-Y>")
   ("math:cal Z" "<cal-Z>")
 
+  ("math:bold:cal a" "<b-cal-a>")
+  ("math:bold:cal b" "<b-cal-b>")
+  ("math:bold:cal c" "<b-cal-c>")
+  ("math:bold:cal d" "<b-cal-d>")
+  ("math:bold:cal e" "<b-cal-e>")
+  ("math:bold:cal f" "<b-cal-f>")
+  ("math:bold:cal g" "<b-cal-g>")
+  ("math:bold:cal h" "<b-cal-h>")
+  ("math:bold:cal i" "<b-cal-i>")
+  ("math:bold:cal h" "<b-cal-j>")
+  ("math:bold:cal k" "<b-cal-k>")
+  ("math:bold:cal l" "<b-cal-l>")
+  ("math:bold:cal m" "<b-cal-m>")
+  ("math:bold:cal n" "<b-cal-n>")
+  ("math:bold:cal o" "<b-cal-o>")
+  ("math:bold:cal p" "<b-cal-p>")
+  ("math:bold:cal q" "<b-cal-q>")
+  ("math:bold:cal r" "<b-cal-r>")
+  ("math:bold:cal s" "<b-cal-s>")
+  ("math:bold:cal t" "<b-cal-t>")
+  ("math:bold:cal u" "<b-cal-u>")
+  ("math:bold:cal v" "<b-cal-v>")
+  ("math:bold:cal w" "<b-cal-w>")
+  ("math:bold:cal x" "<b-cal-x>")
+  ("math:bold:cal y" "<b-cal-y>")
+  ("math:bold:cal z" "<b-cal-z>")
   ("math:bold:cal A" "<b-cal-A>")
   ("math:bold:cal B" "<b-cal-B>")
   ("math:bold:cal C" "<b-cal-C>")
@@ -1221,7 +1400,8 @@
   ("k var" "<kappa>")
   ("k var var" "<varkappa>")
   ("l var" "<lambda>")
-  ("l var var" "<ell>")
+  ("l var var" "<mathlambda>")
+  ("l var var var" "<ell>")
   ("m var" "<mu>")
   ("n var" "<nu>")
   ("o var" "<omicron>")
@@ -1254,6 +1434,7 @@
   ("D var" "<Delta>")
   ("D var var" "<mathD>")
   ("D var var var" "<daleth>")
+  ("D var var var var" "<mathLaplace>")
   ("E var" "<Epsilon>")
   ("E var var" "<exists>")
   ("E var var var" "<Exists>")
@@ -1261,6 +1442,7 @@
   ("F var" "<Phi>")
   ("G var" "<Gamma>")
   ("G var var" "<gimel>")
+  ("G var var var" "<mathGamma>")
   ("H var" "<Eta>")
   ("I var" "<Iota>")
   ("J var" "<Theta>")
