@@ -19,15 +19,10 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (menu-bind insert-math-menu
-  (when (not (in-math?))
-	("Formula" "$" (begin (noop) (make-with "mode" "math"))))
-  (when (in-math?)
-	("Text" "A-$" (begin (noop) (make-with "mode" "text"))))
+  ("Formula" (make 'math))
   (if (style-has? "env-math-dtd")
-      (when (in-text?)
-	    ---
-	    ("Equation" (make-equation*))
-	    ("Equations" (make-eqnarray*)))))
+      ("Equation" (make-equation*))
+      ("Equations" (make-eqnarray*))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; The Mathematics menu
@@ -39,7 +34,6 @@
   ("N-th root" (make-var-sqrt))
   ("Negation" (make-neg))
   ("Tree" (make-tree))
-  ("Text" (make-with "mode" "text"))
   ---
   (-> "Size tag" (link size-tag-menu))
   (-> "Script"
@@ -49,13 +43,14 @@
       ("Right superscript" (make-script #t #t))
       ("Script below" (make-below))
       ("Script above" (make-above)))
-  (-> "Accent#above"
+  (-> "Accent above"
       ("Tilda" (make-wide "~"))
       ("Hat" (make-wide "^"))
       ("Bar" (make-wide "<bar>"))
       ("Vector" (make-wide "<vect>"))
       ("Check" (make-wide "<check>"))
       ("Breve" (make-wide "<breve>"))
+      ---
       ("Acute" (make-wide "<acute>"))
       ("Grave" (make-wide "<grave>"))
       ("Dot" (make-wide "<dot>"))
@@ -69,13 +64,14 @@
       ("Right arrow" (make-wide "<wide-varrightarrow>"))
       ("Left arrow" (make-wide "<wide-varleftarrow>"))
       ("Wide bar" (make-wide "<wide-bar>")))
-  (-> "Accent#below"
+  (-> "Accent below"
       ("Tilda" (make-wide-under "~"))
       ("Hat" (make-wide-under "^"))
       ("Bar" (make-wide-under "<bar>"))
       ("Vector" (make-wide-under "<vect>"))
       ("Check" (make-wide-under "<check>"))
       ("Breve" (make-wide-under "<breve>"))
+      ---
       ("Acute" (make-wide-under "<acute>"))
       ("Grave" (make-wide-under "<grave>"))
       ("Dot" (make-wide-under "<dot>"))
@@ -90,17 +86,77 @@
       ("Left arrow" (make-wide-under "<wide-varleftarrow>"))
       ("Wide bar" (make-wide-under "<wide-bar>")))
   (-> "Symbol" (link symbol-menu))
+  (-> "Textual operator" (link textual-operator-menu))
+  (if (== (get-preference "semantic editing") "on")
+      (-> "Semantics" (link semantic-annotation-menu)))
   ---
-  (if (or (table-inside? 'eqnarray) (table-inside? 'eqnarray*)
-	  (inside? 'equation) (inside? 'equation*))
-      (if (not (numbered?)) ("Number equation" (toggle-number)))
-      (if (numbered?) ("Remove number" (toggle-number))))
-  (if (not (or (table-inside? 'eqnarray) (table-inside? 'eqnarray*)
-	       (inside? 'equation) (inside? 'equation*)))
-      (when #f ("Number equation" (toggle-number)))))
+  ("Text" (make 'text))
+  (-> "Table" (link insert-table-menu))
+  (-> "Image" (link insert-image-menu))
+  (-> "Link" (link insert-link-menu))
+  (if (detailed-menus?)
+      (if (style-has? "std-fold-dtd")
+	  (-> "Fold" (link insert-fold-menu)))
+      (-> "Animation" (link insert-animation-menu))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;; The mathematical Symbol menu
+;; Semantic math menus
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(menu-bind math-correct-menu
+  ("Correct all" (math-correct-all))
+  (when (with-versioning-tool?)
+    ("Correct manually" (math-correct-manually)))
+  ---
+  (group "Options")
+  ("Remove superfluous invisible operators"
+   (toggle-preference "manual remove superfluous invisible"))
+  ("Insert missing invisible operators"
+   (toggle-preference "manual insert missing invisible"))
+  ("Homoglyph substitutions"
+   (toggle-preference "manual homoglyph correct")))
+
+(menu-bind context-preferences-menu
+  ("Show full context" (toggle-preference "show full context"))
+  (when (inside? 'table)
+    ("Show table cells" (toggle-preference "show table cells")))
+  ("Show current focus" (toggle-preference "show focus"))
+  (when (!= (get-preference "semantic editing") "off")
+    ("Only show semantic focus"
+      (toggle-preference "show only semantic focus"))))
+
+(menu-bind semantic-math-preferences-menu
+  ("Semantic editing" (toggle-preference "semantic editing"))
+  (when (== (get-preference "semantic editing") "on")
+    ("Semantic selections" (toggle-preference "semantic selections"))))
+
+(menu-bind semantic-annotation-menu
+  ("Ordinary symbol" (make 'math-ordinary))
+  ("Ignore" (make 'math-ignore))
+  ---
+  ("Separator" (make 'math-separator))
+  ("Quantifier" (make 'math-quantifier))
+  ("Logical implication" (make 'math-imply))
+  ("Logical or" (make 'math-or))
+  ("Logical and" (make 'math-and))
+  ("Logical not" (make 'math-not))
+  ("Relation" (make 'math-relation))
+  ("Set union" (make 'math-union))
+  ("Set intersection" (make 'math-intersection))
+  ("Set difference" (make 'math-exclude))
+  ("Addition" (make 'math-plus))
+  ("Subtraction" (make 'math-minus))
+  ("Multiplication" (make 'math-times))
+  ("Division" (make 'math-over))
+  ("Prefix" (make 'math-prefix))
+  ("Postfix" (make 'math-postfix))
+  ("Open" (make 'math-open))
+  ("Close" (make 'math-close))
+  ---
+  ("Other" (make 'syntax)))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; The mathematical symbol menus
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (menu-bind symbol-menu
@@ -123,7 +179,7 @@
       ---
       (tile 6 (link long-arrow-menu)))
   (-> "Negation"
-      ("General negation" "/" (key-press "/"))
+      ("General negation" (key-press "/"))
       ---
       (tile 9 (link negation-menu-1))
       ---
@@ -138,10 +194,41 @@
       ---
       (tile 6 (link dots-menu))))
 
+(menu-bind textual-operator-menu
+  ("Normal" (make 'math-up))
+  ("Sans serif" (make 'math-ss))
+  ("Typewriter" (make 'math-tt))
+  ("Bold" (make 'math-bf))
+  ("Italic" (make 'math-it))
+  ("Slanted" (make 'math-sl)))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Large delimiters
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
+(menu-bind large-delimiter-menu
+  (symbol "<left-(-2>" (math-bracket-open "(" ")" 'default))
+  (symbol "<left-)-2>" (math-bracket-open ")" "(" 'default))
+  (symbol "<left-[-2>" (math-bracket-open "[" "]" 'default))
+  (symbol "<left-]-2>" (math-bracket-open "]" "[" 'default))
+  (symbol "<left-{-2>" (math-bracket-open "{" "}" 'default))
+  (symbol "<left-}-2>" (math-bracket-open "}" "{" 'default))
+  (symbol "<left-langle-2>" (math-bracket-open "<langle>" "<rangle>" 'default))
+  (symbol "<left-rangle-2>" (math-bracket-open "<rangle>" "<langle>" 'default))
+  (symbol "<left-lfloor-2>" (math-bracket-open "<lfloor>" "<rfloor>" 'default))
+  (symbol "<left-rfloor-2>" (math-bracket-open "<rfloor>" "<lfloor>" 'default))
+  (symbol "<left-lceil-2>" (math-bracket-open "<lceil>" "<rceil>" 'default))
+  (symbol "<left-rceil-2>" (math-bracket-open "<rceil>" "<lceil>" 'default))
+  (symbol "<left-llbracket-2>"
+          (math-bracket-open "<llbracket>" "<rrbracket>" 'default))
+  (symbol "<left-rrbracket-2>"
+          (math-bracket-open "<rrbracket>" "<llbracket>" 'default))
+  (symbol "<left-|-4>" (math-bracket-open "|" "|" 'default))
+  (symbol "<left-||-4>" (math-bracket-open "<||>" "<||>" 'default))
+  (symbol "<left-/-2>" (math-bracket-open "/" "\\" 'default))
+  (symbol "<left-\\-2>" (math-bracket-open "\\" "/" 'default))
+  (symbol "<left-.-2>" (math-bracket-open "." "." 'default)))
+
 (menu-bind left-delimiter-menu
   (symbol "<left-(-2>" (math-bracket-open "(" ")" #t))
   (symbol "<left-)-2>" (math-bracket-open ")" "(" #t))
@@ -149,16 +236,18 @@
   (symbol "<left-]-2>" (math-bracket-open "]" "[" #t))
   (symbol "<left-{-2>" (math-bracket-open "{" "}" #t))
   (symbol "<left-}-2>" (math-bracket-open "}" "{" #t))
-  (symbol "<left-langle-2>" (math-bracket-open "langle" "rangle" #t))
-  (symbol "<left-rangle-2>" (math-bracket-open "rangle" "langle" #t))
-  (symbol "<left-lfloor-2>" (math-bracket-open "lfloor" "rfloor" #t))
-  (symbol "<left-rfloor-2>" (math-bracket-open "rfloor" "lfloor" #t))
-  (symbol "<left-lceil-2>" (math-bracket-open "lceil" "rceil" #t))
-  (symbol "<left-rceil-2>" (math-bracket-open "rceil" "lceil" #t))
-  (symbol "<left-llbracket-2>" (math-bracket-open "llbracket" "rrbracket" #t))
-  (symbol "<left-rrbracket-2>" (math-bracket-open "rrbracket" "llbracket" #t))
+  (symbol "<left-langle-2>" (math-bracket-open "<langle>" "<rangle>" #t))
+  (symbol "<left-rangle-2>" (math-bracket-open "<rangle>" "<langle>" #t))
+  (symbol "<left-lfloor-2>" (math-bracket-open "<lfloor>" "<rfloor>" #t))
+  (symbol "<left-rfloor-2>" (math-bracket-open "<rfloor>" "<lfloor>" #t))
+  (symbol "<left-lceil-2>" (math-bracket-open "<lceil>" "<rceil>" #t))
+  (symbol "<left-rceil-2>" (math-bracket-open "<rceil>" "<lceil>" #t))
+  (symbol "<left-llbracket-2>"
+          (math-bracket-open "<llbracket>" "<rrbracket>" #t))
+  (symbol "<left-rrbracket-2>"
+          (math-bracket-open "<rrbracket>" "<llbracket>" #t))
   (symbol "<left-|-4>" (math-bracket-open "|" "|" #t))
-  (symbol "<left-||-4>" (math-bracket-open "||" "||" #t))
+  (symbol "<left-||-4>" (math-bracket-open "<||>" "<||>" #t))
   (symbol "<left-/-2>" (math-bracket-open "/" "\\" #t))
   (symbol "<left-\\-2>" (math-bracket-open "\\" "/" #t))
   (symbol "<left-.-2>" (math-bracket-open "." "." #t)))
@@ -170,16 +259,16 @@
   (symbol "<mid-]-2>" (math-separator "]" #t))
   (symbol "<mid-{-2>" (math-separator "{" #t))
   (symbol "<mid-}-2>" (math-separator "}" #t))
-  (symbol "<mid-langle-2>" (math-separator "langle" #t))
-  (symbol "<mid-rangle-2>" (math-separator "rangle" #t))
-  (symbol "<mid-lfloor-2>" (math-separator "lfloor" #t))
-  (symbol "<mid-rfloor-2>" (math-separator "rfloor" #t))
-  (symbol "<mid-lceil-2>" (math-separator "lceil" #t))
-  (symbol "<mid-rceil-2>" (math-separator "rceil" #t))
-  (symbol "<mid-llbracket-2>" (math-separator "llbracket" #t))
-  (symbol "<mid-rrbracket-2>" (math-separator "rrbracket" #t))
+  (symbol "<mid-langle-2>" (math-separator "<langle>" #t))
+  (symbol "<mid-rangle-2>" (math-separator "<rangle>" #t))
+  (symbol "<mid-lfloor-2>" (math-separator "<lfloor>" #t))
+  (symbol "<mid-rfloor-2>" (math-separator "<rfloor>" #t))
+  (symbol "<mid-lceil-2>" (math-separator "<lceil>" #t))
+  (symbol "<mid-rceil-2>" (math-separator "<rceil>" #t))
+  (symbol "<mid-llbracket-2>" (math-separator "<llbracket>" #t))
+  (symbol "<mid-rrbracket-2>" (math-separator "<rrbracket>" #t))
   (symbol "<mid-|-4>" (math-separator "|" #t))
-  (symbol "<mid-||-4>" (math-separator "||" #t))
+  (symbol "<mid-||-4>" (math-separator "<||>" #t))
   (symbol "<mid-/-2>" (math-separator "/" #t))
   (symbol "<mid-\\-2>" (math-separator "\\" #t)))
 
@@ -190,18 +279,18 @@
   (symbol "<right-]-2>" (math-bracket-close "]" "[" #t))
   (symbol "<right-{-2>" (math-bracket-close "{" "}" #t))
   (symbol "<right-}-2>" (math-bracket-close "}" "{" #t))
-  (symbol "<right-langle-2>" (math-bracket-close "langle" "rangle" #t))
-  (symbol "<right-rangle-2>" (math-bracket-close "rangle" "langle" #t))
-  (symbol "<right-lfloor-2>" (math-bracket-close "lfloor" "rfloor" #t))
-  (symbol "<right-rfloor-2>" (math-bracket-close "rfloor" "lfloor" #t))
-  (symbol "<right-lceil-2>" (math-bracket-close "lceil" "rceil" #t))
-  (symbol "<right-rceil-2>" (math-bracket-close "rceil" "lceil" #t))
+  (symbol "<right-langle-2>" (math-bracket-close "<langle>" "<rangle>" #t))
+  (symbol "<right-rangle-2>" (math-bracket-close "<rangle>" "<langle>" #t))
+  (symbol "<right-lfloor-2>" (math-bracket-close "<lfloor>" "<rfloor>" #t))
+  (symbol "<right-rfloor-2>" (math-bracket-close "<rfloor>" "<lfloor>" #t))
+  (symbol "<right-lceil-2>" (math-bracket-close "<lceil>" "<rceil>" #t))
+  (symbol "<right-rceil-2>" (math-bracket-close "<rceil>" "<lceil>" #t))
   (symbol "<right-llbracket-2>"
-	  (math-bracket-close "llbracket" "rrbracket" #t))
+	  (math-bracket-close "<llbracket>" "<rrbracket>" #t))
   (symbol "<right-rrbracket-2>"
-	  (math-bracket-close "rrbracket" "llbracket" #t))
+	  (math-bracket-close "<rrbracket>" "<llbracket>" #t))
   (symbol "<right-|-4>" (math-bracket-close "|" "|" #t))
-  (symbol "<right-||-4>" (math-bracket-close "||" "||" #t))
+  (symbol "<right-||-4>" (math-bracket-close "<||>" "<||>" #t))
   (symbol "<right-/-2>" (math-bracket-close "/" "\\" #t))
   (symbol "<right-\\-2>" (math-bracket-close "\\" "/" #t))
   (symbol "<right-.-2>" (math-bracket-close "." "." #t)))
@@ -884,36 +973,43 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (menu-bind math-icons
-  ((balloon (icon "tm_fraction.xpm") "Insert a fraction")
-   (make-fraction))
-  ((balloon (icon "tm_sqrt.xpm") "Insert a square root")
-   (make-sqrt))
-  ((balloon (icon "tm_root.xpm") "Insert an n-th root")
-   (make-var-sqrt))
-  ((balloon (icon "tm_sub.xpm") "Make a subscript")
-   (make-script #f #t))
-  ((balloon (icon "tm_sup.xpm") "Make a superscript")
-   (make-script #t #t))
-  (=> (balloon (icon "tm_hat.xpm") "Insert an accent")
-      ((icon "tm_hat.xpm") "A-^" (make-wide "^"))
-      ((icon "tm_tilda.xpm") "A-~" (make-wide "~"))
-      ((icon "tm_bar.xpm") "A-B" (make-wide "<bar>"))
-      ((icon "tm_vect.xpm") "A-V" (make-wide "<vect>"))
-      ((icon "tm_check.xpm") "A-C" (make-wide "<check>"))
-      ((icon "tm_breve.xpm") "A-U" (make-wide "<breve>"))
-      ((icon "tm_dot.xpm") "A-." (make-wide "<dot>"))
-      ((icon "tm_ddot.xpm") "A-\"" (make-wide "<ddot>"))
-      ((icon "tm_acute.xpm") "A-'" (make-wide "<acute>"))
-      ((icon "tm_grave.xpm") "A-`" (make-wide "<grave>")))
-  |
+  (=> (balloon (icon "tm_fraction.xpm") "Insert a fraction")
+      ("Standard fraction" (make-fraction))
+      ("Small inline fraction" (make 'tfrac))
+      ("Large displayed fraction" (make 'dfrac))
+      ("Slash" (make 'frac*)))
+  (=> (balloon (icon "tm_root.xpm") "Insert a root")
+      ("Square root" (make-sqrt))
+      ("Multiple root" (make-var-sqrt)))
+  (=> (balloon (icon "tm_subsup.xpm") "Insert a script")
+      ("Subscript" (make-script #f #t))
+      ("Superscript" (make-script #t #t))
+      ("Left subscript" (make-script #f #f))
+      ("Left superscript" (make-script #t #f))
+      ("Subscript below" (make-below))
+      ("Superscript above" (make-above)))
+  /
   (=> (balloon (icon "tm_bigop.xpm") "Insert a big operator")
       (tile 8 (link big-operator-menu)))
-  (=> (balloon (icon "tm_bigleft.xpm") "Insert a large left delimiter")
-      (tile 8 (link left-delimiter-menu)))
-  (=> (balloon (icon "tm_bigsep.xpm") "Insert a large separator")
-      (tile 8 (link middle-delimiter-menu)))
-  (=> (balloon (icon "tm_bigright.xpm") "Insert a large right delimiter")
-      (tile 8 (link right-delimiter-menu)))
+  (=> (balloon (icon "tm_bigaround.xpm") "Insert large delimiters")
+      (tile 8 (link large-delimiter-menu))
+      ---
+      (-> "Opening" (tile 8 (link left-delimiter-menu)))
+      (-> "Middle" (tile 8 (link middle-delimiter-menu)))
+      (-> "Closing" (tile 8 (link right-delimiter-menu))))
+  (=> (balloon (icon "tm_wide.xpm") "Insert an accent")
+      (tile 6
+            ((icon "tm_hat.xpm") (make-wide "^"))
+            ((icon "tm_tilda.xpm") (make-wide "~"))
+            ((icon "tm_bar.xpm") (make-wide "<bar>"))
+            ((icon "tm_vect.xpm") (make-wide "<vect>"))
+            ((icon "tm_check.xpm") (make-wide "<check>"))
+            ((icon "tm_breve.xpm") (make-wide "<breve>"))
+            ((icon "tm_dot.xpm") (make-wide "<dot>"))
+            ((icon "tm_ddot.xpm") (make-wide "<ddot>"))
+            ((icon "tm_acute.xpm") (make-wide "<acute>"))
+            ((icon "tm_grave.xpm") (make-wide "<grave>"))))
+  /
   (=> (balloon (icon "tm_binop.xpm") "Insert a binary operation")
       (tile 8 (link binary-operation-menu)))
   (=> (balloon (icon "tm_binrel.xpm") "Insert a binary relation")
@@ -933,4 +1029,193 @@
   (=> (balloon (icon "tm_miscsymb.xpm") "Insert a miscellaneous symbol")
       (tile 8 (link miscellaneous-symbol-menu))
       ---
-      (tile 6 (link dots-menu))))
+      (tile 6 (link dots-menu)))
+  /
+  (=> (balloon (icon "tm_greek.xpm") "Insert a greek character")
+      (tile 8 (link lower-greek-menu))
+      ---
+      (tile 8 (link upper-greek-menu)))
+  (=> (balloon (icon "tm_mathbold.xpm")
+	       "Insert a bold character")
+      (tile 15 (link bold-num-menu))
+      ---
+      (tile 13 (link bold-alpha-menu))
+      ---
+      (tile 15 (link bold-greek-menu)))
+  (=> (balloon (icon "tm_cal.xpm")
+	       "Insert a calligraphic character")
+      (tile 13 (link cal-menu)))
+  (=> (balloon (icon "tm_frak.xpm")
+	       "Insert a fraktur character")
+      (tile 13 (link frak-menu)))
+  (=> (balloon (icon "tm_bbb.xpm")
+	       "Insert a blackboard bold character")
+      (tile 13 (link bbb-menu)))
+  (=> (balloon (icon "tm_op.xpm") "Insert a textual operator")
+      (link textual-operator-menu))
+  (link math-format-icons)
+  (=> (balloon (icon "tm_math_preferences.xpm")
+               "Preferences for editing mathematical formulas")
+      (group "Keyboard")
+      ("Enforce brackets to match" (toggle-matching-brackets))
+      ("Use extensible brackets" (toggle-preference "use large brackets"))
+      ---
+      (group "Context aids")
+      (link context-preferences-menu)
+      ---
+      (group "Semantics")
+      (link semantic-math-preferences-menu))
+  (if (== (get-preference "semantic editing") "on")
+      (=> (balloon (icon "tm_math_syntax.xpm")
+                   "Specify semantics of a symbol or formula")
+          (link semantic-annotation-menu)))
+  (link texmacs-insert-icons))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; Math focus menus
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(tm-define (focus-tag-name l)
+  (:require (== l 'math))
+  "Formula")
+
+(tm-define (focus-tag-name l)
+  (:require (in? l '(equation equation*)))
+  "Equation")
+
+(tm-define (focus-variants-of t)
+  (:require (tree-in? t '(math equation equation*)))
+  '(formula equation))
+
+(tm-menu (focus-variant-menu t)
+  (:require (tree-in? t '(math equation equation*)))
+  ("Formula" (variant-formula t))
+  ("Equation" (variant-equation t)))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; Script focus menus
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(tm-define (focus-can-insert-remove? t)
+  (:require (script-context? t))
+  #t)
+
+(tm-define (focus-variants-of t)
+  (:require (tree-in? t '(lsub lsup)))
+  '(lsub lsup))
+
+(tm-define (focus-variants-of t)
+  (:require (tree-in? t '(rsub rsup)))
+  '(rsub rsup))
+
+(tm-menu (focus-variant-menu t)
+  (:require (tree-in? t '(lsub lsup)))
+  (when (script-only-script? t)
+    ("Left subscript" (variant-set (focus-tree) 'lsub))
+    ("Left superscript" (variant-set (focus-tree) 'lsup))))
+
+(tm-menu (focus-variant-menu t)
+  (:require (tree-in? t '(rsub rsup)))
+  (when (script-only-script? t)
+    ("Subscript" (variant-set (focus-tree) 'rsub))
+    ("Superscript" (variant-set (focus-tree) 'rsup))))
+
+(tm-menu (focus-insert-menu t)
+  (:require (script-context? t))
+  (assuming (tree-in? t '(lsub rsub))
+    (when (script-only-script? t)
+      ("Insert superscript" (structured-insert-up))))
+  (assuming (tree-in? t '(lsup rsup))
+    (when (script-only-script? t)
+      ("Insert subscript" (structured-insert-down)))))
+
+(tm-menu (focus-insert-icons t)
+  (:require (script-context? t))
+  (assuming (tree-in? t '(lsub rsub))
+    (when (script-only-script? t)
+      ((balloon (icon "tm_insert_up.xpm") "Insert superscript")
+       (structured-insert-up))))
+  (assuming (tree-in? t '(lsup rsup))
+    (when (script-only-script? t)
+      ((balloon (icon "tm_insert_down.xpm") "Insert subscript")
+       (structured-insert-down)))))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; Root focus menus
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(tm-define (focus-can-insert-remove? t)
+  (:require (tree-is? t 'sqrt))
+  #f)
+
+(tm-menu (focus-toggle-menu t)
+  (:require (tree-is? t 'sqrt))
+  ((check "Multiple root" "v"
+          (== (tree-arity (focus-tree)) 2))
+   (sqrt-toggle (focus-tree))))
+
+(tm-menu (focus-toggle-icons t)
+  (:require (tree-is? t 'sqrt))
+  ((check (balloon (icon "tm_root_index.xpm") "Multiple root") "v"
+          (== (tree-arity (focus-tree)) 2))
+   (sqrt-toggle (focus-tree))))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; Wide accent focus menus
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(tm-define (focus-tag-name l)
+  (:require (in? l '(wide wide*)))
+  "Wide")
+
+(tm-define (focus-variants-of t)
+  (:require (tree-in? t '(wide wide*)))
+  '(wide))
+
+(tm-menu (focus-toggle-menu t)
+  (:require (tree-in? t '(wide wide*)))
+  ((check "Accent below" "v"
+	  (alternate-second? (focus-tree)))
+   (alternate-toggle (focus-tree))))
+
+(tm-menu (focus-toggle-icons t)
+  (:require (tree-in? t '(wide wide*)))
+  ((check (balloon (icon "tm_wide_under.xpm") "Accent below") "v"
+	  (alternate-second? (focus-tree)))
+   (alternate-toggle (focus-tree))))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; Around focus menus
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(tm-define (focus-tag-name l)
+  (:require (in? l '(around around*)))
+  "Around")
+
+(tm-define (focus-variants-of t)
+  (:require (tree-in? t '(around around*)))
+  '(around))
+
+(tm-menu (focus-toggle-menu t)
+  (:require (tree-in? t '(around around*)))
+  ((check "Large brackets" "v"
+	  (alternate-second? (focus-tree)))
+   (alternate-toggle (focus-tree))))
+
+(tm-menu (focus-toggle-icons t)
+  (:require (tree-in? t '(around around*)))
+  ((check (balloon (icon "tm_large_around.xpm") "Large brackets") "v"
+	  (alternate-second? (focus-tree)))
+   (alternate-toggle (focus-tree))))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; Eqnarray focus menus
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(tm-define (focus-tag-name l)
+  (:require (in? l '(eqnarray eqnarray*)))
+  "Equations")
+
+(tm-define (focus-variants-of t)
+  (:require (tree-in? t '(eqnarray eqnarray*)))
+  '(eqnarray*))
