@@ -151,7 +151,7 @@
       (link source-quote-menu))
   (=> (balloon (icon "tm_ctrl_flow.xpm") "Insert a control flow instruction")
       (link source-flow-menu))
-  |
+  /
   (=> (balloon (icon "tm_three.xpm") "Insert a numerical operation")
       (link source-arithmetic-menu))
   (=> (balloon (icon "tm_textual.xpm") "Insert a textual operation")
@@ -160,10 +160,11 @@
       (link source-tuple-menu))
   (=> (balloon (icon "tm_equal.xpm") "Insert a condition")
       (link source-condition-menu))
-  |
+  /
   ((balloon (icon "tm_activate.xpm") "Activate")
    (make-mod-active 'active*))
   ((balloon (icon "tm_stretch.xpm") "Stretch")
    (make-style-with "src-compact" "none"))
   ((balloon (icon "tm_compact.xpm") "Compactify")
-   (make-style-with "src-compact" "all")))
+   (make-style-with "src-compact" "all"))
+  (link text-format-icons))
