@@ -19,28 +19,30 @@
 ;; Dynamic menus
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(define (extern-clipboard-item fm name action)
-  (with routine (string->symbol (string-append "clipboard-" action))
-    `(,name (,routine ,fm "primary"))))
+(tm-menu (clipboard-extern-menu cvs fun)
+  (with l (cvs "texmacs-snippet" "-snippet" #t)
+    (for (fm l)
+      (with name (format-get-name fm)
+        ((eval name) (fun fm "primary"))))))
 
-(define-macro (extern-clipboard-menu-promise action)
-  (define (item fm name) (extern-clipboard-item fm name action))
-  (with routine (if (== action "paste-import")
-		    converter-to-menu converter-from-menu)
-    `(menu-dynamic ,@(routine "texmacs-snippet" "-snippet" #t item))))
+(tm-define (clipboard-copy-export-menu)
+  (clipboard-extern-menu converters-from-special clipboard-copy-export))
+(tm-define (clipboard-cut-export-menu)
+  (clipboard-extern-menu converters-from-special clipboard-cut-export))
+(tm-define (clipboard-paste-import-menu)
+  (clipboard-extern-menu converters-to-special clipboard-paste-import))
 
-(define (redo-item i)
-  (list `(concat "Branch " ,(number->string (+ i 1)))
-	(lambda () (redo i))))
-
-(tm-define (redo-menu)
-  (menu-dynamic
-    ,@(map redo-item (.. 0 (redo-possibilities)))))
+(tm-menu (redo-menu)
+  (for (i (.. 0 (redo-possibilities)))
+    ((eval `(concat "Branch " ,(number->string (+ i 1)))) (redo i))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; The Edit menu
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
+(tm-property (search-start forward?) (:interactive #t))
+(tm-property (spell-start) (:interactive #t))
+
 (menu-bind edit-menu
   (when (> (undo-possibilities) 0)
     ("Undo" (undo 0)))
@@ -59,14 +61,18 @@
   (if (detailed-menus?)
       ("Clear" (clipboard-clear "primary")))
   ---
-  ("Search" ... (search-start #t))
+  ("Search" (search-start #t))
   ("Replace" (interactive replace-start-forward))
-  ("Spell" ... (spell-start))
+
+  (if (not (in-math?))
+      ("Spell" (spell-start)))
+  (if (in-math?)
+      (=> "Correct" (link math-correct-menu)))
   (if (detailed-menus?)
       ---
       (when (selection-active-any?)
 	(-> "Copy to"
-	    (promise (extern-clipboard-menu-promise "copy-export"))
+	    (link clipboard-copy-export-menu)
 	    ---
 	    ("Primary" (clipboard-copy "primary"))
 	    ("Secondary" (clipboard-copy "secondary"))
@@ -76,7 +82,7 @@
 	    ---
 	    ("Other" (interactive clipboard-copy)))
 	(-> "Cut to"
-	    (promise (extern-clipboard-menu-promise "cut-export"))
+	    (link clipboard-cut-export-menu)
 	    ---
 	    ("Primary" (clipboard-cut "primary"))
 	    ("Secondary" (clipboard-cut "secondary"))
@@ -86,7 +92,7 @@
 	    ---
 	    ("Other" (interactive clipboard-cut))))
       (-> "Paste from"
-	  (promise (extern-clipboard-menu-promise "paste-import"))
+          (link clipboard-paste-import-menu)
 	  ---
 	  ("Primary" (clipboard-paste "primary"))
 	  ("Secondary" (clipboard-paste "secondary"))
