@@ -27,6 +27,7 @@
 (tm-define (remove-package-menu) (get-remove-package-menu))
 (menu-bind bookmarks-menu)
 (menu-bind test-menu)
+(menu-bind help-icons (if (in-session?) (link session-help-icons)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; The TeXmacs main menu
@@ -35,17 +36,18 @@
 (menu-bind texmacs-menu
   (=> "File" (link file-menu))
   (=> "Edit" (link edit-menu))
-  (if (not (in-graphics?)) (=> "Insert" (link insert-menu)))
-  (if (in-source?) (=> "Source" (link source-menu)))
-  (if (in-text?) (=> "Text" (link text-menu)))
-  (if (in-math?) (=> "Mathematics" (link math-menu)))
-  (if (in-session?) (=> "Session" (link session-menu)))
-  (if (in-graphics?) (=> "Graphics" (link graphics-menu)))
-  (if (in-table?) (link vertical-table-cell-menu))
-  (if (with-linking-tool?)
-      (=> "Link" (link link-menu)))
-  (link texmacs-extra-menu)
-  (if (not (in-graphics?)) (=> "Format" (link format-menu)))
+  (if (in-graphics?)
+      (=> "Insert" (link graphics-insert-menu))
+      (=> "Focus" (link graphics-focus-menu)))
+  (if (not (in-graphics?))
+      (=> "Insert" (link insert-menu))
+      (if (or (in-source?) (with-source-tool?))
+	  (=> "Source" (link source-menu)))
+      (if (with-linking-tool?)
+	  (=> "Link" (link link-menu)))
+      (link texmacs-extra-menu)
+      (=> "Focus" (link focus-menu))
+      (=> "Format" (link format-menu)))
   (=> "Document" (link document-menu))
   (if (and (not (project-attached?))
 	   (== (get-init-tree "sectional-short-style") (tree 'macro "false")))
@@ -64,23 +66,25 @@
       (=> "Test" (link test-menu)))
   (=> "Help" (link help-menu)))
 
-
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; The TeXmacs popup menu
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (menu-bind texmacs-popup-menu
+  (link focus-menu)
+  ---
   (-> "File" (link file-menu))
   (-> "Edit" (link edit-menu))
-  (if (not (in-graphics?)) (-> "Insert" (link insert-menu)))
-  (if (in-source?) (-> "Source" (link source-menu)))
-  (if (in-text?) (-> "Text" (link text-menu)))
-  (if (in-math?) (-> "Mathematics" (link math-menu)))
-  (if (in-graphics?) (-> "Graphics" (link graphics-menu)))
-  (if (in-session?) (-> "Session" (link session-menu)))
-  (if (in-table?) (link horizontal-table-cell-menu))
-  (if (with-linking-tool?) (-> "Link" (link link-menu)))
-  (if (not (in-graphics?)) (-> "Format" (link format-menu)))
+  (if (in-graphics?)
+      (-> "Insert" (link graphics-insert-menu))
+      (-> "Focus" (link graphics-focus-menu)))
+  (if (not (in-graphics?))
+      (-> "Insert" (link insert-menu))
+      (if (or (in-source?) (with-source-tool?))
+	  (-> "Source" (link source-menu)))
+      (if (with-linking-tool?)
+	  (-> "Link" (link link-menu)))
+      (-> "Format" (link format-menu)))
   (-> "Document" (link document-menu))
   (if (== (get-init-tree "sectional-short-style") (tree 'macro "false"))
       (-> "Part" (link document-part-menu)))
@@ -90,8 +94,8 @@
   (-> "Go" (link go-menu))
   (if (detailed-menus?) (-> "Tools" (link tools-menu)))
   (if (with-remote-connections?) (-> "Remote" (link remote-menu)))
-  (if (with-debugging-tool?) (=> "Debug" (link debug-menu)))
-  (if (nnull? (test-menu)) (=> "Test" (link test-menu)))
+  (if (with-debugging-tool?) (-> "Debug" (link debug-menu)))
+  (if (nnull? (test-menu)) (-> "Test" (link test-menu)))
   ---
   (-> "Help" (link help-menu)))
 
@@ -104,10 +108,13 @@
       (link new-file-menu))
   (=> (balloon (icon "tm_open.xpm") "Load a file") (link load-menu))
   (=> (balloon (icon "tm_save.xpm") "Save this buffer") (link save-menu))
-  (=> (balloon (icon "tm_print.xpm") "Print") (link print-menu))
+  (if (experimental-qt-gui?)
+      ((balloon (icon "tm_print.xpm") "Print") (interactive-print-buffer)))
+  (if (not (experimental-qt-gui?))
+      (=> (balloon (icon "tm_print.xpm") "Print") (link print-menu)))
   (if (detailed-menus?)
-      (=> (balloon (icon "tm_style.xpm") "Select a document style")
-	  (link document-style-menu))
+      ;;(=> (balloon (icon "tm_style.xpm") "Select a document style")
+      ;;(link document-style-menu))
       (=> (balloon (icon "tm_language.xpm") "Select a language")
 	  (link global-language-menu)))
   (=> (balloon (icon "tm_cancel.xpm") "Close") (link close-menu))
@@ -125,30 +132,14 @@
        (search-button-next)))
   ((balloon (icon "tm_replace.xpm") "Query replace")
    (interactive replace-start-forward))
-  ((balloon (icon "tm_spell.xpm") "Check text for spelling errors")
-   (spell-start))
+  (if (not (in-math?))
+      ((balloon (icon "tm_spell.xpm") "Check text for spelling errors")
+       (spell-start)))
+  (if (in-math?)
+      (=> (balloon (icon "tm_spell.xpm") "Correct mathematical formulas")
+          (link math-correct-menu)))
   ((balloon (icon "tm_undo.xpm") "Undo last changes") (undo 0))
   ((balloon (icon "tm_redo.xpm") "Redo undone changes") (redo 0))
-  (if (not (in-graphics?))
-      |
-      (=> (balloon (icon "tm_table.xpm") "Insert a table")
-	  (link insert-table-menu))
-      (=> (balloon (icon "tm_image.xpm") "Insert a picture")
-	  (link insert-image-menu))
-      (=> (balloon (icon "tm_link.xpm") "Insert a link")
-	  (link insert-link-menu))
-      (if (detailed-menus?)
-	  (if (style-has? "std-fold-dtd")
-	      (=> (balloon (icon "tm_switch.xpm") "Switching and folding")
-		  (link insert-fold-menu)))
-	  (=> (balloon (icon "tm_animate.xpm") "Animation")
-	      (link insert-animation-menu)))
-      (=> (balloon (icon "tm_math.xpm") "Insert mathematics")
-	  (link insert-math-menu))
-      (if (and (style-has? "program-dtd") (detailed-menus?))
-	  (=> (balloon (icon "tm_shell.xpm")
-		       "Start an interactive session")
-	      (link insert-session-menu))))
   |
   ((balloon (icon "tm_back.xpm") "Browse back")
    (cursor-history-backward))
@@ -158,18 +149,13 @@
    (cursor-history-forward)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;; The context dependent icon bar
+;; The mode dependent icon bar
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(menu-bind texmacs-context-icons
+(menu-bind texmacs-mode-icons
   (if (in-source?) (link source-icons))
   (if (in-text?) (link text-icons))
-  (if (in-session?) (link session-icons))
   (if (in-math?) (link math-icons))
+  (if (in-prog?) (link prog-icons))
   (if (in-graphics?) (link graphics-icons))
-  (if (not (in-graphics?)) |)
-  (if (or (in-source?) (in-text?)) (link text-format-icons))
-  (if (in-math?) (link math-format-icons))
-  (if (in-prog?) (link prog-format-icons))
-  (if (in-table?) (link table-icons))
   (link help-icons))
