@@ -5,10 +5,9 @@
 ;; DESCRIPTION : routines for printing documents
 ;; COPYRIGHT   : (C) 2001  Joris van der Hoeven
 ;;
-;; This software falls under the GNU general public license and comes WITHOUT
-;; ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for details.
-;; If you don't have this file, write to the Free Software Foundation, Inc.,
-;; 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+;; This software falls under the GNU general public license version 3 or later.
+;; It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+;; in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ;;
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
@@ -23,17 +22,31 @@
 ;; or else default to "a4"
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
+(define supported-sizes
+  '("a0" "a1" "a2" "a3" "a4" "a5" "a6" "a7" "a8" "a9"
+    "b0" "b1" "b2" "b3" "b4" "b5" "b6" "b7" "b8" "b9"
+    "archA" "archB" "archC" "archD" "archE"
+    "10x14" "11x17" "C5" "Comm10" "DL" "executive" "halfletter"
+    "halfexecutive" "ledger" "legal" "letter" "Monarch"
+    "csheet" "dsheet" "flsa" "flse" "folio"
+    "lecture note" "note" "quarto" "statement" "tabloid"
+    "user"))
+
+(define (get-default-paper-size-bis)
+  (with psize (getenv "PAPERSIZE")
+    (if (and psize (!= psize "")) psize
+        (with papersizefile (or (getenv "PAPERCONF") "/etc/papersize")
+          (and (access? papersizefile R_OK)
+               (with pps-port (open-input-file papersizefile)
+                 (with size (read-line pps-port)
+                   (close-input-port pps-port)
+                   size)))))))
+
+(tm-define (correct-paper-size s)
+  (if (and (string? s) (in? s supported-sizes)) s "a4"))
+
 (tm-define (get-default-paper-size)
-  (or (getenv "PAPERSIZE")
-      (let ((papersizefile (or (getenv "PAPERCONF") '"/etc/papersize")))
-	(if
-	 (access? papersizefile R_OK)
-	 (let ((pps-port (open-input-file papersizefile)))
-	   (let ((size (read-line pps-port)))
-	     (begin
-	       (close-input-port pps-port)
-	       size)))
-	 "a4"))))
+  (correct-paper-size (get-default-paper-size-bis)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Printing preferences
@@ -100,13 +113,15 @@
   (:argument first "First page")
   (:argument last "Last page"))
 
-(tm-define (preview-with-ghostview)
+(tm-define (preview-buffer)
   (print-to-file "$TEXMACS_HOME_PATH/system/tmp/preview.ps")
   (cond ((!= preview-command "default")
 	 (shell (string-append preview-command
 			       " $TEXMACS_HOME_PATH/system/tmp/preview.ps &")))
         ((os-win32?)
 	 (shell "__previewps__ $TEXMACS_HOME_PATH/system/tmp/preview.ps"))
+        ((os-macos?)
+         (shell "open $TEXMACS_HOME_PATH/system/tmp/preview.ps"))
         ((url-exists-in-path? "ggv")
 	 (shell "ggv $TEXMACS_HOME_PATH/system/tmp/preview.ps &"))
 	((url-exists-in-path? "ghostview")
@@ -120,5 +135,5 @@
 (tm-define (choose-file-and-print-page-selection start end)
   (:argument start "First page")
   (:argument end "Last page")
-  (choose-file (lambda (name) (print-pages-to-file name ,start ,end))
+  (choose-file (lambda (name) (print-pages-to-file name start end))
 	       "Print page selection to file" "postscript"))
