@@ -144,6 +144,8 @@
 		  ("Hiragino Kaku" (make-with "font" "kaku")))
 	      (if (font-exists-in-tt? "ipam")
 		  ("Ipa" (make-with "font" "ipa")))
+	      (if (font-exists-in-tt? "ttf-japanese-gothic")
+		  ("Japanese" (make-with "font" "ttf-japanese")))
 	      (if (font-exists-in-tt? "kochi-mincho")
 		  ("Kochi" (make-with "font" "kochi")))
 	      (if (font-exists-in-tt? "儷黑 Pro")
@@ -158,6 +160,8 @@
 		  ("MS Gothic" (make-with "font" "ms-gothic")))
 	      (if (font-exists-in-tt? "MS Mincho")
 		  ("MS Mincho" (make-with "font" "ms-mincho")))
+	      (if (font-exists-in-tt? "sazanami-gothic")
+		  ("Sazanami" (make-with "font" "sazanami")))
 	      (if (font-exists-in-tt? "simfang")
 		  ("SimFang" (make-with "font" "simfang")))
 	      (if (font-exists-in-tt? "simhei")
@@ -358,20 +362,38 @@
   (-> "Margins"
       ("Left margin" (make-interactive-line-with "par-left"))
       ("Right margin" (make-interactive-line-with "par-right"))
-      ("First indentation" (make-interactive-line-with "par-first")))
+      ("First indentation" (make-interactive-line-with "par-first"))
+      ---
+      (link indentation-menu))
   (-> "Spacing"
       ("Interline separation" (make-interactive-line-with "par-sep"))
       ("Interline space" (make-interactive-line-with "par-line-sep"))
-      ("Interparagraph space" (make-interactive-line-with "par-par-sep")))
+      ("Interparagraph space" (make-interactive-line-with "par-par-sep"))
+      ---
+      (link vertical-space-menu))
   (-> "Hyphenation"
       ("Normal" (make-line-with "par-hyphen" "normal"))
       ("Professional"
-       (make-line-with "par-hyphen" "professional")))
+       (make-line-with "par-hyphen" "professional"))
+      ---
+      (link line-break-menu))
   (-> "Number of columns"
       ("1" (make-line-with "par-columns" "1"))
       ("2" (make-line-with "par-columns" "2"))
       ("3" (make-line-with "par-columns" "3"))))
 
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; The Page menu
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(menu-bind page-menu
+  (-> "Header" (link page-header-menu))
+  (-> "Footer" (link page-footer-menu))
+  (-> "Numbering" (link page-numbering-menu))
+  (-> "Break" (link page-break-menu))
+  (if (and (style-has? "env-float-dtd") (detailed-menus?))
+      (-> "Insertion" (link page-insertion-menu))))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; The main Format menu in text mode
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -379,22 +401,56 @@
 (menu-bind text-format-menu
   (group "Font")
   (link text-font-menu)
+  (if (simple-menus?)
+      (-> "Color" (link color-menu)))
   (if (detailed-menus?)
       ---
-      (group "Content"))
-  (-> "Color" (link color-menu))
-  (if (detailed-menus?)
+      (group "Text")
+      (-> "Color" (link color-menu))
       (-> "Language" (link text-language-menu))
-      (-> "Scripts" (link local-supported-scripts-menu)))
+      (-> "Scripts" (link local-supported-scripts-menu))
+      (-> "Space" (link horizontal-space-menu))
+      (-> "Transform" (link transform-menu))
+      (-> "Specific" (link specific-menu)))
   ---
   (group "Paragraph")
-  (link paragraph-menu))
+  (link paragraph-menu)
+  ---
+  (group "Page")
+  (link page-menu))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Icons for modifying text properties
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (menu-bind text-format-icons
+  (if (style-has? "std-markup-dtd")
+      /)
+  (=> (balloon (icon "tm_parstyle.xpm") "Set paragraph mode")
+      ((balloon (icon "tm_left.xpm") "Align text to the left")
+       (make-line-with "par-mode" "left"))
+      ((balloon (icon "tm_center.xpm") "Center text")
+       (make-line-with "par-mode" "center"))
+      ((balloon (icon "tm_right.xpm") "Align text to the right")
+       (make-line-with "par-mode" "right"))
+      ((balloon (icon "tm_justify.xpm") "Justify text")
+       (make-line-with "par-mode" "justify")))
+  (=> (balloon (icon "tm_parindent.xpm") "Set paragraph margins")
+      ("Left margin" (make-interactive-line-with "par-left"))
+      ("Right margin" (make-interactive-line-with "par-right"))
+      ("First indentation" (make-interactive-line-with "par-first")))
+  (if (and (style-has? "env-float-dtd") (detailed-menus?))
+      (if (not (inside? 'float))
+	  (=> (balloon (icon "tm_pageins.xpm") "Make a page insertion")
+	      (link insert-page-insertion-menu)))
+      (if (inside? 'float)
+	  (=> (balloon (icon "tm_floatpos.xpm") "Position floating object")
+	      (link position-float-menu))))
+  ;;((balloon (icon "tm_footnote.xpm") "Insert a footnote") ())
+  ;;((balloon (icon "tm_margin.xpm") "Insert a marginal note") ())
+  ;;((balloon (icon "tm_floating.xpm") "Insert a floating object") ())
+  ;;((balloon (icon "tm_multicol.xpm") "Start multicolumn context") ())
+  /
   (if (and (style-has? "std-markup-dtd") (not (in-source?)))
       ;;((balloon
       ;;(text (roman rm bold right 12 600) "S")
@@ -421,6 +477,5 @@
        (make-with "font-family" "ss"))
       ((balloon (icon "tm_smallcaps.xpm") "Use small capitals")
        (make-with "font-shape" "small-caps")))
-  (if (not (in-graphics?))
-      (=> (balloon (icon "tm_color.xpm") "Select a foreground color")
-	  (link color-menu))))
+  (=> (balloon (icon "tm_color.xpm") "Select a foreground color")
+      (link color-menu)))
