@@ -12,48 +12,17 @@
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (texmacs-module (text text-menu)
-  (:use (text format-text-edit) (text std-text-edit)))
+  (:use (text format-text-edit)
+        (text std-text-edit)
+        (text tm-structure)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Document headers
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(menu-bind author-menu
-  ("Insert author" (make-doc-data-element 'doc-author-data))
-  ---
-  (when (inside? 'doc-author-data)
-	("Address" (make-author-data-element 'author-address))
-	("Email" (make-author-data-element 'author-email))
-	("Homepage" (make-author-data-element 'author-homepage))
-	("Note" (make-author-data-element 'author-note))))
-
 (menu-bind title-menu
   (when (not (inside? 'doc-data))
 	("Insert title" (make-doc-data)))
-  ---
-  (when (inside? 'doc-data)
-	("Subtitle" (make-doc-data-element 'doc-subtitle))
-	(-> "Author" (link author-menu))
-	(-> "Date"
-	    ("Default" (make-doc-data-element 'doc-date))
-	    ("Today"
-	     (begin (make-doc-data-element 'doc-date) (make-arity 'date 0))))
-	(-> "Note"
-	    ("General note" (make-doc-data-element 'doc-note))
-	    ("Written with TeXmacs" (begin (make-doc-data-element 'doc-note)
-					   (make 'with-TeXmacs-text))))
-	(-> "Hidden"
-	    (if (doc-data-disactivated?)
-		("Activate hidden" (doc-data-activate-all)))
-	    (if (not (doc-data-disactivated?))
-		("Show hidden" (doc-data-disactivate-all)))
-	    ---
-	    ("Running title" (make-doc-data-element 'doc-running-title))
-	    ("Running author" (make-doc-data-element 'doc-running-author))
-	    ("Keywords" (make-doc-data-element 'doc-keywords))
-	    ("A.M.S. subject classification"
-	     (make-doc-data-element 'doc-AMS-class))))
-  ---
   (when (and (not (inside? 'doc-data)) (not (inside? 'abstract)))
 	("Abstract" (make 'abstract))))
 
@@ -99,36 +68,69 @@
   ("Subparagraph" (make-section 'subparagraph)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;; Theorem like environments
+;; Enunciations, quotations and programs
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-(menu-bind environment-menu
-  (if (style-has? "header-exam-dtd")
-      ("Exercise" (make 'exercise))
-      ("Problem" (make 'problem)))
-  (if (not (style-has? "header-exam-dtd"))
-      (if (style-has? "env-theorem-dtd")
-	  ("Theorem" (make 'theorem))
-	  ("Proposition" (make 'proposition))
-	  ("Lemma" (make 'lemma))
-	  ("Corollary" (make 'corollary))
-	  ("Proof" (make 'proof))
-	  ("Axiom" (make 'axiom))
-	  ("Definition" (make 'definition))
-	  ("Notation" (make 'notation))
-	  ---
-	  ("Remark" (make 'remark))
-	  ("Note" (make 'note))
-	  ("Example" (make 'example))
-	  ("Warning" (make 'warning))
-	  ("Exercise" (make 'exercise))
-	  ("Problem" (make 'problem))
-	  ---)
-      ("Verbatim" (make 'verbatim))
-      ("Code" (make 'code))
-      ("Quote" (make 'quote-env))
-      ("Quotation" (make 'quotation))
-      ("Verse" (make 'verse))))
+(menu-bind enunciation-menu
+  (if (style-has? "env-theorem-dtd")
+      ("Theorem" (make 'theorem))
+      ("Proposition" (make 'proposition))
+      ("Lemma" (make 'lemma))
+      ("Corollary" (make 'corollary))
+      ("Proof" (make 'proof))
+      ---
+      ("Axiom" (make 'axiom))
+      ("Definition" (make 'definition))
+      ("Notation" (make 'notation))
+      ---
+      ("Remark" (make 'remark))
+      ("Note" (make 'note))
+      ("Example" (make 'example))
+      ("Warning" (make 'warning))
+      ---)
+  ("Exercise" (make 'exercise))
+  ("Problem" (make 'problem))
+  ("Solution" (make 'solution))
+  ("Question" (make 'question))
+  ("Answer" (make 'answer)))
+
+(menu-bind prominent-menu
+  ("Quote" (make 'quote-env))
+  ("Quotation" (make 'quotation))
+  ("Verse" (make 'verse))
+  ---
+  ("Padded" (make 'padded))
+  ("Underlined" (make 'underlined))
+  ("Lines around" (make 'bothlined))
+  ("Framed" (make 'framed))
+  (if (style-has? "ornaments-dtd")
+      ---
+      ("Manila paper" (make 'manila-paper))
+      ("Rough paper" (make 'rough-paper))
+      ("Ridged paper" (make 'ridged-paper))
+      ("Pine" (make 'pine))
+      ("Granite" (make 'granite))
+      ("Metal" (make 'metal))))
+
+(menu-bind code-menu
+  ("Algorithm" (make 'algorithm))
+  ("Pseudo code" (make 'pseudo-code))
+  ---
+  ("Indent" (make 'indent))
+  ("Tabbed" (make 'tabbed))
+  ---
+  (-> "Inline code"
+      ("C++" (make 'cpp))
+      ("Mathemagix" (make 'mmx))
+      ("Scheme" (make 'scm))
+      ("Shell" (make 'shell))
+      ("Verbatim" (make 'verbatim)))
+  (-> "Block of code"
+      ("C++" (make 'cpp-code))
+      ("Mathemagix" (make 'mmx-code))
+      ("Scheme" (make 'scm-code))
+      ("Shell" (make 'shell-code))
+      ("Verbatim" (make 'verbatim-code))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Tags
@@ -197,6 +199,28 @@
   ("Dashes" (make-tmlist 'description-dash))
   ("Long" (make-tmlist 'description-long)))
 
+(menu-bind list-menu
+  ("Itemize" (make-tmlist 'itemize))
+  ---
+  ("Bullets" (make-tmlist 'itemize-dot))
+  ("Dashes" (make-tmlist 'itemize-minus))
+  ("Arrows" (make-tmlist 'itemize-arrow))
+  ---
+  ("Enumerate" (make-tmlist 'enumerate))
+  ---
+  ("1, 2, 3, ..." (make-tmlist 'enumerate-numeric))
+  ("i, ii, iii, ..." (make-tmlist 'enumerate-roman))
+  ("I, II, III, ..." (make-tmlist 'enumerate-Roman))
+  ("a, b, c, ..." (make-tmlist 'enumerate-alpha))
+  ("A, B, C, ..." (make-tmlist 'enumerate-Alpha))
+  ---
+  ("Description" (make-tmlist 'description))
+  ---
+  ("Compact" (make-tmlist 'description-compact))
+  ("Aligned" (make-tmlist 'description-aligned))
+  ("Dashes" (make-tmlist 'description-dash))
+  ("Long" (make-tmlist 'description-long)))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Automatically generated content
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -229,32 +253,44 @@
   (if (and (style-has? "section-base-dtd")
 	   (not (style-has? "header-exam-dtd")))
       (-> "Section" (link section-menu)))
+  (if (or (style-has? "env-theorem-dtd")
+          (style-has? "header-exam-dtd"))
+      (-> "Enunciation" (link enunciation-menu)))
+  (if (style-has? "std-markup-dtd")
+      (-> "Prominent" (link prominent-menu)))
+  (if (style-has? "std-markup-dtd")
+      (-> "Program" (link code-menu)))
+  (if (style-has? "section-base-dtd")
+      (-> "Automatic" (link automatic-menu)))
+  (if (style-has? "std-list-dtd")
+      ---
+      (-> "Itemize" (link itemize-menu))
+      (-> "Enumerate" (link enumerate-menu))
+      (-> "Description" (link description-menu)))
+  ---
   (if (style-has? "std-markup-dtd")
-      (-> "Environment" (link environment-menu))
       (-> "Content tag" (link content-tag-menu))
       (-> "Size tag" (link size-tag-menu)))
   (-> "Presentation tag" (link presentation-tag-menu))
-  (if (style-has? "std-dtd")
-      ---)
-  (if (style-has? "std-list-dtd")
-      (-> "Itemize" (link itemize-menu))
-      (-> "Enumerate" (link enumerate-menu))
-	(-> "Description" (link description-menu))
-	(when (inside-list-tag?) ("New item" (make-item)))
-	---)
-  (if (style-has? "section-base-dtd")
-      (-> "Automatic" (link automatic-menu))))
+  ---
+  (-> "Mathematics" (link insert-math-menu))
+  (-> "Table" (link insert-table-menu))
+  (-> "Image" (link insert-image-menu))
+  (-> "Link" (link insert-link-menu))
+  (if (detailed-menus?)
+      (if (style-has? "std-fold-dtd")
+	  (-> "Fold" (link insert-fold-menu)))
+      (-> "Animation" (link insert-animation-menu)))
+  (if (and (style-has? "session-dtd") (detailed-menus?))
+      (-> "Session" (link insert-session-menu))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Icons for text mode
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (menu-bind text-icons
-  (if (and (style-has? "header-title-dtd")
-	   (not (style-has? "header-letter-dtd"))
-	   (not (style-has? "header-exam-dtd")))
-      (=> (balloon (icon "tm_title.xpm") "Enter a title")
-	  (link title-menu)))
+  ;;("Goedenmiddag" (display* "Hi there\n"))
+  ;;(input (display* answer "\n") "string" '("Hello" "Bonjour") "0.5w")
   (if (style-has? "header-letter-dtd")
       (=> (balloon (icon "tm_title.xpm") "Make a letter environment")
 	  (link letter-header-menu)))
@@ -268,45 +304,195 @@
 	   (not (style-has? "header-exam-dtd")))
       (=> (balloon (icon "tm_section.xpm") "Start a new section")
 	  (link section-menu)))
+  (if (or (style-has? "env-theorem-dtd")
+          (style-has? "header-exam-dtd"))
+      (=> (balloon (icon "tm_theorem.xpm") "Insert an enunciation")
+	  (link enunciation-menu)))
   (if (style-has? "std-markup-dtd")
-      (=> (balloon (icon "tm_theorem.xpm") "Insert an environment")
-	  (link environment-menu)))
-  (=> (balloon (icon "tm_parstyle.xpm") "Set paragraph mode")
-      ((balloon (icon "tm_left.xpm") "Align text to the left")
-       (make-line-with "par-mode" "left"))
-      ((balloon (icon "tm_center.xpm") "Center text")
-       (make-line-with "par-mode" "center"))
-      ((balloon (icon "tm_right.xpm") "Align text to the right")
-       (make-line-with "par-mode" "right"))
-      ((balloon (icon "tm_justify.xpm") "Justify text")
-       (make-line-with "par-mode" "justify")))
-  (=> (balloon (icon "tm_parindent.xpm") "Set paragraph margins")
-      ("Left margin" (make-interactive-line-with "par-left"))
-      ("Right margin" (make-interactive-line-with "par-right"))
-      ("First indentation" (make-interactive-line-with "par-first")))
-  (if (and (style-has? "env-float-dtd") (detailed-menus?))
-      (if (not (inside? 'float))
-	  (=> (balloon (icon "tm_pageins.xpm") "Make a page insertion")
-	      (link insert-page-insertion-menu)))
-      (if (inside? 'float)
-	  (=> (balloon (icon "tm_floatpos.xpm") "Position floating object")
-	      (link position-float-menu))))
-;;((balloon (icon "tm_footnote.xpm") "Insert a footnote") ())
-;;((balloon (icon "tm_margin.xpm") "Insert a marginal note") ())
-;;((balloon (icon "tm_floating.xpm") "Insert a floating object") ())
-;;((balloon (icon "tm_multicol.xpm") "Start multicolumn context") ())
+      (=> (balloon (icon "tm_prominent.xpm") "Insert a prominent piece of text")
+	  (link prominent-menu)))
+  (if (style-has? "std-markup-dtd")
+      (=> (balloon (icon "tm_program.xpm") "Insert a computer program")
+	  (link code-menu)))
+  (if (style-has? "std-list-dtd")
+      (=> (balloon (icon "tm_list.xpm") "Insert a list")
+	  (link list-menu)))
   (if (style-has? "section-base-dtd")
       (=> (balloon (icon "tm_index.xpm")
 		   "Insert automatically generated content")
 	  (link automatic-menu)))
-  (if (style-has? "std-list-dtd")
-      |
-      (=> (balloon (icon "tm_itemize.xpm") "Itemize")
-	  (link itemize-menu))
-      (=> (balloon (icon "tm_enumerate.xpm") "Enumerate")
-	  (link enumerate-menu))
-      (=> (balloon (icon "tm_description.xpm") "Description")
-	  (link description-menu))
-      (if (inside-list-tag?)
-	  ((balloon (icon "tm_item.xpm") "Insert a new item")
-	   (make-item)))))
+  (link text-format-icons)
+  (link texmacs-insert-icons))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; Focus menus for entering title information
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(tm-menu (focus-document-extra-menu t)
+  (:require (document-propose-title?))
+  ("Title" (make-doc-data)))
+
+(tm-menu (focus-document-extra-icons t)
+  (:require (document-propose-title?))
+  (minibar
+    ((balloon "Title" "Insert title") (make-doc-data))))
+
+(tm-menu (focus-document-extra-menu t)
+  (:require (document-propose-abstract?))
+  ("Abstract" (make 'abstract)))
+
+(tm-menu (focus-document-extra-icons t)
+  (:require (document-propose-abstract?))
+  (minibar
+    ((balloon "Abstract" "Insert abstract") (make 'abstract))))
+
+(tm-define (focus-can-move? t)
+  (:require (doc-title-context? t))
+  #f)
+
+(tm-menu (focus-title-menu)
+  ("Subtitle" (make-doc-data-element 'doc-subtitle))
+  ("Author" (make-doc-data-element 'doc-author-data))
+  ("Date" (make-doc-data-element 'doc-date))
+  ("Today"
+   (begin (make-doc-data-element 'doc-date) (make-arity 'date 0)))
+  ("Note" (make-doc-data-element 'doc-note))
+  ("TeXmacs notice" (begin (make-doc-data-element 'doc-note)
+                           (make 'with-TeXmacs-text))))
+
+(tm-menu (focus-title-hidden-menu)
+  ("Running title" (make-doc-data-element 'doc-running-title))
+  ("Running author" (make-doc-data-element 'doc-running-author))
+  ("Keywords" (make-doc-data-element 'doc-keywords))
+  ("M.S.C."
+   (make-doc-data-element 'doc-AMS-class)))
+
+(tm-menu (focus-title-icons)
+  (assuming (doc-data-has-hidden?)
+    ((check (balloon (icon "tm_show_hidden.xpm") "Show hidden") "v"
+	    (doc-data-disactivated?))
+     (doc-data-activate-toggle)))
+  (mini #t
+    (inert ("Title" (noop))))
+  (=> (balloon (icon "tm_add.xpm") "Add title information")
+      (link focus-title-menu)
+      (-> "Hidden" (link focus-title-hidden-menu))))
+
+(tm-menu (focus-ancestor-menu t)
+  (:require (doc-title-context? t))
+  (group "Title")
+  (link focus-title-menu)
+  ---
+  (group "Hidden")
+  (link focus-title-hidden-menu)
+  ---)
+
+(tm-menu (focus-ancestor-icons t)
+  (:require (doc-title-context? t))
+  (minibar (dynamic (focus-title-icons)))
+  (glue #f #f 5 0))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; Focus menus for entering authors
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(tm-define (focus-can-move? t)
+  (:require (doc-author-context? t))
+  #f)
+
+(tm-menu (focus-author-menu)
+  ("Address" (make-author-data-element 'author-address))
+  ("Email" (make-author-data-element 'author-email))
+  ("Homepage" (make-author-data-element 'author-homepage))
+  ("Note" (make-author-data-element 'author-note)))
+
+(tm-menu (focus-author-icons)
+  (mini #t
+    (inert ("Author" (noop))))
+  (=> (balloon (icon "tm_add.xpm") "Add author information")
+      (link focus-author-menu)))
+
+(tm-menu (focus-ancestor-menu t)
+  (:require (doc-author-context? t))
+  (group "Title")
+  (link focus-title-menu)
+  ---
+  (group "Hidden")
+  (link focus-title-hidden-menu)
+  ---
+  (group "Author")
+  (link focus-author-menu)
+  ---)
+
+(tm-menu (focus-ancestor-icons t)
+  (:require (doc-author-context? t))
+  (minibar (dynamic (focus-title-icons)))
+  (glue #f #f 5 0)
+  (minibar (dynamic (focus-author-icons)))
+  (glue #f #f 5 0))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; Focus menus for sections
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(tm-menu (focus-section-menu)
+  (for (s (tree-search-sections (buffer-tree)))
+    ((eval (tm/section-get-title-string s))
+     (when (and (tree->path s) (section-context? s))
+       (tree-go-to s 0 :end)))))
+
+(tm-menu (focus-document-extra-menu t)
+  (:require (previous-section))
+  (-> "Sections" (link focus-section-menu)))
+
+(tm-menu (focus-document-extra-icons t)
+  (:require (previous-section))
+  (mini #t
+    (=> (eval (tm/section-get-title-string (previous-section)))
+	(link focus-section-menu))))
+
+(tm-menu (focus-extra-menu t)
+  (:require (section-context? t))
+  ---
+  (-> "Go to section"
+      (link focus-section-menu)))
+
+(tm-menu (focus-extra-icons t)
+  (:require (section-context? t))
+  (glue #f #f 5 0)
+  (mini #t
+    (=> (eval (tm/section-get-title-string t))
+	(link focus-section-menu))))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;; Focus menus for algorithms
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+(tm-define (focus-tag-name l)
+  (:require (in? l (algorithm-tag-list)))
+  (with r (algorithm-root l)
+    (with s (upcase-first (tree-name (tree r)))
+      (string-replace s "-" " "))))
+
+(tm-menu (focus-toggle-menu t)
+  (:require (algorithm-context? t))
+  (when (not (algorithm-named? (focus-tree)))
+    ((check "Numbered" "v" (algorithm-numbered? (focus-tree)))
+     (algorithm-toggle-number (focus-tree))))
+  ((check "Named" "v" (algorithm-named? (focus-tree)))
+   (algorithm-toggle-name t))
+  ((check "Specified" "v" (algorithm-specified? (focus-tree)))
+   (algorithm-toggle-specification t)))
+
+(tm-menu (focus-toggle-icons t)
+  (:require (algorithm-context? t))
+  (when (not (algorithm-named? (focus-tree)))
+    ((check (balloon (icon "tm_numbered.xpm") "Toggle numbering") "v"
+	    (algorithm-numbered? (focus-tree)))
+     (algorithm-toggle-number (focus-tree))))
+  ((check (balloon (icon "tm_small_textual.xpm") "Toggle name") "v"
+	  (algorithm-named? (focus-tree)))
+   (algorithm-toggle-name t))
+  ((check (balloon (icon "tm_specified.xpm") "Toggle specification") "v"
+	  (algorithm-specified? (focus-tree)))
+   (algorithm-toggle-specification t)))
