@@ -41,6 +41,14 @@ check_tree (tree t) {
 }
 */
 
+static bool
+is_var_compound (tree t, string s, int n) {
+  return
+    is_compound (t, s, n) ||
+    (is_func (t, APPLY, n+1) && t[0] == s) ||
+    (is_func (t, EXPAND, n+1) && t[0] == s);
+}
+
 /******************************************************************************
 * Preprocess preamble
 ******************************************************************************/
@@ -50,40 +58,61 @@ filter_preamble (tree t) {
   int i, n=N(t);
   bool in_preamble= true;
   tree r (CONCAT);
+  tree doc (CONCAT);
   tree preamble (CONCAT);
   tree title_info (CONCAT);
 
   for (i=0; i<n; i++) {
     tree u= t[i];
-    if (in_preamble) {
+    if (is_tuple (u, "\\title") ||
+	     is_tuple (u, "\\author") ||
+	     is_tuple (u, "\\address"))
+      title_info << u;
+    else if (is_tuple (u, "\\thanks")) {
+      tree v= copy (u);
+      v[0]= "\\title-thanks";
+      title_info << v;
+    }
+    else if (is_tuple (u, "\\email")) {
+      tree v= copy (u);
+      v[0]= "\\title-email";
+      title_info << v;
+    }
+    else if (is_tuple (u, "\\urladdr"))
+      title_info << u;
+    else if (is_tuple (u, "\\keywords"))
+      title_info << u;
+    else if (is_tuple (u, "\\classification"))
+      title_info << u;
+    else if (is_tuple (u, "\\subjclass"))
+      title_info << u;
+    else if (in_preamble) {
       if (u == tuple ("\\begin-document")) {
 	r << u;
 	if (N(preamble) > 0)
 	  r << tuple ("\\begin-hide-preamble") << A(preamble)
 	    << tuple ("\\end-hide-preamble");
-	r << A(title_info);
 	in_preamble= false;
       }
       else if (is_tuple (u, "\\documentclass") ||
 	       is_tuple (u, "\\documentclass*") ||
 	       is_tuple (u, "\\documentstyle") ||
 	       is_tuple (u, "\\documentstyle*"))
-	r << u;
+	doc << u;
       else if (is_tuple (u, "\\def") ||
 	       is_tuple (u, "\\def*"))
 	preamble << u << "\n" << "\n";
-      else if (is_tuple (u, "\\newtheorem"))
+      else if (is_tuple (u, "\\newtheorem") ||
+	       is_tuple (u, "\\newtheorem*"))
 	preamble << u << "\n" << "\n";
       else if (is_tuple (u, "\\newenvironment") ||
 	       is_tuple (u, "\\newenvironment*"))
 	preamble << u << "\n" << "\n";
-      else if (is_tuple (u, "\\title") ||
-	       is_tuple (u, "\\author") ||
-	       is_tuple (u, "\\address"))
-	title_info << u;
     }
-    else r << u;
+    else doc << u;
   }
+  r << A(title_info);
+  r << A(doc);
   if (in_preamble) return t;
   return r;
 }
@@ -159,6 +188,7 @@ latex_symbol_to_tree (string s) {
       if (s == "\\hfill")  return tree (HTAB, "1fn");
       if (s == "\\hline")  return tree (APPLY, "hline");
       if (s == "\\appendix") { textm_appendices= true; return ""; }
+      if (s == "\\limits") return ""; // tree (FORMAT, "with limits");
       if (s == "\\nolimits") return ""; // temporarily
       if (s == "\\vert") return "|";
       if (s == "\\Vert") return "<||>";
@@ -171,6 +201,17 @@ latex_symbol_to_tree (string s) {
 	return compound ("table-of-contents", "toc", tree (DOCUMENT, ""));
       if (s == "\\bgroup") return "";
       if (s == "\\egroup") return "";
+      if (s == "\\colon") return ":";
+      if (s == "\\dotsc") return "<ldots>";
+      if (s == "\\dotsb") return "<cdots>";
+      if (s == "\\dotsm") return "<cdots>";
+      if (s == "\\dotsi") return "<cdots>";
+      if (s == "\\dotso") return "<ldots>";
+      if (s == "\\lvert") return "|";
+      if (s == "\\rvert") return "|";
+      if (s == "\\lVert") return "<||>";
+      if (s == "\\rVert") return "<||>";
+      if (s == "\\qed") return compound ("math", "<Box>");
     }
 
     if (latex_type (s) == "texmacs") {
@@ -364,8 +405,7 @@ latex_concat_to_tree (tree t, bool& new_flag) {
 	      string s= t[i-1][0]->label;
 	      if ((s[0] == '\\') && (latex_type (s) == "command") &&
 		  (s!="\\end-math") && (s!="\\end-displaymath"))
-		if ((arity(t[i-1])==1) || (s=="\\label"))
-		  continue;
+		if ((arity(t[i-1])==1) || (s=="\\label")) continue;
 	      if (starts (s, "\\begin-") &&
 		  (command_type["!verbatim"] != "true"))
 		continue;
@@ -551,7 +591,7 @@ latex_command_to_tree (tree t) {
     return tree (ASSIGN, var, f);
   }
 
-  if (is_tuple (t, "\\newtheorem", 2)) {
+  if (is_tuple (t, "\\newtheorem", 2) || is_tuple (t, "\\newtheorem*", 2)) {
     string var= l2e(t[1])->label;
     string val= l2e(t[2])->label;
     return compound ("new-theorem", var, val);
@@ -604,6 +644,10 @@ latex_command_to_tree (tree t) {
   if (is_tuple (t, "\\textit", 1)) return m2e (t, FONT_SHAPE, "italic");
   if (is_tuple (t, "\\textsl", 1)) return m2e (t, FONT_SHAPE, "slanted");
   if (is_tuple (t, "\\textsc", 1)) return m2e (t, FONT_SHAPE, "small-caps");
+  if (is_tuple (t, "\\tmrsub", 1)) return tree (RSUB, l2e (t[1]));
+  if (is_tuple (t, "\\tmrsup", 1)) return tree (RSUP, l2e (t[1]));
+  if (is_tuple (t, "\\textsubscript", 1)) return tree (RSUB, l2e (t[1]));
+  if (is_tuple (t, "\\textsuperscript", 1)) return tree (RSUP, l2e (t[1]));
   if (is_tuple (t, "\\tmtextrm", 1)) return m2e (t, FONT_FAMILY, "rm");
   if (is_tuple (t, "\\tmtexttt", 1)) return m2e (t, FONT_FAMILY, "tt");
   if (is_tuple (t, "\\tmtextsf", 1)) return m2e (t, FONT_FAMILY, "ss");
@@ -630,6 +674,7 @@ latex_command_to_tree (tree t) {
   if (is_tuple (t, "\\mathfrak", 1)) return m2e (t, MATH_FONT, "Euler");
   if (is_tuple (t, "\\mathbb", 1)) return m2e (t, MATH_FONT, "Bbb");
   if (is_tuple (t, "\\mathbbm", 1)) return m2e (t, MATH_FONT, "Bbb*");
+  if (is_tuple (t, "\\mathscr", 1)) return m2e (t, MATH_FONT, "cal*");
 
   if (is_tuple (t, "\\prime", 1)) return tree (RPRIME, string_arg (t[1]));
   if (is_tuple (t, "\\frac", 2)) return tree (FRAC, l2e (t[1]), l2e (t[2]));
@@ -657,7 +702,7 @@ latex_command_to_tree (tree t) {
     return tree (WIDE, l2e (t[1]), "<abovering>");
   if (is_tuple (t, "\\hspace", 1) || is_tuple (t, "\\hspace*", 1)) {
     tree r= t2e (t[1]);
-    if (is_compound (r, "fill", 0)) return tree (HTAB, "1fn");
+    if (is_var_compound (r, "fill", 0)) return tree (HTAB, "1fn");
     return tree (SPACE, r);
   }
   if (is_tuple (t, "\\vspace", 1) || is_tuple (t, "\\vspace*", 1))
@@ -709,6 +754,10 @@ latex_command_to_tree (tree t) {
     return tree (ABOVE, l2e (t[2]), l2e (t[1]));
   if (is_tuple (t, "\\underset", 2))
     return tree (BELOW, l2e (t[2]), l2e (t[1]));
+  if (is_tuple (t, "\\parbox", 2))
+    return compound ("mini-paragraph", v2e (t[1]), l2e (t[2]));
+  if (is_tuple (t, "\\parbox*", 3))
+    return compound ("mini-paragraph", v2e (t[2]), l2e (t[3]));
 
   int dtype= 0;
   if (is_large_delimiter (t, dtype)) {
@@ -778,7 +827,8 @@ latex_command_to_tree (tree t) {
       << tree (END, "array") << tree (RIGHT, ".");
     return r;
   }
-  if (is_tuple (t, "\\includegraphics", 1)) {
+  if (is_tuple (t, "\\includegraphics", 1) ||
+      is_tuple (t, "\\includegraphics*", 1)) {
     tree name= v2e (t[1]);
     if (name == "") return "";
     else {
@@ -787,6 +837,21 @@ latex_command_to_tree (tree t) {
       return g;
     }
   }
+  if (is_tuple (t, "\\includegraphics*", 2) ||
+      is_tuple (t, "\\includegraphics**", 2)) {
+    tree name= v2e (t[2]);
+    tree data= v2e (t[1]);
+    if (data == "" || name == "") return "";
+    else {
+      tree g (IMAGE, 7);
+      g[0]= name;
+      tree width = latex_eps_get (data, "width");
+      tree height= latex_eps_get (data, "height");
+      g[1]= width;
+      g[2]= height;
+      return g;
+    }
+  }
   if (is_tuple (t, "\\epsfig", 1)) {
     tree data  = v2e (t[1]);
     tree name  = latex_eps_get (data, "file");
@@ -794,13 +859,16 @@ latex_command_to_tree (tree t) {
     tree height= latex_eps_get (data, "height");
     if (name == "") return "";
     else {
-      tree g (IMAGE, 7);
+      tree g (IMAGE, 5);
       g[0]= name;
       g[1]= width;
       g[2]= height;
       return g;
+
     }
   }
+  if (is_tuple (t, "\\fbox", 1)) return compound ("frame", l2e (t[1]));
+  if (is_tuple (t, "\\centerline", 1)) return compound ("center", l2e (t[1]));
   if (is_tuple (t, "\\noalign", 1))
     return ""; // FIXME: for larger space in maple matrices
   if (is_tuple (t, "\\etalchar", 1)) return t2e (t[1]);
@@ -940,7 +1008,7 @@ parse_matrix_params (tree t) {
 }
 
 static void
-parse_pmatrix (tree& r, tree t, int& i, string lb, string rb) {
+parse_pmatrix (tree& r, tree t, int& i, string lb, string rb, string fm) {
   tree tformat= parse_matrix_params (t[i]);
   if (lb != "") r << tree (LEFT, lb);
 
@@ -963,37 +1031,55 @@ parse_pmatrix (tree& r, tree t, int& i, string lb, string rb) {
       E= tree (CONCAT);
       continue;
     }
-    else if (is_func (v, BEGIN) && ((v[0] == "array") ||(v[0] == "tabular"))) {
-      parse_pmatrix (E, t, i, "", "");
+    else if (is_func (v, BEGIN) && (v[0] == "array" || v[0] == "tabular")) {
+      parse_pmatrix (E, t, i, "", "", "tabular*");
+      if (i<N(t)) continue;
+      break;
+    }
+    else if (v == tree (BEGIN, "stack")) {
+      parse_pmatrix (E, t, i, "", "", "stack");
+      if (i<N(t)) continue;
+      break;
+    }
+    else if (v == tree (BEGIN, "cases")) {
+      parse_pmatrix (E, t, i, "", "", "choice");
       if (i<N(t)) continue;
       break;
     }
     else if (v == tree (BEGIN, "matrix")) {
-      parse_pmatrix (E, t, i, "", "");
+      parse_pmatrix (E, t, i, "", "", "tabular*");
       if (i<N(t)) continue;
       break;
     }
     else if (v == tree (BEGIN, "pmatrix")) {
-      parse_pmatrix (E, t, i, "(", ")");
+      parse_pmatrix (E, t, i, "", "", "matrix");
       if (i<N(t)) continue;
       break;
     }
     else if (v == tree (BEGIN, "bmatrix")) {
-      parse_pmatrix (E, t, i, "{", "}");
+      parse_pmatrix (E, t, i, "[", "]", "tabular*");
       if (i<N(t)) continue;
       break;
     }
     else if (v == tree (BEGIN, "vmatrix")) {
-      parse_pmatrix (E, t, i, "|", "|");
+      parse_pmatrix (E, t, i, "", "", "det");
+      if (i<N(t)) continue;
+      break;
+    }
+    else if (v == tree (BEGIN, "smallmatrix")) {
+      parse_pmatrix (E, t, i, "", "", "matrix*");
       if (i<N(t)) continue;
       break;
     }
     else if (v == tree (END, "array")) break;
     else if (v == tree (END, "tabular")) break;
+    else if (v == tree (END, "cases")) break;
+    else if (v == tree (END, "stack")) break;
     else if (v == tree (END, "matrix")) break;
     else if (v == tree (END, "pmatrix")) break;
     else if (v == tree (END, "bmatrix")) break;
     else if (v == tree (END, "vmatrix")) break;
+    else if (v == tree (END, "smallmatrix")) break;
     else if (v == tree (APPLY, "hline")) {
       int    row  = N(V)+ (N(L)==0? 0: 1);
       string row_s= row==0? as_string (row+1): as_string (row);
@@ -1015,17 +1101,15 @@ parse_pmatrix (tree& r, tree t, int& i, string lb, string rb) {
   rows= N(V);
 
   int x, y;
-  tree M (OLD_MATRIX);
-  for (y=0; y<rows; y++)
+  tree M (TABLE);
+  for (y=0; y<rows; y++) {
+    tree R (ROW);
     for (x=0; x<cols; x++)
-      if (x<N(V[y])) M << V[y][x];
-      else M << "";
-  M << as_string (cols) << as_string (rows);
-  if (tformat != tree (TFORMAT)) {
-    tformat << M;
-    M= tformat;
+      if (x<N(V[y])) R << tree (CELL, V[y][x]);
+      else R << tree (CELL, "");
+    M << R;
   }
-  r << M;
+  r << compound (fm, tree (TFORMAT, M));
   if (rb != "") r << tree (RIGHT, rb);
 }
 
@@ -1039,12 +1123,24 @@ finalize_pmatrix (tree t) {
     tree r (CONCAT);
     for (i=0; i<n; i++)
       if (is_func (u[i], BEGIN)) {
-	if (u[i][0] == "array") parse_pmatrix (r, u, i, "", "");
-	else if (u[i][0] == "tabular") parse_pmatrix (r, u, i, "", "");
-	else if (u[i][0] == "matrix") parse_pmatrix (r, u, i, "", "");
-	else if (u[i][0] == "pmatrix") parse_pmatrix (r, u, i, "(", ")");
-	else if (u[i][0] == "bmatrix") parse_pmatrix (r, u, i, "[", "]");
-	else if (u[i][0] == "vmatrix") parse_pmatrix (r, u, i, "|", "|");
+	if (u[i][0] == "array")
+	  parse_pmatrix (r, u, i, "", "", "tabular*");
+	else if (u[i][0] == "tabular")
+	  parse_pmatrix (r, u, i, "", "", "tabular*");
+	else if (u[i][0] == "cases")
+	  parse_pmatrix (r, u, i, "", "", "choice");
+	else if (u[i][0] == "stack")
+	  parse_pmatrix (r, u, i, "", "", "stack");
+	else if (u[i][0] == "matrix")
+	  parse_pmatrix (r, u, i, "", "", "tabular*");
+	else if (u[i][0] == "pmatrix")
+	  parse_pmatrix (r, u, i, "", "", "matrix");
+	else if (u[i][0] == "bmatrix")
+	  parse_pmatrix (r, u, i, "[", "]", "tabular*");
+	else if (u[i][0] == "vmatrix")
+	  parse_pmatrix (r, u, i, "", "", "det");
+	else if (u[i][0] == "smallmatrix")
+	  parse_pmatrix (r, u, i, "", "", "matrix*");
 	else r << u[i];
       }
       else r << u[i];
@@ -1052,6 +1148,16 @@ finalize_pmatrix (tree t) {
   }
   else if (is_func (u, APPLY, 2) && (u[0] == "matrix"))
     return tree (APPLY, "tabular*", u[1]);
+  else if (is_func (u, APPLY, 2) && (u[0] == "smallmatrix"))
+    return tree (APPLY, "matrix*", u[1]);
+  else if (is_func (u, APPLY, 2) && (u[0] == "substack")) {
+    tree cc (CONCAT);
+    cc << tree (BEGIN, "stack");
+    if (is_func (u[1], CONCAT)) cc << A(u[1]);
+    else cc << u[1];
+    cc << tree (END, "stack");
+    return finalize_pmatrix (cc);
+  }
   else return u;
 }
 
@@ -1079,6 +1185,7 @@ space_eater (tree t) {
 static bool
 admissible_env (tree t) {
   string s= t[0]->label;
+  if (ends (s, "*")) s= s (0, N(s)-1);
   if (latex_type ("\\begin-" * s) == "list") return true;
   if (latex_type ("\\begin-" * s) == "environment") return true;
   if (latex_type ("\\begin-" * s) == "math-environment") return true;
@@ -1095,6 +1202,10 @@ translate_list (string s) {
   if (s == "enumerateromancap") return "enumerate-romancap";
   if (s == "enumeratealpha") return "enumerate-alpha";
   if (s == "enumeratealphacap") return "enumerate-alphacap";
+  if (s == "inparaenum") return "enumerate";
+  if (s == "itemize*") return "itemize";
+  if (s == "enumerate*") return "enumerate";
+  if (s == "inparaenum*") return "enumerate";
   return s;
 }
 
@@ -1159,7 +1270,8 @@ finalize_layout (tree t) {
       }
       */
 
-      if (is_func (v, BEGIN, 1) && admissible_env (v)) {
+      if ((is_func (v, BEGIN, 1) || is_func (v, BEGIN, 2))
+          && admissible_env (v) && v[0] != "thebibliography") {
 	if (v == tree (BEGIN, "verbatim")) {
 	  r << v; i++;
 	  if ((i<n) && (t[i] == tree (FORMAT, "new line"))) {
@@ -1236,6 +1348,49 @@ finalize_layout (tree t) {
   else return u;
 }
 
+static tree
+finalize_sections (tree t) {
+  tree r (DOCUMENT);
+  for (int i=0; i<N(t); i++) {
+    tree u= t[i];
+    if (is_concat (u) && N(u) >= 2 &&
+	(is_var_compound (u[0], "part", 1) ||
+	 is_var_compound (u[0], "part*", 1) ||
+	 is_var_compound (u[0], "chapter", 1) ||
+	 is_var_compound (u[0], "chapter*", 1) ||
+	 is_var_compound (u[0], "section", 1) ||
+	 is_var_compound (u[0], "section*", 1) ||
+	 is_var_compound (u[0], "subsection", 1) ||
+	 is_var_compound (u[0], "subsection*", 1) ||
+	 is_var_compound (u[0], "subsubsection", 1) ||
+	 is_var_compound (u[0], "subsubsection*", 1)))
+      {
+	if (N(u) > 2 && u[1] == " ")
+	  u= u (0, 1) * u (2, N(u));
+	if (!is_func (u[1], LABEL) || (N(u) >= 3 && !is_func (u[2], LABEL))) {
+	  if (!is_func (u[1], LABEL)) {
+	    r << u[0];
+	    if (N(u) == 2) u= u[1];
+	    else u= u (1, N(u));
+	  }
+	  else {
+	    r << u (0, 2);
+	    if (N(u) == 3) u= u[2];
+	    else u= u (2, N(u));
+	  }
+	  if (is_atomic (u) && starts (u->label, " "))
+	    u= u->label (1, N(u->label));
+	  if (is_concat (u) && is_atomic (u[0]) && starts (u[0]->label, " "))
+	    u= tree (CONCAT, u[0]->label (1, N(u[0]->label))) * u (1, N(u));
+	  r << u;
+	}
+	else r << u;
+      }
+    else r << u;
+  }
+  return r;
+}
+
 static tree
 finalize_document (tree t) {
   if (is_atomic (t)) t= tree (CONCAT, t);
@@ -1254,7 +1409,7 @@ finalize_document (tree t) {
     else r << t(start,i);
     if (i==(N(t)-1)) r << "";
   }
-  return r;
+  return finalize_sections (r);
 }
 
 bool
@@ -1395,7 +1550,7 @@ env_hacks (tree t) {
     else ea << end[i];
   beg= tree (CONCAT, ba);
   end= tree (CONCAT, ea);
-  for (i=N(beg); i>0 && is_func (beg[i-1], RESET, 1); i--);
+  for (i=N(beg); i>0 && is_func (beg[i-1], RESET, 1); i--) ;
   bool ok= (i<<1) >= N(beg);
   for (int k=0; k<N(beg)-i; k++) {
     ok= ok && is_func (beg[i-k-1], SET, 2) && beg[i-k-1][0] == beg[i+k][0];
@@ -1442,8 +1597,8 @@ handle_improper_matches (tree t) {
 tree
 float_body (tree t) {
   if (is_atomic (t)) return t;
-  else if (is_compound (t, "caption", 1)) return "";
-  else if (is_compound (t, "center", 1)) return float_body (t[0]);
+  else if (is_var_compound (t, "caption", 1)) return "";
+  else if (is_var_compound (t, "center", 1)) return float_body (t[N(t)-1]);
   else {
     int i, n= N(t);
     tree r (t, n);
@@ -1457,7 +1612,7 @@ float_body (tree t) {
 tree
 find_caption (tree t) {
   if (is_atomic (t)) return "";
-  else if (is_compound (t, "caption", 1)) return t[0];
+  else if (is_var_compound (t, "caption", 1)) return t[N(t)-1];
   else {
     int i, n= N(t);
     for (i=0; i<n; i++) {
@@ -1471,12 +1626,12 @@ find_caption (tree t) {
 tree
 finalize_floats (tree t) {
   if (is_atomic (t)) return t;
-  else if (is_compound (t, "bigfigure", 1)) {
+  else if (is_var_compound (t, "bigfigure", 1)) {
     tree body= float_body (t[N(t)-1]);
     tree capt= find_caption (t[N(t)-1]);
     return tree (make_tree_label ("big-figure"), body, capt);
   }
-  else if (is_compound (t, "bigtable", 1)) {
+  else if (is_var_compound (t, "bigtable", 1)) {
     tree body= float_body (t[N(t)-1]);
     tree capt= find_caption (t[N(t)-1]);
     return tree (make_tree_label ("big-table"), body, capt);
@@ -1500,17 +1655,6 @@ finalize_textm (tree t) {
   return simplify_correct (u);
 }
 
-/******************************************************************************
-* Final corrections
-******************************************************************************/
-
-tree
-tex_correct (tree t) {
-  t= superfluous_invisible_correct (t);
-  t= missing_invisible_correct (t, 1);
-  return t;
-}
-
 /******************************************************************************
 * Interface
 ******************************************************************************/
@@ -1538,7 +1682,7 @@ latex_to_tree (tree t1) {
   tree t5= is_document? finalize_preamble (t4, style): t4;
   // cout << "\n\nt5= " << t5 << "\n\n";
   tree t6= handle_improper_matches (t5);
-  //cout << "\n\nt6= " << t6 << "\n\n";
+  // cout << "\n\nt6= " << t6 << "\n\n";
   if ((!is_document) && is_func (t6, DOCUMENT, 1)) t6= t6[0];
   tree t7= upgrade_tex (t6);
   // cout << "\n\nt7= " << t7 << "\n\n";
@@ -1548,10 +1692,6 @@ latex_to_tree (tree t1) {
   // cout << "\n\nt9= " << t9 << "\n\n";
   tree t10= drd_correct (std_drd, t9);
   // cout << "\n\nt10= " << t10 << "\n\n";
-  tree t11= simplify_correct (t10);
-  //cout << "\n\nt11= " << t11 << "\n\n";
-  tree t12= tex_correct (t11);
-  //cout << "\n\nt12= " << t12 << "\n\n";
 
   if (!exists (url ("$TEXMACS_STYLE_PATH", style * ".ts")))
     style= "generic";
@@ -1569,6 +1709,14 @@ latex_to_tree (tree t1) {
     initial << tree (ASSOCIATE, LANGUAGE, lan);
     mods << tree (LANGUAGE) << tree (lan);
   }
+
+  tree t11= t10;
+  if (is_document) t11= simplify_correct (t10);
+  else if (N (mods) > 0) { t11= mods; t11 << t10; }
+  // cout << "\n\nt11= " << t11 << "\n\n";
+  tree t12= latex_correct (t11);
+  // cout << "\n\nt12= " << t12 << "\n\n";
+
   if (is_document) {
     tree the_body   = compound ("body", t12);
     tree the_style  = compound ("style", style);
@@ -1578,10 +1726,7 @@ latex_to_tree (tree t1) {
     if (N (initial) == 0) return tree (DOCUMENT, the_style, the_body);
     else return tree (DOCUMENT, the_style, the_body, the_initial);
   }
-  else {
-    if (N (mods) > 0) { mods << t10; return mods; }
-    return t10;
-  }
+  else return t12;
 }
 
 tree
