@@ -39,6 +39,58 @@ drd_decode (int i) {
   return decode_table[i];
 }
 
+/******************************************************************************
+* Names for drd types
+******************************************************************************/
+
+string
+drd_decode_type (int i) {
+  switch (i) {
+  case TYPE_REGULAR: return "regular";
+  case TYPE_ADHOC: return "adhoc";
+  case TYPE_VARIABLE: return "variable";
+  case TYPE_ARGUMENT: return "argument";
+  case TYPE_BOOLEAN: return "boolean";
+  case TYPE_INTEGER: return "integer";
+  case TYPE_STRING: return "string";
+  case TYPE_LENGTH: return "length";
+  case TYPE_NUMERIC: return "numeric";
+  case TYPE_CODE: return "code";
+  case TYPE_IDENTIFIER: return "identifier";
+  case TYPE_URL: return "url";
+  case TYPE_GRAPHICAL: return "graphical";
+  case TYPE_POINT: return "point";
+  case TYPE_ANIMATION: return "animation";
+  case TYPE_DURATION: return "duration";
+  case TYPE_UNKNOWN: return "unknown";
+  case TYPE_ERROR: return "error";
+  default: return "unknown";
+  }
+}
+
+int
+drd_encode_type (string s) {
+  if (s == "regular") return TYPE_REGULAR;
+  else if (s == "adhoc") return TYPE_ADHOC;
+  else if (s == "variable") return TYPE_VARIABLE;
+  else if (s == "argument") return TYPE_ARGUMENT;
+  else if (s == "boolean") return TYPE_BOOLEAN;
+  else if (s == "integer") return TYPE_INTEGER;
+  else if (s == "string") return TYPE_STRING;
+  else if (s == "length") return TYPE_LENGTH;
+  else if (s == "numeric") return TYPE_NUMERIC;
+  else if (s == "code") return TYPE_CODE;
+  else if (s == "identifier") return TYPE_IDENTIFIER;
+  else if (s == "url") return TYPE_URL;
+  else if (s == "graphical") return TYPE_GRAPHICAL;
+  else if (s == "point") return TYPE_POINT;
+  else if (s == "animation") return TYPE_ANIMATION;
+  else if (s == "duration") return TYPE_DURATION;
+  else if (s == "unknown") return TYPE_UNKNOWN;
+  else if (s == "error") return TYPE_ERROR;
+  else return -1;
+}
+
 /******************************************************************************
 * Properties of the tag
 ******************************************************************************/
@@ -314,15 +366,14 @@ tag_info_rep::locals (int i, string var, string val) {
   return tag_info (pi, ci, extra);
 }
 
-tag_info
-tag_info_rep::name (string s) {
-  set_attribute ("name", s);
-  return tag_info (pi, ci, extra);
-}
-
 void
 tag_info_rep::set_attribute (string which, tree val) {
   if (extra == "") extra= tree (ATTR);
+  for (int i=0; i+1<N(extra); i+=2)
+    if (extra[i] == tree (which)) {
+      extra[i+1]= val;
+      return;
+    }
   extra << tree (which) << val;
 }
 
@@ -336,6 +387,34 @@ tag_info_rep::get_attribute (string which) {
   return "";
 }
 
+tag_info
+tag_info_rep::name (string s) {
+  set_attribute ("name", s);
+  return tag_info (pi, ci, extra);
+}
+
+tag_info
+tag_info_rep::long_name (string s) {
+  set_attribute ("long-name", s);
+  return tag_info (pi, ci, extra);
+}
+
+tag_info
+tag_info_rep::name (int i, string s) {
+  if (i < 0 || i >= N(ci)) cout << i << " out of " << N(ci) << "\n";
+  ASSERT (i >= 0 && i<N(ci), "index out of range");
+  set_attribute ("name-" * as_string (i), s);
+  return tag_info (pi, ci, extra);
+}
+
+tag_info
+tag_info_rep::long_name (int i, string s) {
+  if (i < 0 || i >= N(ci)) cout << i << " out of " << N(ci) << "\n";
+  ASSERT (i >= 0 && i<N(ci), "index out of range");
+  set_attribute ("long-name-" * as_string (i), s);
+  return tag_info (pi, ci, extra);
+}
+
 int
 tag_info_rep::get_index (int child, int n) {
   int r= 0;
