@@ -4,10 +4,9 @@
 * DESCRIPTION: environment variables for typesetting
 * COPYRIGHT  : (C) 1999  Joris van der Hoeven
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #ifndef VARS_H
@@ -24,8 +23,13 @@ extern string PREAMBLE;
 extern string SAVE_AUX;
 extern string MODE;
 extern string INFO_FLAG;
+extern string WINDOW_BARS;
+extern string SCROLL_BARS;
 extern string IDENTITY;
 extern string TABULAR;
+extern string THE_LABEL;
+extern string THE_TAGS;
+extern string THE_MODULES;
 
 extern string FONT;
 extern string FONT_FAMILY;
@@ -36,6 +40,8 @@ extern string FONT_BASE_SIZE;
 extern string MAGNIFICATION;
 extern string COLOR;
 extern string BG_COLOR;
+extern string LOCUS_COLOR;
+extern string VISITED_COLOR;
 extern string LANGUAGE;
 extern string ATOM_DECORATIONS;
 extern string LINE_DECORATIONS;
@@ -52,6 +58,8 @@ extern string MATH_LEVEL;
 extern string MATH_DISPLAY;
 extern string MATH_CONDENSED;
 extern string MATH_VPOS;
+extern string MATH_NESTING_MODE;
+extern string MATH_NESTING_LEVEL;
 
 extern string PROG_LANGUAGE;
 extern string PROG_SCRIPTS;
@@ -78,6 +86,7 @@ extern string PAR_COLUMNS;
 extern string PAR_COLUMNS_SEP;
 
 extern string PAGE_MEDIUM;
+extern string PAGE_PRINTED;
 extern string PAGE_TYPE;
 extern string PAGE_ORIENTATION;
 extern string PAGE_WIDTH_MARGIN;
@@ -166,11 +175,11 @@ extern string CELL_ROW_SPAN;
 extern string CELL_COL_SPAN;
 extern string CELL_VCORRECT;
 extern string CELL_HYPHEN;
+extern string CELL_BLOCK;
 extern string CELL_ROW_NR;
 extern string CELL_COL_NR;
 
 extern string POINT_STYLE;
-
 extern string LINE_WIDTH;
 extern string DASH_STYLE;
 extern string DASH_STYLE_UNIT;
@@ -203,4 +212,19 @@ extern string SRC_SPECIAL;
 extern string SRC_COMPACT;
 extern string SRC_CLOSE;
 
+extern string CANVAS_TYPE;
+extern string CANVAS_COLOR;
+extern string CANVAS_HPADDING;
+extern string CANVAS_VPADDING;
+extern string CANVAS_BAR_WIDTH;
+extern string CANVAS_BAR_PADDING;
+extern string CANVAS_BAR_COLOR;
+extern string ORNAMENT_SHAPE;
+extern string ORNAMENT_BORDER;
+extern string ORNAMENT_HPADDING;
+extern string ORNAMENT_VPADDING;
+extern string ORNAMENT_COLOR;
+extern string ORNAMENT_SUNNY_COLOR;
+extern string ORNAMENT_SHADOW_COLOR;
+
 #endif // defined VARS_H
