@@ -18,5 +18,17 @@ tree with_correct (tree t);
 tree superfluous_with_correct (tree t);
 tree superfluous_invisible_correct (tree t);
 tree missing_invisible_correct (tree t, int force= -1);
+tree upgrade_brackets (tree t, string mode= "text");
+tree downgrade_brackets (tree t, bool delete_missing= false);
+tree move_brackets (tree t);
+
+int  count_math_errors (tree t, int mode= 0);
+void math_status_cumul (tree t);
+void math_status_print ();
+void math_status_reset ();
+
+tree latex_correct (tree t);
+tree automatic_correct (tree t, string version);
+tree manual_correct (tree t);
 
 #endif // defined TREE_CORRECT_H
