@@ -4,10 +4,9 @@
 * DESCRIPTION: graphics between the editor and the window manager
 * COPYRIGHT  : (C) 2003  Joris van der Hoeven and Henri Lesourd
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #include "Interface/edit_graphics.hpp"
@@ -16,13 +15,16 @@
 #include "Graphics/curve.hpp"
 #include "Boxes/graphics.hpp"
 #include "Bridge/impl_typesetter.hpp"
-#include "merge_sort.hpp"
 
 /******************************************************************************
 * Constructors and destructors
 ******************************************************************************/
 
-edit_graphics_rep::edit_graphics_rep () { graphical_object= tree(); }
+edit_graphics_rep::edit_graphics_rep () {
+  gr_x= gr_y= 0.0;
+  graphical_object= tree();
+}
+
 edit_graphics_rep::~edit_graphics_rep () {}
 
 /******************************************************************************
@@ -34,9 +36,10 @@ edit_graphics_rep::inside_graphics (bool b) {
   path p   = path_up (tp);
   bool flag= false;
   tree st  = et;
-  while (!nil (p)) {
+  while (!is_nil (p)) {
     if (is_func (st, GRAPHICS)) flag= true;
-    if (b && is_func (st, TEXT_AT )) flag= false;
+    if (b && is_func (st, TEXT_AT)) flag= false;
+    if (is_atomic (st) || p->item < 0 || p->item >= N(st)) break;
     st= st[p->item];
     p = p->next;
   }
@@ -51,7 +54,7 @@ edit_graphics_rep::inside_active_graphics (bool b) {
 bool
 edit_graphics_rep::over_graphics (SI x, SI y) {
   frame f= find_frame ();
-  if (!nil (f)) {
+  if (!is_nil (f)) {
     point lim1, lim2;
     find_limits (lim1, lim2);
     point p = adjust (f [point (x, y)]);
@@ -69,7 +72,7 @@ edit_graphics_rep::get_graphics () {
   path p   = path_up (tp);
   tree st  = et;
   tree res = tree ();
-  while (!nil (p)) {
+  while (!is_nil (p)) {
     if (is_func (st, GRAPHICS)) res= st;
     st= st[p->item];
     p = p->next;
@@ -77,11 +80,21 @@ edit_graphics_rep::get_graphics () {
   return res;
 }
 
+double
+edit_graphics_rep::get_x () {
+  return gr_x;
+}
+
+double
+edit_graphics_rep::get_y () {
+  return gr_y;
+}
+
 frame
-edit_graphics_rep::find_frame () {
+edit_graphics_rep::find_frame (bool last) {
   bool bp_found;
   path bp= eb->find_box_path (tp, bp_found);
-  if (bp_found) return eb->find_frame (path_up (bp));
+  if (bp_found) return eb->find_frame (path_up (bp), last);
   else return frame ();
 }
 
@@ -107,7 +120,7 @@ edit_graphics_rep::find_graphical_region (SI& x1, SI& y1, SI& x2, SI& y2) {
   find_limits (lim1, lim2);
   if (lim1 == point ()) return false;
   frame f= find_frame ();
-  if (nil (f)) return false;
+  if (is_nil (f)) return false;
   point p1= f (point (lim1[0], lim1[1]));
   point p2= f (point (lim2[0], lim2[1]));
   x1= (SI) p1[0]; y1= (SI) p1[1];
@@ -118,18 +131,64 @@ edit_graphics_rep::find_graphical_region (SI& x1, SI& y1, SI& x2, SI& y2) {
 point
 edit_graphics_rep::adjust (point p) {
   frame f= find_frame ();
-  point pmin= p, pmax= p;
-  if (!nil (f)) {
-    static const int NB= 10;
-    point p2= f (p);
-    pmin= f [point (p2[0]-NB*PIXEL, p2[1]-NB*PIXEL)];
-    pmax= f [point (p2[0]+NB*PIXEL, p2[1]+NB*PIXEL)];
-  }
   grid g= find_grid ();
-  if (nil (g))
+  if (!is_nil (g) && !is_nil (gr0) && g!=gr0) {
+    graphical_select (p[0], p[1]);
+    g= gr0;
+  }
+  if (is_nil (g))
     return p;
-  else
-    return g->find_closest_point (p, pmin, pmax);
+  else {
+    point res;
+    gr_selections sels= gs;
+    frame f2= find_frame (true);
+    if (!is_nil (f2)) {
+      point fp= f2 (p);
+      int i;
+      if ((tree)g == "empty_grid") {
+	if (N(pts)>0)
+	  res= pts[0];
+	for (i=0; i<N(pts); i++) {
+	  point sp= pts[i];
+	  if (N(sp)>0 && norm (fp - sp) < 5*get_pixel_size ())
+	    res= pts[i];
+	}
+	int n= N(sels);
+	for (i=0; i<n; i++) {
+	  point sp= sels[i]->p;
+	  if (N(res)==0 || (N(sp)>0 && norm (fp - sp) < 5*get_pixel_size ()
+			            && norm (fp - sp) < norm (fp - res)))
+	    res= sels[i]->p;
+	}
+      }
+      else
+      if (!is_nil (f)) { 
+	res= f2 (g->find_point_around (p, 10*get_pixel_size (), f));
+	for (i=0; i<N(pts); i++) {
+	  point sp= pts[i];
+	  if (N(sp)>0 && norm (fp - sp) < norm (fp - res))
+	    res= pts[i];
+	}
+	for (i=0; i<N(ci); i++) {
+	  point sp= ci[i];
+	  if (N(sp)>0 && norm (fp - sp) < norm (fp - res))
+	    res= ci[i];
+	}
+	for (i=0; i<N(cgi); i++) {
+	  point sp= cgi[i];
+	  if (N(sp)>0 && norm (fp - sp) < norm (fp - res))
+	    res= cgi[i];
+	}
+      //TODO: Adjusting by means on freely moving on surface of closed curves
+	;
+      }
+      if (N(res)>0)
+	res= f2[res];
+      else
+	res= p;
+    }
+    return res;
+  }
 }
 
 tree
@@ -137,24 +196,52 @@ edit_graphics_rep::find_point (point p) {
   return tree (_POINT, as_string (p[0]), as_string (p[1]));
 }
 
-struct less_eq_gr_selection {
-  static inline bool leq (gr_selection& a, gr_selection& b) {
-    return a->dist <= b->dist; }
-};
-
 tree
 edit_graphics_rep::graphical_select (double x, double y) { 
   frame f= find_frame ();
-  if (nil (f)) return tuple ();
+  if (is_nil (f)) return tuple ();
   gr_selections sels;
-  point p = f (point (x, y));
-  sels= eb->graphical_select ((SI)p[0], (SI)p[1], 10 * get_pixel_size ());
-  merge_sort_leq <gr_selection, less_eq_gr_selection> (sels);
-  int i, n= N(sels);
-  array<array<path> > gs (n);
-  for (i=0; i<n; i++)
-    gs[i]= sels[i]->cp;
-  return (tree) gs;
+  point p0 = point (x, y);
+  point p = f (p0);
+  sels= eb->graphical_select ((SI)p[0], (SI)p[1], 10*get_pixel_size ());
+  gs= sels;
+  pts= array<point> (0);
+  ci= array<point> (0);
+  cgi= array<point> (0);
+  gr0= empty_grid ();
+  grid g= find_grid ();
+  frame f2= find_frame (true);
+  if (!is_nil (g) && !is_nil (f2)) {
+    gr0= g;
+    p = f2 (point (x, y));
+    int i, j, n= N(sels);
+    for (i=0; i<n; i++) {
+      array<point> pts2= sels[i]->pts;
+      if (N(pts2)>0 && norm (pts2[0] - p) <= 10*get_pixel_size ())
+	pts= pts << pts2[0];
+      if (N(pts2)>1 && norm (pts2[1] - p) <= 10*get_pixel_size ())
+	pts= pts << pts2[1];
+    }
+    double eps= get_pixel_size () / 10.0;
+    for (i=0; i<n; i++) {
+      for (j=0; j<n; j++)
+        if (i<j) {
+	  curve c1= sels[i]->c;
+	  curve c2= sels[j]->c;
+	  if (!is_nil (c1) && !is_nil (c2))
+	    ci= ci << intersection (c1, c2, p, eps);
+        }
+    }
+    array<grid_curve> gc= g->get_curves_around (p0, 10*get_pixel_size (), f);
+    //FIXME: Too slow
+    for (i=0; i<N(gc); i++) {
+      curve c= f2 (gc[i]->c);
+      for (j=0; j<n; j++)
+	if (!is_nil (sels[j]->c))
+	  cgi= cgi << intersection (c, sels[j]->c, p, eps);
+    }
+  }
+  return as_tree (sels);
 }
 
 tree
@@ -162,16 +249,11 @@ edit_graphics_rep::graphical_select (
   double x1, double y1, double x2, double y2)
 { 
   frame f= find_frame ();
-  if (nil (f)) return tuple ();
+  if (is_nil (f)) return tuple ();
   gr_selections sels;
   point p1 = f (point (x1, y1)), p2= f (point (x2, y2));
   sels= eb->graphical_select ((SI)p1[0], (SI)p1[1], (SI)p2[0], (SI)p2[1]);
-  merge_sort_leq <gr_selection, less_eq_gr_selection> (sels);
-  int i, n= N(sels);
-  array<array<path> > gs (n);
-  for (i=0; i<n; i++)
-    gs[i]= sels[i]->cp;
-  return (tree) gs;
+  return as_tree (sels);
 }
 
 tree
@@ -188,7 +270,7 @@ edit_graphics_rep::set_graphical_object (tree t) {
   //tree old_fr= env->local_begin (GR_FRAME, (tree) find_frame ());  
   frame f_env= env->fr;
   env->fr= find_frame ();
-  if (!nil (env->fr)) {
+  if (!is_nil (env->fr)) {
     int i,n=0;
     go_box= typeset_as_concat (env, t, path (0));
     for (i=0; i<N(go_box); i++)
@@ -214,7 +296,7 @@ edit_graphics_rep::set_graphical_object (tree t) {
 void
 edit_graphics_rep::invalidate_graphical_object () {
   SI gx1, gy1, gx2, gy2;
-  if (find_graphical_region (gx1, gy1, gx2, gy2) && !nil (go_box)) {
+  if (find_graphical_region (gx1, gy1, gx2, gy2) && !is_nil (go_box)) {
     int i;
     rectangles rs;
     rectangle gr (gx1, gy1, gx2, gy2);
@@ -228,51 +310,66 @@ edit_graphics_rep::invalidate_graphical_object () {
 }
 
 void
-edit_graphics_rep::draw_graphical_object (ps_device dev) {
-  if (nil (go_box)) set_graphical_object (graphical_object);
-  if (nil (go_box)) return;
+edit_graphics_rep::draw_graphical_object (renderer ren) {
+  if (is_nil (go_box)) set_graphical_object (graphical_object);
+  if (is_nil (go_box)) return;
   SI ox1, oy1, ox2, oy2;
-  dev->get_clipping (ox1, oy1, ox2, oy2);
+  ren->get_clipping (ox1, oy1, ox2, oy2);
   SI gx1, gy1, gx2, gy2;
   if (find_graphical_region (gx1, gy1, gx2, gy2))
-    dev->extra_clipping (gx1, gy1, gx2, gy2);
+    ren->extra_clipping (gx1, gy1, gx2, gy2);
   int i;
   for (i=0; i<go_box->subnr(); i++) {
     box b= go_box->subbox (i);
     if ((tree)b=="point" || (tree)b=="curve")
-      b->display (dev);
+      b->display (ren);
     else {
       rectangles rs;
-      b->redraw (dev, path (), rs);
+      b->redraw (ren, path (), rs);
     }
   }
-  dev->set_clipping (ox1, oy1, ox2, oy2);
+  ren->set_clipping (ox1, oy1, ox2, oy2);
 }
 
 bool
-edit_graphics_rep::mouse_graphics (string type, SI x, SI y, time_t t) {
+edit_graphics_rep::mouse_graphics (string type, SI x, SI y, int m, time_t t) {
   (void) t;
   // apply_changes (); // FIXME: remove after review of synchronization
   frame f= find_frame ();
-  if (!nil (f)) {
+  if (!is_nil (f)) {
     if (!over_graphics (x, y)) return false;
-    point p = adjust (f [point (x, y)]);
     if (type == "move" || type == "dragging")
-      if (dis->check_event (MOTION_EVENT))
+      if (check_event (MOTION_EVENT))
 	return true;
+    point p = f [point (x, y)];
+    graphical_select (p[0], p[1]); // init the caching for adjust().
+    p = adjust (p);
+    gr_x= p[0];
+    gr_y= p[1];
     string sx= as_string (p[0]);
     string sy= as_string (p[1]);
     invalidate_graphical_object ();
-    if (type == "move"            ) call ("graphics-move-point"      , sx, sy);
-    if (type == "release-left"    ) call ("graphics-insert-point"    , sx, sy);
-    if (type == "release-middle"  ) call ("graphics-remove-point"    , sx, sy);
-    if (type == "release-right"   ) call ("graphics-last-point"      , sx, sy);
-    if (type == "start-drag"      ) call ("graphics-start-drag"      , sx, sy);
-    if (type == "dragging"        ) call ("graphics-dragging"        , sx, sy);
-    if (type == "end-drag"        ) call ("graphics-end-drag"        , sx, sy);
-    if (type == "start-right-drag") call ("graphics-start-right-drag", sx, sy);
-    if (type == "right-dragging"  ) call ("graphics-right-dragging"  , sx, sy);
-    if (type == "end-right-drag"  ) call ("graphics-end-right-drag"  , sx, sy);
+    call ("set-keyboard-modifiers", object (m));
+    if (type == "move")
+      call ("graphics-move-point", sx, sy);
+    else if (type == "release-left")
+      call ("graphics-insert-point", sx, sy);
+    else if (type == "release-middle")
+      call ("graphics-remove-point", sx, sy);
+    else if (type == "release-right")
+      call ("graphics-last-point", sx, sy);
+    else if (type == "start-drag")
+      call ("graphics-start-drag", sx, sy);
+    else if (type == "dragging")
+      call ("graphics-dragging", sx, sy);
+    else if (type == "end-drag")
+      call ("graphics-end-drag", sx, sy);
+    else if (type == "start-right-drag")
+      call ("graphics-start-right-drag", sx, sy);
+    else if (type == "right-dragging")
+      call ("graphics-right-dragging", sx, sy);
+    else if (type == "end-right-drag")
+      call ("graphics-end-right-drag", sx, sy);
     invalidate_graphical_object ();
     notify_change (THE_CURSOR);
     return true;
