@@ -18,12 +18,12 @@
 #include "data_cache.hpp"
 #include "drd_mode.hpp"
 #include "message.hpp"
+#include "tree_traverse.hpp"
 #ifdef EXPERIMENTAL
 #include "../../Style/Evaluate/evaluate_main.hpp"
 #endif
 
 extern void (*env_next_prog)(void);
-extern void selection_correct (tree t, path i1, path i2, path& o1, path& o2);
 
 /*static*/ string
 MODE_LANGUAGE (string mode) {
@@ -74,6 +74,10 @@ edit_interface_rep::operator tree () {
 
 void
 edit_interface_rep::suspend () {
+  if (got_focus) {
+    interrupt_shortcut ();
+    set_message ("", "", false);
+  }
   got_focus= false;
   notify_change (THE_FOCUS);
   if (is_attached (this)) {
@@ -88,8 +92,9 @@ edit_interface_rep::resume () {
   got_focus= true;
   SERVER (menu_main ("(horizontal (link texmacs-menu))"));
   SERVER (menu_icons (0, "(horizontal (link texmacs-main-icons))"));
-  SERVER (menu_icons (1, "(horizontal (link texmacs-context-icons))"));
-  SERVER (menu_icons (2, "(horizontal (link texmacs-extra-icons))"));
+  SERVER (menu_icons (1, "(horizontal (link texmacs-mode-icons))"));
+  SERVER (menu_icons (2, "(horizontal (link texmacs-focus-icons))"));
+  SERVER (menu_icons (3, "(horizontal (link texmacs-extra-icons))"));
   cur_sb= 2;
   tp= make_cursor_accessible (tp, true);
   notify_change (THE_FOCUS + THE_EXTENTS + THE_CURSOR);
@@ -261,20 +266,64 @@ is_graphical (tree t) {
     is_func (t, SPLINE) || is_func (t, CSPLINE);
 }
 
+static void
+correct_adjacent (rectangles& rs1, rectangles& rs2) {
+  if (N(rs1) != 1 || N(rs2) != 1) return;
+  SI bot1= rs1->item->y1;
+  SI top2= rs2->item->y2;
+  if (rs1->item->y1 <= rs2->item->y1) {
+    //cout << "Discard " << rs1->item->y1 << ", " << rs2->item->y1 << "\n";
+    return;
+  }
+  if (rs1->item->y2 <= rs2->item->y2) {
+    //cout << "Discard " << rs1->item->y2 << ", " << rs2->item->y2 << "\n";
+    return;
+  }
+  SI mid= (bot1 + top2) >> 1;
+  rs1->item->y1= mid;
+  rs2->item->y2= mid;
+}
+
 void
 edit_interface_rep::compute_env_rects (path p, rectangles& rs, bool recurse) {
   if (p == rp) return;
   tree st= subtree (et, p);
-  if (is_atomic (st) ||
-      drd->is_child_enforcing (st) ||
-      //is_document (st) || is_concat (st) ||
-      is_func (st, TABLE) || is_func (st, SUBTABLE) ||
-      is_func (st, ROW) || is_func (st, TFORMAT) ||
-      is_graphical (st) ||
-      (is_func (st, WITH) && is_graphical (st[N(st)-1])) ||
-      (is_func (st, WITH) && is_func (st[N(st)-1], TEXT_AT)) ||
-      (is_compound (st, "math", 1) &&
-       is_compound (subtree (et, path_up (p)), "input")))
+  if ((is_func (st, TABLE) || is_func (st, SUBTABLE)) &&
+      recurse && get_preference ("show table cells") == "on") {
+    rectangles rl;
+    for (int i=0; i<N(st); i++) {
+      if (is_func (st[i], ROW))
+        for (int j=0; j<N(st[i]); j++) {
+          selection sel= eb->find_check_selection (p*i*j*0, p*i*j*1);
+          rectangles rsel= copy (thicken (sel->rs, 0, 2 * pixel));
+          if (i > 0 && is_func (st[i-1], ROW) && j < N(st[i-1])) {
+            selection bis= eb->find_check_selection (p*(i-1)*j*0, p*(i-1)*j*1);
+            rectangles rbis= copy (thicken (bis->rs, 0, 2 * pixel));
+            correct_adjacent (rbis, rsel);
+          }
+          if (i+1 < N(st) && is_func (st[i+1], ROW) && j < N(st[i+1])) {
+            selection bis= eb->find_check_selection (p*(i+1)*j*0, p*(i+1)*j*1);
+            rectangles rbis= copy (thicken (bis->rs, 0, 2 * pixel));
+            correct_adjacent (rsel, rbis);
+          }
+          rectangles selp= thicken (rsel,  pixel/2,  pixel/2);
+          rectangles selm= thicken (rsel, -pixel/2, -pixel/2);
+          rl << simplify (::correct (selp - selm));
+        }
+    }
+    rs << simplify (rl);
+    if (recurse) compute_env_rects (path_up (p), rs, recurse);
+  }
+  else if (is_atomic (st) ||
+           drd->is_child_enforcing (st) ||
+           //is_document (st) || is_concat (st) ||
+           is_func (st, TABLE) || is_func (st, SUBTABLE) ||
+           is_func (st, ROW) || is_func (st, TFORMAT) ||
+           is_graphical (st) ||
+           (is_func (st, WITH) && is_graphical (st[N(st)-1])) ||
+           (is_func (st, WITH) && is_func (st[N(st)-1], TEXT_AT)) ||
+           (is_compound (st, "math", 1) &&
+            is_compound (subtree (et, path_up (p)), "input")))
     compute_env_rects (path_up (p), rs, recurse);
   else {
     int new_mode= DRD_ACCESS_NORMAL;
@@ -289,12 +338,15 @@ edit_interface_rep::compute_env_rects (path p, rectangles& rs, bool recurse) {
 	p2= end   (et, p * 0);
       }
       if (is_func (st, CELL)) { q1= p1; q2= p2; }
-      else selection_correct (et, p1, p2, q1, q2);
+      else selection_correct (p1, p2, q1, q2);
       selection sel= eb->find_check_selection (q1, q2);
-      rs << outline (sel->rs, pixel);
+      if (N(focus_get ()) >= N(p))
+        if (!recurse || get_preference ("show full context") == "on")
+          rs << outline (sel->rs, pixel);
     }
     set_access_mode (old_mode);
-    if (recurse) compute_env_rects (path_up (p), rs, recurse);
+    if (recurse || N(rs) == 0)
+      compute_env_rects (path_up (p), rs, recurse);
   }
 }
 
@@ -306,6 +358,8 @@ void
 edit_interface_rep::notify_change (int change) {
   env_change= env_change | change;
   needs_update ();
+  if ((change & (THE_TREE | THE_SELECTION | THE_CURSOR)) != 0)
+    manual_focus_set (path (), (change & THE_TREE) != 0);
 }
 
 bool
@@ -340,8 +394,9 @@ edit_interface_rep::apply_changes () {
     {
       SERVER (menu_main ("(horizontal (link texmacs-menu))"));
       SERVER (menu_icons (0, "(horizontal (link texmacs-main-icons))"));
-      SERVER (menu_icons (1, "(horizontal (link texmacs-context-icons))"));
-      SERVER (menu_icons (2, "(horizontal (link texmacs-extra-icons))"));
+      SERVER (menu_icons (1, "(horizontal (link texmacs-mode-icons))"));
+      SERVER (menu_icons (2, "(horizontal (link texmacs-focus-icons))"));
+      SERVER (menu_icons (3, "(horizontal (link texmacs-extra-icons))"));
       set_footer ();
       if (!get_renderer (this) -> interrupted ()) drd_update ();
       cache_memorize ();
@@ -363,7 +418,11 @@ edit_interface_rep::apply_changes () {
     ::get_size (get_window (this), wx, wy);
     if (get_init_string (SCROLL_BARS) == "false") sb= 0;
     if (get_server () -> in_full_screen_mode ()) sb= 0;
+#ifdef QTTEXMACS
+    if (sb) wx -= 24 * PIXEL;
+#else
     if (sb) wx -= 20 * PIXEL;
+#endif
     if (wx != cur_wx || wy != cur_wy) {
       cur_wx= wx; cur_wy= wy;
       init_env (PAGE_SCREEN_WIDTH, as_string (wx*sfactor) * "tmpt");
@@ -397,21 +456,23 @@ edit_interface_rep::apply_changes () {
   
   // cout << "Handling selection\n";
   if (env_change & (THE_TREE+THE_ENVIRONMENT+THE_SELECTION)) {
-    if (made_selection) invalidate (selection_rects);
+    if (made_selection) {
+      invalidate (selection_rects);
+      if (!selection_active_any ()) {
+        made_selection= false;
+        set_selection (tp, tp);
+        selection_rects= rectangles ();
+      }
+    }
   }
   
   // cout << "Handling environment\n";
   if (env_change & THE_ENVIRONMENT)
     typeset_invalidate_all ();
-  
+
   // cout << "Handling tree\n";
   if (env_change & (THE_TREE+THE_ENVIRONMENT)) {
     typeset_invalidate_env ();
-    if (input_mode == INPUT_NORMAL) {
-      made_selection= false;
-      set_selection (tp, tp);
-    }
-    selection_rects= rectangles ();
     SI x1, y1, x2, y2;
     typeset (x1, y1, x2, y2);
     invalidate (x1- 2*pixel, y1- 2*pixel, x2+ 2*pixel, y2+ 2*pixel);
@@ -439,7 +500,7 @@ edit_interface_rep::apply_changes () {
   // cout << "Cursor\n";
   temp_invalid_cursor= false;
   if (env_change & (THE_TREE+THE_ENVIRONMENT+THE_EXTENTS+
-                    THE_CURSOR+THE_FOCUS)) {
+                    THE_CURSOR+THE_SELECTION+THE_FOCUS)) {
     SI /*P1= pixel,*/ P2= 2*pixel, P3= 3*pixel;
     int THE_CURSOR_BAK= env_change & THE_CURSOR;
     go_to_here ();
@@ -448,7 +509,6 @@ edit_interface_rep::apply_changes () {
       if (!inside_active_graphics ())
         cursor_visible ();
     
-    
     cursor cu= get_cursor();
     rectangle ocr (oc->ox+ ((SI) (oc->y1*oc->slope))- P3, oc->oy+ oc->y1- P3,
                    oc->ox+ ((SI) (oc->y2*oc->slope))+ P2, oc->oy+ oc->y2+ P3);
@@ -463,38 +523,56 @@ edit_interface_rep::apply_changes () {
     // set hot spot in the gui
     send_cursor (this, (cu->ox-sfactor+1)/sfactor, (cu->oy-sfactor+1)/sfactor);
 
-    rectangles old_rects= env_rects;
+    path sp= selection_get_cursor_path ();
+    bool semantic_flag= semantic_active (path_up (sp));
+    bool full_context= (get_preference ("show full context") == "on");
+    bool table_cells= (get_preference ("show table cells") == "on");
+    bool show_focus= (get_preference ("show focus") == "on");
+    bool semantic_only= (get_preference ("show only semantic focus") == "on");
+    rectangles old_env_rects= env_rects;
+    rectangles old_foc_rects= foc_rects;
     env_rects= rectangles ();
+    foc_rects= rectangles ();
     path pp= path_up (tp);
     tree pt= subtree (et, pp);
-    if (is_func (pt, SPACE) || is_func (pt, IMAGE));
+    if (none_accessible (pt));
     else pp= path_up (pp);
-    compute_env_rects (pp, env_rects, true);
-    if (env_rects != old_rects) {
-      invalidate (old_rects);
+    if (full_context || table_cells)
+      compute_env_rects (pp, env_rects, true);
+    if (show_focus && (!semantic_flag || !semantic_only))
+      compute_env_rects (pp, foc_rects, false);
+    if (env_rects != old_env_rects) {
+      invalidate (old_env_rects);
       invalidate (env_rects);
     }
     else if (env_change & THE_FOCUS) invalidate (env_rects);
+    if (foc_rects != old_foc_rects) {
+      invalidate (old_foc_rects);
+      invalidate (foc_rects);
+    }
+    else if (env_change & THE_FOCUS) invalidate (foc_rects);
     
-    old_rects= sem_rects;
-    bool old_correct= sem_correct;
+    rectangles old_sem_rects= sem_rects;
+    bool old_sem_correct= sem_correct;
     sem_rects= rectangles ();
     sem_correct= true;
-    if (semantic_active (path_up (tp))) {
+    if (semantic_flag && show_focus) {
+      path sp= selection_get_cursor_path ();
       path p1= tp, p2= tp;
-      sem_correct= semantic_select (path_up (tp), p1, p2, 2);
+      if (selection_active_any ()) selection_get (p1, p2);
+      sem_correct= semantic_select (path_up (sp), p1, p2, 2);
       if (!sem_correct) {
-        path sr= semantic_root (path_up (tp));
+        path sr= semantic_root (path_up (sp));
         p1= start (et, sr);
         p2= end (et, sr);
       }
       path q1, q2;
-      selection_correct (et, p1, p2, q1, q2);
+      selection_correct (p1, p2, q1, q2);
       selection sel= eb->find_check_selection (q1, q2);
       sem_rects << outline (sel->rs, pixel);
     }
-    if (sem_rects != old_rects || sem_correct != old_correct) {
-      invalidate (old_rects);
+    if (sem_rects != old_sem_rects || sem_correct != old_sem_correct) {
+      invalidate (old_sem_rects);
       invalidate (sem_rects);
     }
     else if (env_change & THE_FOCUS) invalidate (sem_rects);
@@ -553,6 +631,7 @@ edit_interface_rep::apply_changes () {
   env_change  = 0;
   last_change = texmacs_time ();
   last_update = last_change-1;
+  manual_focus_release ();
 }
 
 /******************************************************************************
@@ -585,6 +664,7 @@ edit_interface_rep::full_screen_mode (bool flag) {
 
 void
 edit_interface_rep::before_menu_action () {
+  archive_state ();
   start_editing ();
   set_input_normal ();
 }
