@@ -10,6 +10,7 @@
 ******************************************************************************/
 
 #include "bibtex_functions.hpp"
+#include "vars.hpp"
 
 /******************************************************************************
 * Helper functions
@@ -231,6 +232,10 @@ bib_first_char (tree t) {
     if (beg < N(s)) return &(s[beg]);
     else return 0;
   }
+  else if (is_compound (t, "verbatim"))
+    return 0;
+  else if (is_func (t, WITH, 3) && t[0] == FONT_FAMILY && t[1] == "tt")
+    return 0;
   else {
     int pos= 0;
     if (L(t) == WITH) pos= N(t)-1;
@@ -284,6 +289,7 @@ bib_change_case (string s, string op) {
 void
 bib_change_case (tree& t, string (*change_case) (string)) {
   if (is_atomic (t) && change_case) t->label= change_case (t->label);
+  else if (is_compound (t, "verbatim"));
   else if (L(t) == WITH) bib_change_case (t[N(t)-1], change_case);
   else if (L(t) == as_tree_label ("keepcase")) t= t[0];
   else if (L(t) == CONCAT || L(t) == DOCUMENT)
@@ -563,6 +569,7 @@ get_first_von_last (string s) {
     return res;
   }
   if (a != "" && b != "" && c != "") return get_vl_j_f (a, b, c);
+  return "";
 }
 
 string
@@ -598,6 +605,7 @@ bib_purify_tree (tree t, string& res) {
   else if (L(t) == CONCAT || L(t) == DOCUMENT) {
     for (int i= 0; i<N(t); i++) bib_purify_tree (t[i], res);
   }
+  cout << UNINDENT;
 }
 
 string
@@ -656,6 +664,7 @@ bib_tree_length (tree t) {
     int s= 0;
     for (int i= 0; i<N(t); i++) s += bib_tree_length (t[i]);
   }
+  return 0;
 }
 
 int
