@@ -155,9 +155,9 @@ QTMWidget::QTMWidget (simple_widget_rep *_wid)
   : QTMScrollView (), backingPixmap(), imwidget(NULL) {
   setObjectName("A QTMWidget");
   setProperty ("texmacs_widget", QVariant::fromValue ((void*) _wid));
-  QAbstractScrollArea::viewport()->setMouseTracking (true);
+  surface()->setMouseTracking (true);
   setFocusPolicy (Qt::StrongFocus);
-  backing_pos = origin;
+  backing_pos = origin();
   setAttribute(Qt::WA_InputMethodEnabled);
 }
 
@@ -168,16 +168,22 @@ QTMWidget::~QTMWidget () {
 
 void 
 QTMWidget::invalidate_rect (int x1, int y1, int x2, int y2) {
-  // larger rectangle is needed to avoid artifacts during cursor movements
-  // see bug #28664
-  rectangle r = rectangle (x1-3, y1-1, x2+4, y2+1);
+#ifdef Q_WS_MAC
+  //HACK: for unknown reasons we need to enlarge the invalid rect to prevent
+  //artifacts while moving the cursor (for example at the end of a formula like
+  // $a+f$. These artifacts seems present only on 64 bit Macs. 
+  rectangle r = rectangle (x1-10, y1-10, x2+10, y2+10);
+#else
+  rectangle r = rectangle (x1, y1, x2, y2);
+#endif
   // cout << "invalidating " << r << LF;
   invalid_regions = invalid_regions | rectangles (r);
 }
 
 void 
 QTMWidget::invalidate_all () {
-  QSize sz = QAbstractScrollArea::viewport()->size();
+  QSize sz = surface()->size();
+ // QPoint pt = QAbstractScrollArea::viewport()->pos();
    //cout << "invalidate all " << LF;
   invalid_regions = rectangles();
   invalidate_rect (0, 0, sz.width(), sz.height());
@@ -230,11 +236,11 @@ QTMWidget::repaint_invalid_regions () {
   // qrgn is to keep track of the area on the sceen which needs to be updated 
 
   // update backing store origin wrt. TeXmacs document
-  if ( backing_pos != origin ) {
+  if ( backing_pos != origin() ) {
 
-    int dx =  origin.x() - backing_pos.x();
-    int dy =  origin.y() - backing_pos.y();
-    backing_pos = origin;
+    int dx =  origin().x() - backing_pos.x();
+    int dy =  origin().y() - backing_pos.y();
+    backing_pos = origin();
     
     QPixmap newBackingPixmap (backingPixmap.size());
     QPainter p (&newBackingPixmap);
@@ -279,19 +285,27 @@ QTMWidget::repaint_invalid_regions () {
     qrgn += QRect(QPoint(0,0),sz);
   }
   
+  
   // update backing store size
   {
     QSize _oldSize = backingPixmap.size();
-    QSize _newSize = QAbstractScrollArea::viewport()->size();
+    QSize _newSize = surface()->size();
     if (_newSize != _oldSize) {
       // cout << "RESIZING BITMAP"<< LF;
       QPixmap newBackingPixmap (_newSize);
-      //QPainter p (&newBackingPixmap);
-      //p.drawPixmap(0,0,backingPixmap);
-      //p.end();
+      QPainter p (&newBackingPixmap);
+      p.drawPixmap(0,0,backingPixmap);
+      //p.fillRect(0, 0, _newSize.width(), _newSize.height(), Qt::red);
+      if (_newSize.width() >= _oldSize.width()) {
+        invalidate_rect(_oldSize.width(), 0, _newSize.width(), _newSize.height());
+        p.fillRect(QRect(_oldSize.width(), 0, _newSize.width()-_oldSize.width(), _newSize.height()), Qt::gray);
+      }
+      if (_newSize.height() >= _oldSize.height()) {
+        invalidate_rect(0,_oldSize.height(), _newSize.width(), _newSize.height());
+        p.fillRect(QRect(0,_oldSize.height(), _newSize.width(), _newSize.height()-_oldSize.height()), Qt::gray);
+      }
+      p.end();
       backingPixmap = newBackingPixmap;
-      invalidate_all();
-      the_gui -> process_resize(tm_widget(), 0, 0); // FIXME
     }
   }
   
@@ -340,28 +354,42 @@ QTMWidget::repaint_invalid_regions () {
   }
 
   // propagate immediatly the changes to the screen  
-  QAbstractScrollArea::viewport()->repaint(qrgn);
+  surface()->repaint(qrgn);
   
 }
 
 void 
 QTMWidget::scrollContentsBy ( int dx, int dy ) {
   QTMScrollView::scrollContentsBy (dx,dy);
-  // the_gui::update needs to be run as soon as possible to refresh the status
-  // of the widget.
-  needs_update(); 
+
+  force_update();
+  // we force an update of the internal state to be in sync with the moving
+  // scrollbars
 }
 
 void 
 QTMWidget::resizeEvent( QResizeEvent* event ) {
+  (void) event;
+  
   // cout << "QTMWidget::resizeEvent (" << event->size().width()
   //      << "," << event->size().height() << ")" << LF;
-  QTMScrollView::resizeEvent (event);
-  // the_gui::update needs to be run as soon as possible to refresh the status
-  // of the widget.
-  needs_update(); 
+  
+  the_gui -> process_resize(tm_widget(), 0, 0); // FIXME
+
+  // force_update();
+
+  //FIXME: I would like to have a force_update here but this cause a failed
+  //assertion in TeXmacs since the at the boot not every internal structure is
+  //initialized at this point. It seems not too difficult to fix but I
+  //postpone this to discuss with Joris. 
+  //
+  //Not having a force_update results in some lack of sync of the surface
+  //while the user is actively resizing with the mouse.
 }
 
+
+
+
 void
 QTMWidget::paintEvent (QPaintEvent* event) {
   // In the current implementation repainting take place during the call to
@@ -380,13 +408,13 @@ QTMWidget::paintEvent (QPaintEvent* event) {
   }
     
   {    
-    QPainter p (QAbstractScrollArea::viewport());
+    QPainter p (surface());
     QVector<QRect> rects = event->region().rects();
     for (int i=0; i< rects.count(); i++) {
       QRect qr = rects.at(i);
       p.drawPixmap(qr,backingPixmap,qr);
     }
-  }
+  } 
   
 }
 
@@ -661,16 +689,46 @@ QTMWidget::inputMethodEvent (QInputMethodEvent* event) {
 #endif
   }
   
-  if (!preedit_string.isEmpty()) {
-    if (DEBUG_QT)
-      cout << "IM preediting :" << preedit_string.toUtf8().data() << LF;
-
+  if (DEBUG_QT)
+    cout << "IM preediting :" << preedit_string.toUtf8().data() << LF;
+  
+  string r = "pre-edit:";
+  if (!preedit_string.isEmpty())
+  {
     
-    string r = "pre-edit:" * from_qstring(preedit_string);
-    simple_widget_rep *wid =  tm_widget();
-    if (wid)
-      the_gui -> process_keypress (wid, r, texmacs_time());
+    // find cursor position in the preedit string
+    QList<QInputMethodEvent::Attribute>  const & attrs = event->attributes();
+    //    int pos = preedit_string.count();
+    int pos = 0;
+    bool visible_cur = false;
+    for(int i=0; i< attrs.count(); i++) 
+      if (attrs[i].type == QInputMethodEvent::Cursor) {
+        pos = attrs[i].start;
+        visible_cur = (attrs[i].length != 0);
+      }
+    
+    // find selection in the preedit string
+    int sel_start = 0;
+    int sel_length = 0;
+    if (pos <  preedit_string.count()) {
+      for(int i=0; i< attrs.count(); i++) 
+        if ((attrs[i].type == QInputMethodEvent::TextFormat) &&
+            (attrs[i].start <= pos) && (pos < attrs[i].start + attrs[i].length)) {
+          sel_start = attrs[i].start;
+          sel_length =  attrs[i].length;
+          if (!visible_cur) pos += attrs[i].length;
+        }
+    } else {
+      sel_start = pos;
+      sel_length = 0;
+    }
+    
+    r = r * as_string(pos) * ":" 
+    * from_qstring(preedit_string);
   }
+  simple_widget_rep *wid =  tm_widget();
+  if (wid)
+    the_gui -> process_keypress (wid, r, texmacs_time());
   event->accept();
 }  
 
@@ -678,9 +736,9 @@ QVariant
 QTMWidget::inputMethodQuery ( Qt::InputMethodQuery query ) const {
   switch (query) {
     case Qt::ImMicroFocus :
-      return QVariant(QRect(cursor_pos ,QSize(5,5)));
+      return QVariant (QRect (cursor_pos, QSize (5,5)));
     default:
-      return QVariant();
+      return QWidget::inputMethodQuery (query);
   }
 }
 
@@ -693,7 +751,7 @@ void
 QTMWidget::mousePressEvent (QMouseEvent* event) {
   simple_widget_rep *wid= tm_widget ();
   if (!wid) return;
-  QPoint point = event->pos() + origin;
+  QPoint point = event->pos() + origin();
   scale (point);
   unsigned int mstate= mouse_state (event, false);
   string s= "press-" * mouse_decode (mstate);
@@ -709,7 +767,7 @@ void
 QTMWidget::mouseReleaseEvent (QMouseEvent* event) {
   simple_widget_rep *wid = tm_widget();
   if (!wid) return;
-  QPoint point = event->pos() + origin;;
+  QPoint point = event->pos() + origin();
   scale (point);
   unsigned int mstate= mouse_state (event, true);
   string s= "release-" * mouse_decode (mstate);
@@ -725,7 +783,7 @@ void
 QTMWidget::mouseMoveEvent (QMouseEvent* event) {
   simple_widget_rep *wid = tm_widget();
   if (!wid) return;
-  QPoint point = event->pos() + origin;
+  QPoint point = event->pos() + origin();
   scale (point);
   unsigned int mstate= mouse_state (event, false);
   string s= "move";
@@ -744,7 +802,7 @@ QTMWidget::event (QEvent* event) {
     QKeyEvent *ke = static_cast<QKeyEvent*> (event);
     keyPressEvent (ke);
     return true;
-  }
+  } 
   return QTMScrollView::event (event);
 }
 
