@@ -49,13 +49,13 @@ public:
 	    tree style, hashmap<string,tree>& H, tree& drd, bool& flag) = 0;
 
   /* Control global server parameters */
-  virtual string get_preference (string var) = 0;
   virtual void   set_font_rules (scheme_tree rules) = 0;
   virtual bool   kbd_get_command (string s, string& help, command& cmd) = 0;
   virtual void   insert_kbd_wildcard (string key, string im,
 				      bool post, bool l, bool r) = 0;
   virtual string kbd_pre_rewrite (string l) = 0;
-  virtual string kbd_post_rewrite (string l) = 0;
+  virtual string kbd_post_rewrite (string l, bool var_flag= true) = 0;
+  virtual tree   kbd_system_rewrite (string l) = 0;
   virtual void   set_variant_keys (string var, string unvar) = 0;
   virtual void   get_keycomb (string& s, int& status,
 			      command& cmd, string& sh, string& help) = 0;
@@ -95,7 +95,7 @@ public:
   virtual bool visible_footer () = 0;
   virtual void set_left_footer (string s) = 0;
   virtual void set_right_footer (string s) = 0;
-  virtual void set_message (string left, string right, bool temp= false) = 0;
+  virtual void set_message (tree left, tree right, bool temp= false) = 0;
   virtual void recall_message () = 0;
   virtual void dialogue_start (string name, widget wid) = 0;
   virtual void dialogue_inquire (int i, string& arg) = 0;
@@ -159,6 +159,7 @@ public:
   virtual void pretend_save_buffer () = 0;
 
   /* Miscellaneous routines */
+  virtual void   refresh () = 0;
   virtual void   interpose_handler () = 0;
   virtual void   wait_handler (string message, string arg) = 0;
   virtual void   set_script_status (int i) = 0;
