@@ -11,7 +11,6 @@
 
 #include "Boxes/composite.hpp"
 #include "Boxes/construct.hpp"
-#include "timer.hpp"
 #include "file.hpp"
 
 /******************************************************************************
@@ -70,6 +69,7 @@ anim_constant_box_rep::anim_constant_box_rep (path ip, box b, int length2):
 
 void
 anim_constant_box_rep::pre_display (renderer& ren) {
+  (void) ren;
   if (!started) anim_start_at (texmacs_time ());
   else if (!finished) {
     finished= (texmacs_time () - (started_at+length) >= 0);
@@ -209,6 +209,7 @@ anim_compose_box_rep::find_tag (string name) {
 
 void
 anim_compose_box_rep::pre_display (renderer& ren) {
+  (void) ren;
   if (!started) anim_start_at (texmacs_time ());
   else if (!finished) {
     time_t now= texmacs_time ();
@@ -330,6 +331,7 @@ anim_repeat_box_rep::anim_repeat_box_rep (path ip, box b):
 
 void
 anim_repeat_box_rep::pre_display (renderer& ren) {
+  (void) ren;
   if (!started) anim_start_at (texmacs_time ());
   else if (length > 0) {
     time_t now= texmacs_time ();
@@ -508,6 +510,7 @@ struct sound_box_rep: public box_rep {
       system ("play", u, "&");
     started= true; }
   void pre_display (renderer& ren) {
+    (void) ren;
     if (!started) anim_start_at (texmacs_time ()); }
   int  anim_length () { return 0; }
   bool anim_started () { return started; }
@@ -546,7 +549,7 @@ add_frames (array<box>& a, path ip, url u, int w, int h, int msecs) {
     add_frames (a, ip, u[2], w, h, msecs);
   }
   else {
-    box imb= image_box (ip, u, w, h, 0.0, 0.0, 1.0, 1.0);
+    box imb= image_box (ip, u, w, h);
     a << anim_constant_box (ip, imb, msecs);
   }
 }
