@@ -21,15 +21,14 @@
 ******************************************************************************/
 
 box empty_box (path ip, int x1=0, int y1=0, int x2=0, int y2=0);
-box marker_box (path ip, int x1, int y1, int x2, int y2);
+box marker_box (path ip, int x1, int y1, int x2, int y2, box ref);
 box test_box (path ip);
 box line_box (path ip, SI x1, SI y1, SI x2, SI y2, SI w, color c);
 box arc_box (path ip, SI x1, SI y1, SI x2, SI y2,
 	     int a1, int a2, SI w, color c);
 box polygon_box (path ip, array<SI> x, array<SI> y, color c);
 box polygon_box (path ip, array<SI> x, array<SI> y, SI w, color cf, color cl);
-box image_box (path ip, url u, SI w, SI h,
-	       double cx1, double cy1, double cx2, double cy2);
+box image_box (path ip, url u, SI w, SI h);
 
 box text_box (path ip, int pos, string s, font fn, color col);
 box delimiter_box (path ip, string s, font fn, color col, SI y1, SI y2);
@@ -44,8 +43,6 @@ box wide_check_box (path ip, SI x1, SI x2, SI penw, color col);
 box wide_breve_box (path ip, SI x1, SI x2, SI penw, color col);
 box wide_squbr_box (path ip, SI x1, SI x2, SI penw, color col);
 box wide_sqobr_box (path ip, SI x1, SI x2, SI penw, color col);
-box image_box (path ip, string fname, SI w, SI h, double mw, double mh,
-	       int x1, int y1, int x2, int y2);
 box control_box (path ip, tree t, font fn);
 box control_box (path ip, lazy lz, font fn);
 
