@@ -17,11 +17,11 @@
 ******************************************************************************/
 
 void
-concater_rep::typeset_large (tree t, path ip, int type, string prefix) {
+concater_rep::typeset_large (tree t, path ip, int tp, int otp, string prefix) {
   if ((N(t) == 1) && is_atomic (t[0])) {
     string s= prefix * t[0]->label * ">";
     box b= text_box (ip, 0, s, env->fn, env->col);
-    print (type, b);
+    print (tp, otp, b);
     // temporarary: use parameters from group-open class in std-math.syx
     // bug: allow hyphenation after ) and before *
   }
@@ -30,7 +30,7 @@ concater_rep::typeset_large (tree t, path ip, int type, string prefix) {
     box b= text_box (ip, 0, s, env->fn, env->col);
     SI dy= env->fn->yfrac - ((b->y1 + b->y2) >> 1);
     box mvb= move_box (ip, b, 0, dy, false, true);
-    print (STD_ITEM, macro_box (ip, mvb, env->fn));
+    print (STD_ITEM, otp, macro_box (ip, mvb, env->fn));
   }
   else if ((N(t) >= 2) && is_atomic (t[0])) {
     SI y1, y2;
@@ -45,8 +45,9 @@ concater_rep::typeset_large (tree t, path ip, int type, string prefix) {
     }
     string s= prefix * t[0]->label * ">";
     box b= delimiter_box (ip, s, env->fn, env->col, y1, y2);
-    print (STD_ITEM, b);
+    print (STD_ITEM, otp, b);
   }
+  else typeset_error (t, ip);
 }
 
 void
@@ -59,12 +60,13 @@ concater_rep::typeset_bigop (tree t, path ip) {
     box b= big_operator_box (ip, s, env->fn, env->col,
 			     env->display_style? 2: 1);
     if (flag) print (space (spc->min>>1, spc->def>>1, spc->max));
-    print (STD_ITEM, b);
+    print (STD_ITEM, OP_BIG, b);
     penalty_min (HYPH_PANIC);
     if ((l != "int") && (l != "oint")) with_limits (LIMITS_DISPLAY);
     if (flag) print (spc);
     // temporarary: use parameters from operator-big class in std-math.syx
   }
+  else typeset_error (t, ip);
 }
 
 void
@@ -85,9 +87,10 @@ concater_rep::typeset_lprime (tree t, path ip) {
 		  flag? 0: env->as_length (string ("-0.05fn")),
 		  flag? env->as_length ("-0.75ex"): 0);
     if (!flag) env->local_end_script (old_il);
-    print (LSUP_ITEM, script_box (ip, b1, b2, env->fn));
+    print (LSUP_ITEM, OP_SKIP, script_box (ip, b1, b2, env->fn));
     penalty_max (HYPH_INVALID);
   }
+  else typeset_error (t, ip);
 }
 
 void
@@ -110,8 +113,9 @@ concater_rep::typeset_rprime (tree t, path ip) {
     if (!flag) env->local_end_script (old_il);
     penalty_max (HYPH_INVALID);
     if (N(a)>0) a[N(a)-1]->limits= false;
-    print (RSUP_ITEM, script_box (ip, b1, b2, env->fn));
+    print (RSUP_ITEM, OP_SKIP, script_box (ip, b1, b2, env->fn));
   }
+  else typeset_error (t, ip);
 }
 
 /******************************************************************************
@@ -120,6 +124,7 @@ concater_rep::typeset_rprime (tree t, path ip) {
 
 void
 concater_rep::typeset_below (tree t, path ip) {
+  if (N(t) != 2) { typeset_error (t, ip); return; }
   box b1= typeset_as_concat (env, t[0], descend (ip, 0));
   tree old_ds= env->local_begin (MATH_DISPLAY, "false");
   tree old_mc= env->local_begin (MATH_CONDENSED, "true");
@@ -128,11 +133,12 @@ concater_rep::typeset_below (tree t, path ip) {
   env->local_end_script (old_il);
   env->local_end (MATH_CONDENSED, old_mc);
   env->local_end (MATH_DISPLAY, old_ds);
-  print (STD_ITEM, limit_box (ip, b1, b2, box (), env->fn, false));
+  print (limit_box (ip, b1, b2, box (), env->fn, false));
 }
 
 void
 concater_rep::typeset_above (tree t, path ip) {
+  if (N(t) != 2) { typeset_error (t, ip); return; }
   box b1= typeset_as_concat (env, t[0], descend (ip, 0));
   tree old_ds= env->local_begin (MATH_DISPLAY, "false");
   tree old_mc= env->local_begin (MATH_CONDENSED, "true");
@@ -141,11 +147,12 @@ concater_rep::typeset_above (tree t, path ip) {
   env->local_end_script (old_il);
   env->local_end (MATH_CONDENSED, old_mc);
   env->local_end (MATH_DISPLAY, old_ds);
-  print (STD_ITEM, limit_box (ip, b1, box (), b2, env->fn, false));
+  print (limit_box (ip, b1, box (), b2, env->fn, false));
 }
 
 void
 concater_rep::typeset_script (tree t, path ip, bool right) {
+  if (N(t) != 1) { typeset_error (t, ip); return; }
   int type= RSUP_ITEM;
   box b1, b2;
   tree old_ds= env->local_begin (MATH_DISPLAY, "false");
@@ -167,7 +174,8 @@ concater_rep::typeset_script (tree t, path ip, bool right) {
   env->local_end (MATH_CONDENSED, old_mc);
   env->local_end (MATH_DISPLAY, old_ds);
   if (right) penalty_max (HYPH_INVALID);
-  a << line_item (type, script_box (ip, b1, b2, env->fn), HYPH_INVALID);
+  a << line_item (type, OP_SKIP,
+                  script_box (ip, b1, b2, env->fn), HYPH_INVALID);
   // do not use print, because of italic space
   if (!right) penalty_max (HYPH_INVALID);
 }
@@ -178,6 +186,7 @@ concater_rep::typeset_script (tree t, path ip, bool right) {
 
 void
 concater_rep::typeset_frac (tree t, path ip) {
+  if (N(t) != 2) { typeset_error (t, ip); return; }
   bool disp= env->display_style;
   tree old;
   if (disp) old= env->local_begin (MATH_DISPLAY, "false");
@@ -190,12 +199,12 @@ concater_rep::typeset_frac (tree t, path ip) {
   font sfn= env->fn;
   if (disp) env->local_end (MATH_DISPLAY, old);
   else env->local_end_script (old);
-  print (STD_ITEM, frac_box (ip, nom, den, env->fn, sfn, env->col));
+  print (frac_box (ip, nom, den, env->fn, sfn, env->col));
 }
 
 void
 concater_rep::typeset_sqrt (tree t, path ip) {
-  if (N(t)==0) return; // FIXME: this should never happen
+  if (N(t) != 1 && N(t) != 2) { typeset_error (t, ip); return; }
   box b= typeset_as_concat (env, t[0], descend (ip, 0));
   box ind;
   if (N(t)==2) {
@@ -209,12 +218,13 @@ concater_rep::typeset_sqrt (tree t, path ip) {
   }
   SI sep= env->fn->sep;
   box sqrtb= delimiter_box (decorate_left (ip), "<large-sqrt>",
-			    env->fn, env->col, b->y1, b->y2+ sep);
-  print (STD_ITEM, sqrt_box (ip, b, ind, sqrtb, env->fn, env->col));
+                            env->fn, env->col, b->y1, b->y2+ sep);
+  print (sqrt_box (ip, b, ind, sqrtb, env->fn, env->col));
 }
 
 void
 concater_rep::typeset_wide (tree t, path ip, bool above) {
+  if (N(t) != 2) { typeset_error (t, ip); return; }
   bool wide;
   box b= typeset_as_concat (env, t[0], descend (ip, 0));
   string s= as_string (t[1]);
@@ -230,7 +240,7 @@ concater_rep::typeset_wide (tree t, path ip, bool above) {
   else {
     wide= (b->w() >= (env->fn->wfn));
     if (ends (s, "dot>") || (s == "<acute>") ||
-	(s == "<grave>") || (s == "<abovering>"))
+        (s == "<grave>") || (s == "<abovering>"))
       wide= false;
   }
   if (wide) {
@@ -253,11 +263,11 @@ concater_rep::typeset_wide (tree t, path ip, bool above) {
     else if (s == "<sqoverbrace>" || s == "<sqoverbrace*>")
       wideb= wide_sqobr_box (decorate_middle (ip), b->x1, b->x2, w, env->col);
     else wideb= wide_box (decorate_middle (ip),
-			  "<rubber-" * s (1, N(s)-1) * ">",
-			  env->fn, env->col, b->x2- b->x1);
-    print (STD_ITEM, wide_box (ip, b, wideb, env->fn, env->fn->sep, above));
+                          "<rubber-" * s (1, N(s)-1) * ">",
+                          env->fn, env->col, b->x2- b->x1);
+    print (wide_box (ip, b, wideb, env->fn, env->fn->sep, above));
     if ((s == "<underbrace>") || (s == "<overbrace>") ||
-	(s == "<squnderbrace>") || (s == "<sqoverbrace>"))
+        (s == "<squnderbrace>") || (s == "<sqoverbrace>"))
       with_limits (LIMITS_ALWAYS);
   }
   else {
@@ -265,15 +275,16 @@ concater_rep::typeset_wide (tree t, path ip, bool above) {
     box wideb= text_box (decorate_middle (ip), 0, s, env->fn, env->col);
     if (env->fn->type == FONT_TYPE_UNICODE && b->right_slope () != 0)
       wideb= shift_box (decorate_middle (ip), wideb,
-			(SI) (-0.5 * b->right_slope () * env->fn->yx), 0);
-    print (STD_ITEM, wide_box (ip, b, wideb, env->fn, sep, above));
+                        (SI) (-0.5 * b->right_slope () * env->fn->yx), 0);
+    print (wide_box (ip, b, wideb, env->fn, sep, above));
   }
 }
 
 void
 concater_rep::typeset_neg (tree t, path ip) {
+  if (N(t) != 1) { typeset_error (t, ip); return; }
   box b= typeset_as_concat (env, t[0], descend (ip, 0));
-  print (STD_ITEM, neg_box (ip, b, env->fn, env->col));
+  print (neg_box (ip, b, env->fn, env->col));
 }
 
 /******************************************************************************
@@ -313,21 +324,32 @@ concater_rep::typeset_around (tree t, path ip, bool colored) {
     marker (descend (ip, 0));
     switch (L(t)) {
     case AROUND:
-      typeset (t[0], descend (ip, 0));
-      typeset (t[1], descend (ip, 1));
-      typeset (t[2], descend (ip, 2));
+      if (N(t) == 3) {
+        box br1= typeset_as_concat (env, t[0], descend (ip, 0));
+        print (STD_ITEM, OP_OPENING_BRACKET, br1);
+        typeset (t[1], descend (ip, 1));
+        box br2= typeset_as_concat (env, t[2], descend (ip, 2));
+        print (STD_ITEM, OP_CLOSING_BRACKET, br2);
+      }
+      else typeset_error (t, ip);
       break;
     case VAR_AROUND:
-      typeset (make_large (LEFT, t[0]),
-	       decorate_middle (descend (ip, 0)));
-      typeset (t[1], descend (ip, 1));
-      typeset (make_large (RIGHT, t[2]),
-	       decorate_middle (descend (ip, 2)));
+      if (N(t) == 3) {
+        typeset (make_large (LEFT, t[0]),
+                 decorate_middle (descend (ip, 0)));
+        typeset (t[1], descend (ip, 1));
+        typeset (make_large (RIGHT, t[2]),
+                 decorate_middle (descend (ip, 2)));
+      }
+      else typeset_error (t, ip);
       break;
     case BIG_AROUND:
-      typeset (make_large (BIG, t[0]),
-	       decorate_middle (descend (ip, 0)));
-      typeset (t[1], descend (ip, 1));
+      if (N(t) == 2) {
+        typeset (make_large (BIG, t[0]),
+                 decorate_middle (descend (ip, 0)));
+        typeset (t[1], descend (ip, 1));
+      }
+      else typeset_error (t, ip);
       break;
     default:
       break;
@@ -338,14 +360,15 @@ concater_rep::typeset_around (tree t, path ip, bool colored) {
 
 void
 concater_rep::typeset_tree (tree t, path ip) {
+  if (N(t) == 0) { typeset_error (t, ip); return; }
   int i, n= N(t);
   array<box> bs(n);
   for (i=0; i<n; i++) bs[i]= typeset_as_concat (env, t[i], descend (ip, i));
-  print (STD_ITEM, tree_box (ip, bs, env->fn, env->col));
+  print (tree_box (ip, bs, env->fn, env->col));
 }
 
 void
 concater_rep::typeset_table (tree t, path ip) {
   box b= typeset_as_table (env, t, ip);
-  print (STD_ITEM, b);
+  print (b);
 }
