@@ -288,6 +288,8 @@ edit_env_rep::update_mode () {
   else if (s == "math") mode=2;
   else if (s == "prog") mode=3;
   else mode=0;
+  if (mode == 2) mode_op= OP_SYMBOL;
+  else mode_op= OP_TEXT;
 }
 
 void
@@ -304,6 +306,7 @@ edit_env_rep::update_language () {
     lan= prog_language (get_string (PROG_LANGUAGE));
     break;
   }
+  hl_lan= lan->hl_lan;
 }
 
 void
@@ -399,7 +402,7 @@ edit_env_rep::update_dash_style () {
   dash_style= array<bool>(0);
   if (is_string (t)) {
     string s= as_string (t);
-    if (s == "none") ;
+    if (s == "none") {}
   }
   else
   if (is_tuple (t)) {
@@ -428,7 +431,7 @@ edit_env_rep::update_line_arrows () {
   line_arrows= array<box>(2);
   if (is_string (t)) {
     string s= as_string (t);
-    if (s == "none") ;
+    if (s == "none") {}
   }
   else
   if (is_tuple (t) && N(t)<=2 && N(t)>0) {
