@@ -5,10 +5,9 @@
 *              hyphenation and creation of page items
 * COPYRIGHT  : (C) 1999  Joris van der Hoeven
 *******************************************************************************
-* This software falls under the GNU general public license and comes WITHOUT
-* ANY WARRANTY WHATSOEVER. See the file $TEXMACS_PATH/LICENSE for more details.
-* If you don't have this file, write to the Free Software Foundation, Inc.,
-* 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+* This software falls under the GNU general public license version 3 or later.
+* It comes WITHOUT ANY WARRANTY WHATSOEVER. For details, see the file LICENSE
+* in the root directory or <http://www.gnu.org/licenses/gpl-3.0.html>.
 ******************************************************************************/
 
 #include "Line/lazy_paragraph.hpp"
@@ -29,7 +28,7 @@ line_breaks (array<line_item> a, int start, int end,
 
 lazy_paragraph_rep::lazy_paragraph_rep (edit_env env2, path ip):
   lazy_rep (LAZY_PARAGRAPH, ip),
-  env (env2), style (""), sss (new stacker_rep)
+  env (env2), style (""), sss (tm_new<stacker_rep> ())
 {
   sss->ip= ip; // is this necessary?
   style (PAR_FIRST)   = env->read (PAR_FIRST);
@@ -60,7 +59,7 @@ lazy_paragraph_rep::lazy_paragraph_rep (edit_env env2, path ip):
 }
 
 lazy_paragraph_rep::~lazy_paragraph_rep () {
-  delete sss;
+  tm_delete (sss);
 }
 
 lazy_paragraph_rep::operator tree () {
@@ -116,13 +115,13 @@ lazy_paragraph_rep::line_print (line_item item) {
 
 void
 lazy_paragraph_rep::line_print (line_item item, path left, path right) {
-  if (nil (left) && nil (right)) line_print (item);
+  if (is_nil (left) && is_nil (right)) line_print (item);
   else {
     line_item item1, item2;
-    hyphenate (item, nil (left)? right->item: left->item, item1, item2);
-    line_print (nil (left) ? item1: item2,
-		nil (left) ? left : left->next,
-		nil (right)? right: right->next);
+    hyphenate (item, is_nil (left)? right->item: left->item, item1, item2);
+    line_print (is_nil (left) ? item1: item2,
+		is_nil (left) ? left : left->next,
+		is_nil (right)? right: right->next);
   }
 }
 
@@ -135,7 +134,7 @@ lazy_paragraph_rep::line_print (path start, path end) {
     line_print (a[start->item], start->next, path ());
     for (i=start->item+1; i<end->item; i++)
       line_print (a[i]);
-    if (!atom (end))
+    if (!is_atom (end))
       line_print (a[end->item], path (), end->next);
   }
 }
@@ -411,7 +410,7 @@ void
 lazy_paragraph_rep::format_paragraph () {
   width -= right;
 
-  int start= 0, i, j;
+  int start= 0, i, j, k;
   // cout << "Typeset " << a << "\n";
   for (i=0; i<=N(a); i++) {
     // determine the next unit
@@ -427,8 +426,14 @@ lazy_paragraph_rep::format_paragraph () {
     if (no_first) style (PAR_FIRST)= "0cm";
     for (j=start; j<i; j++)
       if (a[j]->type == CONTROL_ITEM)
-	if (is_tuple (a[j]->t, "env_par"))
+	if (is_tuple (a[j]->t, "env_par")) {
+	  if (a[j]->t[1]->label == PAR_FIRST) {
+	    for (k=j-1; k>=start; k--)
+	      if (a[k]->b->w () != 0) break;
+	    if (k >= start) continue;
+	  }
 	  style (a[j]->t[1]->label)= a[j]->t[2];
+	}
     no_first= (style [PAR_NO_FIRST] == "true");
     if (no_first) env->monitored_write_update (PAR_NO_FIRST, "true");
     if (mode == "center") first= 0;
@@ -465,12 +470,12 @@ convert (edit_env env, array<box> bs, path ip) {
     if (i==0) {
       box b= empty_box (decorate (ip), 0, 0, 0, env->fn->yx);
       tree ct= tuple ("env_par", PAR_FIRST, "0cm");
-      a << line_item (CONTROL_ITEM, b, 0, ct);
+      a << line_item (CONTROL_ITEM, OP_SKIP, b, 0, ct);
     }
-    a << line_item (STD_ITEM, bs[i], 0);
+    a << line_item (STD_ITEM, env->mode_op, bs[i], 0);
     if (i<(n-1)) {
       box b= empty_box (decorate (ip), 0, 0, 0, env->fn->yx);
-      a << line_item (CONTROL_ITEM, b, 0, NEXT_LINE);
+      a << line_item (CONTROL_ITEM, OP_SKIP, b, 0, NEXT_LINE);
     }
   }
   return a;
