@@ -137,6 +137,7 @@ public:
   double       magn;
   double       flexibility;
   int          mode;
+  int          mode_op;
   language     lan;
   int          hl_lan;
   font         fn;
@@ -403,6 +404,7 @@ public:
 
   tree      as_tmlen (tree t);
   SI        as_length (tree t);
+  SI        as_length (tree t, string perc);
   space     as_hspace (tree t);
   space     as_vspace (tree t);
   point     as_point (tree t);
